/* calib.c                                   */
/* Interferometry Data Reduction Software    */
/* by Stefan Kraus                           */
/* 2002-05                                   */
/*                                           */
/* copyright laws apply                      */

#define calibavg(ia,ja)           pcalibavg[ia*(max_nights)+ja]
#define calibavgerror(ia,ja)      pcalibavgerror[ia*(max_nights)+ja]
#define calibset_vis(ia,ja)       pcalibset_vis[ia*(max_calibsets)+ja]
#define calibset_nr(ia,ja)        pcalibset_nr[ia*(max_calibsets)+ja]
#define calibset_visscat(ia,ja)   pcalibset_visscat[ia*(max_calibsets)+ja]

// Matrix1 = obb-Matrix (o = shutter open, b = shutter blocked)
// Matrix2 = bob-Matrix
// Matrix3 = bbo-Matrix
// Matrix4 = NULL-Matrix

#include "general.c"

//*** calculate the calibrator-levelsamp"
//calibset gives the level for each group of star
//calibavg gives the level for a whole night

int calc_calibrator_levels(char *filename_in, char *filename_out, int only_calib, int *caliblist, int *srelist, float *pcalibavg, float *pcalibavgerror, float *pcalibset_vis, float *pcalibset_visscat, float *calibset_cp, float *calibset_cpscat, float *calibset_jd, int *calibset_objid, int *calibset_night, float *pcalibset_nr, int *calibset_from, int *calibset_to, float *nr_calibs)
{
  int i, n, nr_night, nr_calibset;
  int scriptnrall, oldscriptnrall;
  int nr_var[3], objid, cpcount;
  int is_calib, is_target, over1vis[3];
  float temp, temp2;
  float oldjd, newjd;
  float calc_var[3][max_vispoints];
  int calibtag;
  float uavg[3], vavg[3], sreamp[3], sreamperror[3], sreampprob[3], mean, variance, biascorr[3], corrfactor[3], last_sreamp[3];
  float cpsum, cpsumsqr;
  FILE *streamamp, *stream2;




  //Determine calibrator-level for each night
  for (i=0; i<=max_nights; i++)
    for (n=0; n<3; n++)
      calibavg(n,i) = 0.0;
  
  for (n=0; n<3; n++)
    {
      nr_calibs[n] = 0.0;
      nr_var[n] = 0;
      last_sreamp[3] = -1.0;
    }
  nr_calibset = 0;
  


  for (i=0; i<max_calibsets; i++)
    {
      for (n=0; n<3; n++)
	{
	  calibset_nr(n, i) = 0.0;
	  calibset_vis(n, i) = 0.0;
	  calibset_visscat(n, i) = 0.0;
	  calibset_cp[i] = 0.0;
	  calibset_cpscat[i] = 0.0;
	  calibset_from[i] = -1;
	  calibset_to[i] = -1;
	}
      
      calibset_night[i] = -1;
    }

  streamamp = fopen(filename_in, "r");

  over1vis[0] = 0; over1vis[1] = 0; over1vis[2] = 0;
  nr_night = 0;
  oldjd = 0.0;
  cpcount = 0;
  while (feof(streamamp) == 0)
    {
      fscanf(streamamp, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
      fscanf(streamamp, streamampstring2, &biascorr[0],  &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);


      //extract and group the calibrators
      if (((calibset_objid[nr_calibset] != objid) || (abs2(newjd - oldjd) > 0.3)) && (calibset_night[nr_calibset] != -1) && ((calibset_nr(0, nr_calibset) > 0.0) || (calibset_nr(1, nr_calibset) > 0.0) || (calibset_nr(2, nr_calibset) > 0.0)))
	{
	  for (i=0; i<3; i++)
	    {
	      if (calibset_nr(i, nr_calibset) == 0.0)
		{
		  calibset_vis(i, nr_calibset) = -1.0;
		  calibset_visscat(i, nr_calibset) = -1.0;
		}
	      else
		{
		  temp2 = calibset_nr(i, nr_calibset);
		  
		  if (only_calib == 2)
		    calibset_visscat(i, nr_calibset) /= temp2;
		  else
		  {
		      temp = calibset_visscat(i, nr_calibset);
		      calibset_visscat(i, nr_calibset) = (temp-calibset_vis(i, nr_calibset)*calibset_vis(i, nr_calibset)/temp2)/temp2;
		  }

		  calibset_vis(i, nr_calibset) /= temp2;

		  if (calibset_vis(i, nr_calibset) > filterampthreshold)
		      calibset_vis(i, nr_calibset) = -2.0;
		  
		  if (over1vis[i] > filterover1vis)
		    calibset_visscat(i, nr_calibset) = 1.0e10;
		}
	    }

	  calibset_to[nr_calibset] = scriptnrall;
	  calibset_cp[nr_calibset] = cpsum / cpcount;
	  calibset_cpscat[nr_calibset] = sqrt((cpsumsqr-cpsum*cpsum/cpcount)/cpcount);
	  cpsum = 0.0;
	  cpsumsqr =  0.0;
	  cpcount = 0;
 
	  nr_calibset++;
	  over1vis[0] = 0; over1vis[1] = 0; over1vis[2] = 0;
	  for (i=0; i<3; i++)
	    {
	      last_sreamp[i] = -1.0;
	      calibset_nr(i, nr_calibset) = 0.0;
	    }
	}
      
      if (abs2(newjd - oldjd) > 0.3)
	{
	  //normalize average of last night
	  for (i=0; i<3; i++)
	    if (nr_calibs[i] > 0)
	      {
		temp = calibavg(i, nr_night);
		temp /= nr_calibs[i];
		calibavg(i, nr_night) = temp;
	      }
	  
	  for (i=0; i<3; i++)
	    {
	      calibavgerror(i, nr_night) = 0.0;
	      for (n=0; n<nr_var[i]; n++)
		{
		  temp = calibavgerror(i, nr_night);
		  temp += (calibavg(i, nr_night) - calc_var[i][n])*(calibavg(i, nr_night) - calc_var[i][n]);
		  calibavgerror(i, nr_night) = temp;
		}
	      
	      temp = calibavgerror(i, nr_night);
	      temp = sqrt(temp/nr_var[i]);
	      calibavgerror(i, nr_night) = sqrt(temp/sqrt((float)nr_var[i]-1));


	      if (calibavgerror(i, nr_night) > calibavg(i, nr_night))
	      {
		  calibavg(i, nr_night) = -1.0;
		  calibavgerror(i, nr_night) = 0.0;

		  printf("CALIBRATOR REJECTED: Channel %i, Night %i: Error larger than value itself!\n\n", i, nr_night);
	      }

	      nr_var[i] = -1;
	    }
	  
	  if ((nr_night != 0) && (Viscalibmethod == 0))
	    printf("Night #%i (%f): Calib-Amp: %f +/- %f; %f +/- %f; %f +/- %f (#Calibs: %f %f %f)\n", nr_night, newjd, calibavg(0, nr_night), calibavgerror(0, nr_night), calibavg(1, nr_night), calibavgerror(1, nr_night), calibavg(2, nr_night), calibavgerror(2, nr_night), nr_calibs[0], nr_calibs[1], nr_calibs[2]);
	  
	  nr_night++;
	  for (n=0; n<3; n++)
	    nr_calibs[n] = 0.0;
	}
      oldjd = newjd;
      
      is_calib = 0;
      is_target = 0;
      
      for (i=0; i<max_targets; i++)
	{
	  if (objid == caliblist[i]) is_calib = 1;
	  if (objid == srelist[i]) is_calib = 1;

	  //printf("CALIB: %i\n", caliblist[i]);

	}

      if ((is_calib == 1) || ((only_calib > 0) && (is_target == 1))) 
	{
	    if ((mean >= -PI) && ((variance < filtercperror) || (dofilterscat == 0)))
	    {
		cpsum += mean;
		cpsumsqr += (mean*mean);
		//(calibset_cp[nr_calibset]) += mean;
		cpcount++;
	    }

	  if (calibset_from[nr_calibset] == -1) calibset_from[nr_calibset] = scriptnrall;

	  for (i=0; i<3; i++)
	    {
	      if (sreamp[i] > 0.0)
		{
		  temp = calibavg(i, nr_night);
		  temp += sreamp[i]*1.0/(sqrt(abs2(sreamperror[i])));
		  calibavg(i, nr_night) = temp;
		  
		  nr_calibs[i] += 1.0/(sqrt(abs2(sreamperror[i])));
		  
		  nr_var[i]++;
		  calc_var[i][nr_var[i]] = sreamp[i];
		  
		  if (sreamp[i] > 1.0)
		    over1vis[i]++;
		  else
		    {
		      if (only_calib == 2)
			{
			  if (last_sreamp[i] != -1.0) (calibset_visscat(i, nr_calibset)) += (sreamp[i] - last_sreamp[i])*(sreamp[i] - last_sreamp[i]);
			  last_sreamp[i] = sreamp[i];
			}
		      else
			(calibset_visscat(i, nr_calibset)) += sreamp[i]*sreamp[i];
		    }
		  
		  (calibset_vis(i, nr_calibset)) += sreamp[i];
		  calibset_night[nr_calibset] = nr_night;
		  calibset_objid[nr_calibset] = objid;
		  calibset_jd[nr_calibset] = newjd;
		  
		  (calibset_nr(i, nr_calibset)) += 1.0;
		}
	    }
	  }
    }
  fclose(streamamp);
  
  
  //normalize last average
  for (i=0; i<3; i++)
    if (nr_calibs[i] > 0)
      calibavg(i, nr_night) /= nr_calibs[i];
  
  if ((calibset_nr(0, nr_calibset-1) > 0.0) || (calibset_nr(1, nr_calibset-1) > 0.0) || (calibset_nr(2, nr_calibset-1) > 0.0))
    {
      calibset_to[nr_calibset] = scriptnrall;
      calibset_cp[nr_calibset] = cpsum / cpcount;
      calibset_cpscat[nr_calibset] = sqrt((cpsumsqr-cpsum*cpsum/cpcount)/cpcount);
      cpsum = 0.0;
      cpsumsqr =  0.0;
      cpcount = 0;

      for (i=0; i<3; i++)
	{
	  if (calibset_nr(i, nr_calibset) == 0.0)
	    {
	      calibset_visscat(i, nr_calibset) = -1.0;
	      calibset_vis(i, nr_calibset) = -1.0;
	    }
	  else
	    {
	      temp2 = calibset_nr(i, nr_calibset);
	      
	      if (only_calib == 2)
		calibset_visscat(i, nr_calibset) /= temp2;
	      else
	      {
		  temp = calibset_visscat(i, nr_calibset);
		  calibset_visscat(i, nr_calibset) = (temp-calibset_vis(i, nr_calibset)*calibset_vis(i, nr_calibset)/temp2)/temp2;
	      }
	      calibset_vis(i, nr_calibset) /= temp2;
	      
	      if (calibset_vis(i, nr_calibset) > filterampthreshold)
		  calibset_vis(i, nr_calibset) = -1.0;

	      if (over1vis[i] > filterover1vis)
		calibset_visscat(i, nr_calibset) = 1.0e10;
	    }
	}

      for (i=0; i<3; i++)
	{
	      calibavgerror(i, nr_night) = 0.0;
	      for (n=0; n<nr_var[i]; n++)
	      {
		  temp = calibavgerror(i, nr_night);
		  temp += (calibavg(i, nr_night) - calc_var[i][n])*(calibavg(i, nr_night) - calc_var[i][n]);
		  calibavgerror(i, nr_night) = temp;
	      }
	      
	      temp = calibavgerror(i, nr_night);
	      temp = sqrt(temp/nr_var[i]);
	      calibavgerror(i, nr_night) = sqrt(temp/sqrt((float)nr_var[i]-1));

/*
	  calibavgerror(i, nr_night) = 0.0;
	  for (n=0; n<nr_var[i]; n++)
	      (calibavgerror(i, nr_night)) += (calibavg(i, nr_night) - calc_var[i][n])*(calibavg(i, nr_night) - calc_var[i][n]);

	  calibavgerror(i, nr_night) = sqrt((calibavgerror(i, nr_night))/nr_var[i]);
*/
	  if (calibavgerror(i, nr_night) > calibavg(i, nr_night))
	  {
	      calibavg(i, nr_night) = -1.0;
	      calibavgerror(i, nr_night) = 0.0;
	      
	      printf("CALIBRATOR REJECTED: Channel %i, Night %i: Error larger than value itself!\n\n", i, nr_night);
	  }
	}
      
      if ((nr_night != 0) && (Viscalibmethod == 0))
	printf("Night #%i (%f): Calib-Amp: %f +/- %f; %f +/- %f; %f +/- %f (#Calibs: %f %f %f)\n", nr_night, newjd, calibavg(0, nr_night), calibavgerror(0, nr_night), calibavg(1, nr_night), calibavgerror(1, nr_night), calibavg(2, nr_night), calibavgerror(2, nr_night), nr_calibs[0], nr_calibs[1], nr_calibs[2]);
      nr_calibset++;
      over1vis[0] = 0; over1vis[1] = 0; over1vis[2] = 0;
    }

  
  //*** Save calibrator levels for plotting
  stream2 = fopen(filename_out, "w");
  streamamp = fopen(filename_in, "r");
  nr_night = 0;
  oldjd = 0.0;
  oldscriptnrall = 0;
  while (feof(streamamp) == 0)
    {
      fscanf(streamamp, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
      fscanf(streamamp, streamampstring2, &biascorr[0], &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);
      
      if (abs2(newjd - oldjd) > 0.3) nr_night++;
      oldjd = newjd;
      
      for (n = oldscriptnrall; n <= scriptnrall; n++)
	fprintf(stream2, "%i %f %f %f %f\n", n, newjd, calibavg(0, nr_night), calibavg(1, nr_night), calibavg(2, nr_night));
      oldscriptnrall = scriptnrall;
    }
  fclose(streamamp);
  fclose(stream2);

  for (i=0; i<nr_calibset; i++)
    {
      if (only_calib <2)
	printf("Calibator #%2i (ID %2i) at JD %8.4f: %10.4f %10.4f %10.4f\n", i, calibset_objid[i], calibset_jd[i], calibset_vis(0, i), calibset_vis(1, i), calibset_vis(2, i));

      if (only_calib == 2)
	printf("Group #%3i (ID %2i, JD %6.2f) scatters with %8f %8f %8f from %i to %i (No. Points: %4.0f  %4.0f  %4.0f)\n", i, calibset_objid[i], calibset_jd[i], calibset_visscat(0, i), calibset_visscat(1, i), calibset_visscat(2, i), calibset_from[i], calibset_to[i], calibset_nr(0, i), calibset_nr(1, i), calibset_nr(2, i));
    }

  
  //FOR CAPELLA NOV02:
  //calibavg(0,5) = 138.5;
  //calibavg(2,5) = 256.8;

  return nr_calibset;
}


//averages Phases in the Re/Im-plane to take the [-PI,PI]-Periodicity into account

void average_cp(double *data, int n, float *mean, float *variance)
{
    double x, y, data2, sum, sumsqr;
    int i;

    if (n == 0)
    {
	*mean = -4.0;
	*variance = 7.0;
    }
    else
    {
	x = 0.0;
	y = 0.0;
	
	for (i=1; i<=n; i++)
	{
	    x += cos(data[i]);
	    y += sin(data[i]);
	}
	
	*mean = atan2(y, x);
	
	while (*mean < -PI) (*mean) += 2.0*PI;
	while (*mean >= PI) (*mean) -= 2.0*PI;
	
	
  /*
	*variance = 0.0;

	for (i=1; i<=n; i++)
	{
	    data2 = data[i]-(*mean);
	    while (data2 < -PI) data2 += 2.0*PI;
	    while (data2 >= PI) data2 -= 2.0*PI;
	    
	    (*variance) += data2*data2;
	}
	(*variance) = sqrt(*variance/(float)n);
*/	

	sum = 0.0;
	sumsqr = 0.0;
	for (i=1; i<=n; i++)
	{
	    data2 = data[i]-(*mean);
	    while (data2 > PI) data2 -= 2.0*PI;
	    while (data2 < -PI) data2 += 2.0*PI;
	    while (data2 > PI) data2 -= 2.0*PI;
	    sum += data2;

	    sumsqr += data2*data2;
	}

	*variance = sqrt(sumsqr/((float)n));
	//if ((sumsqr/((float)n-1.0) > 0.0) && ((float)n > 0.0))
	//    (*variance) = sqrt(sumsqr/((float)n-1.0))/(2.0*sqrt((float)n));
	//else
	//    (*variance) = 0.0;
    }
}

//generates matrix file, averaged over of all scans
    
double generate_matrixfiles(char *filename, int *ch_baseline, int N_RUNS, int scriptnrall, int calibfile)
{
  int i, j, i_run, count[6], nullpix, mincounts;
  double **deriv, **pixel, avg[6], scattering[6], mean[6], maxscattering;
  double singlescan[6];
  char  scriptnrtext[10], calibfiletext[10], filename2[100], filename3[100], filename4[100];
  FILE *stream, *stream2, *stream3, *stream4;
  
  sprintf(scriptnrtext, "%i", scriptnrall);
  sprintf(calibfiletext, "%i", calibfile);
  strcpy(filename2, ProcDir);
  strcat(filename2, "matrix");
  //strcpy(filename2, "matrixfiles/matrix");
  strcat(filename2, scriptnrtext);
  strcat(filename2, ".m");
  strcat(filename2, calibfiletext);

  strcpy(filename3, ProcDir);
  strcat(filename3, "matrixscat");
  //strcpy(filename3, "matrixfiles/matrixscat");
  strcat(filename3, scriptnrtext);
  strcat(filename3, ".m");
  strcat(filename3, calibfiletext);

  strcpy(filename4, ProcDir);
  strcat(filename4, "matrixavg");
  //strcpy(filename4, "matrixfiles/matrixavg");
  strcat(filename4, scriptnrtext);
  strcat(filename4, ".m");
  strcat(filename4, calibfiletext);


  //deriv = dmatrix(0, 5, 0, N_RUNS-1);
  deriv = dmatrix(0, 5, 0, nPtsScan-1);
  pixel = dmatrix(0, 5, 0, nPtsScan-1);

  if (ch_baseline[0] == 1)
    {
      //*** South Delayed Case ABC
      if (calibfile == 1) nullpix = 0; //black
      if (calibfile == 2) nullpix = 2; //blue
      if (calibfile == 3) nullpix = 4; //red
    }
  
  
  if (ch_baseline[0] == 0)
    {
      //*** North Delayed Case BAC
      if (calibfile == 1) nullpix = 2; //blue
      if (calibfile == 2) nullpix = 0; //black
      if (calibfile == 3) nullpix = 4; //red
    }
  
  
  for (i=0; i<nPtsScan; i++)
    for (j=0; j<6; j++)
      deriv[j][i] = 0.0;
  
  for (j=0; j<6; j++)
    {
      mean[j] = 0.0;
      scattering[j] = 0.0;
    }
  
  for (j=0; j<6; j++)
    count[j] = 0;
  
  stream2 = fopen(filename, "r");
  stream3 = fopen(filename3, "w");
  
  for (i_run = 0; i_run < N_RUNS; i_run++)
  {
/* 
   if (calibfile == 4)
      {
	  for (i=0; i<nPtsScan; i++)
	  {
	      read_singlescan(stream2, singlescan);
	      for (j=0; j<6; j++)
	      {
		  pixel[j][i] = singlescan[j];
		  //printf("%lf\n", singlescan[j]);
	      }
	  }
      }
      else
*/
	  read_scan_deriv(stream2, pixel[0], pixel[1], pixel[2], pixel[3], pixel[4], pixel[5]);
      
      for (j=0; j<6; j++)
	avg[j] = 0.0;
      
      for (i=0; i<nPtsScan; i++)
	for (j=0; j<6; j++)
	  avg[j] += pixel[j][i];
      
      for (j=0; j<6; j++)
	{
	  if (calibfile == 4)
	    {
	      for (i=0; i<nPtsScan; i++)
		deriv[j][i] += pixel[j][i];
	      
	      count[j]++;
	    }
	  else
	    if ((j == nullpix) || (j == nullpix+1) || (abs2(avg[j]) > abs2((avg[nullpix]+avg[nullpix+1])/2.0)*matrix_threshold))
	      {
		for (i=0; i<nPtsScan; i++)
		  deriv[j][i] += pixel[j][i];
		
		count[j]++;
	      }
	}
      
      fprintf(stream3, "%i %f %f %f %f %f %f\n", i_run, avg[0], avg[1], avg[2], avg[3], avg[4], avg[5]);
  }
  
  fclose(stream2);
  fclose(stream3);
  
  printf("   Number of used scans: %i %i %i %i %i %i (File %s)\n", count[0], count[1], count[2], count[3], count[4], count[5], filename2);
  //printf("Mean: %f %f %f %f %f %f\n", mean[0], mean[1], mean[2], mean[3], mean[4], mean[5]);
  

  //*** Calculate the mean
  stream = fopen(filename2, "w");

  for (i=0; i<nPtsScan; i++)
    {
      for (j=0; j<6; j++)
	{
	  if (count[j] > 0) deriv[j][i] /= count[j];

	  mean[j] += deriv[j][i];
	}
      
      fprintf(stream, "%i %f %f %f %f %f %f\n", i, deriv[0][i], deriv[1][i], deriv[2][i], deriv[3][i], deriv[4][i], deriv[5][i]);
    }

  for (j=0; j<6; j++)
    {
      avg[j] /= nPtsScan;
      mean[j] /= nPtsScan;
    }

  //*** Write avg in file
  stream4 = fopen(filename4, "w");
  
  fprintf(stream3, "%f %f %f %f %f %f\n", avg[0], avg[1], avg[2], avg[3], avg[4], avg[5]);

  fclose(stream4);


  mincounts = 1E5;
  for (j=0; j<6; j++)
    if (count[j] < mincounts)
      mincounts = count[j];

  fprintf(stream, "%i\n", mincounts);

  fclose(stream);


  //*** Calculate the scattering within the file
  /*
  stream3 = fopen(filename3, "r");

  for (i_run = 0; i_run < N_RUNS; i_run++)
    {
      fscanf(stream3, "%i %f %f %f %f %f %f\n", &i, &avg[0], &avg[1], &avg[2], &avg[3], &avg[4], &avg[5]);

      for (j=0; j<6; j++)
	scattering[j] += (avg[j] - mean[j])*(avg[j] - mean[j]);
    }
  
  fclose(stream3);


  for (j=0; j<6; j++)
    {
      scattering[j] /= N_RUNS;
      
      if (mean[j] != 0.0)
	scattering[j] /= sqrt(mean[j]*mean[j]);
    }

  maxscattering = -1.0e10;
  for (j=0; j<6; j++)
    if (scattering[j] > maxscattering) maxscattering = scattering[j];

  fprintf(stream, "%f %f %f %f %f %f  %f\n", scattering[0], scattering[1], scattering[2], scattering[3], scattering[4], scattering[5], maxscattering);
  printf("    Scattering: %f %f %f %f %f %f\n", scattering[0], scattering[1], scattering[2], scattering[3], scattering[4], scattering[5]);

  fclose(stream);
   */

  //Calculating the median
  /*
  for (i=0; i<nPtsScan; i++)
    {
      stream2 = fopen(filename, "r");
  
      for (i_run = 0; i_run < N_RUNS; i_run++)
	{
	  read_scan_deriv(stream2, pixel[0], pixel[1], pixel[2], pixel[3], pixel[4], pixel[5]);
	  
          for (j=0; j<6; j++)
            deriv[j][i_run] = pixel[j][i];
	}
      
      fclose(stream2);

      fprintf(stream, "%i %f %f %f %f %f %f\n", i, median(N_RUNS-1, deriv[0]), median(N_RUNS-1, deriv[1]), median(N_RUNS-1, deriv[2]), median(N_RUNS-1, deriv[3]), median(N_RUNS-1, deriv[4]), median(N_RUNS-1, deriv[5]));
    }
  */

  free_dmatrix(deriv, 0, 5, 0, N_RUNS-1);
  free_dmatrix(pixel, 0, 5, 0, nPtsScan-1);

  return maxscattering;
}



int generate_photfiles(int scriptnrall, int *ch_baseline, FILE *streamrepcorr, FILE *streamlog, int objid, fftw_plan fftw_f, fftw_plan fftw_b)
{
  int i, j, j2, calibfile, matrixcount, count[4][6];
  char  scriptnrtext[10], calibfiletext[10], filename[100];
  FILE *stream;
  float value[6], value_, calibtag; 
  double **corr, **matrix1, **matrix2, **matrix3, **matrix4;
  double matrixavg[6];
  double matrix1level[6], matrix2level[6], matrix3level[6], matrix4level[6];
  //float scattering[6], maxscattering;
  int nr_points;
  double corravg[3], corrmin[3], corrmax[3], A1, A2, B1, B2;
  float c;
  double **matrix4_s;

  double sumsig1[6], sumsig2[6];


  corr = dmatrix(0, 5, 0, nPtsScan-1);
  matrix1 = dmatrix(0, 5, 0, nPtsScan-1);
  matrix2 = dmatrix(0, 5, 0, nPtsScan-1);
  matrix3 = dmatrix(0, 5, 0, nPtsScan-1);
  matrix4 = dmatrix(0, 5, 0, nPtsScan-1);
  matrix4_s = dmatrix(0, 5, 0, nPtsScan-1);

  matrixcount = 0;

  for (calibfile=1; calibfile<=4; calibfile++)
  {
      i = scriptnrall - 4 + calibfile;

      sprintf(scriptnrtext, "%i", i);
      sprintf(calibfiletext, "%i", calibfile);
      strcpy(filename, ProcDir);
      strcat(filename, "matrix");
      //strcpy(filename, "matrixfiles/matrix");
      strcat(filename, scriptnrtext);
      strcat(filename, ".m");
      strcat(filename, calibfiletext);

      for (j=0; j<6; j++)
	matrixavg[j] = 0;

      if (file_exist(filename))
      {
	  printf("%s\n", filename);
	  stream = fopen(filename, "r");

	  for (i=0; i<nPtsScan; i++)
	    {
	      fscanf(stream, "%i %f %f %f %f %f %f\n", &j2, &value[0], &value[1], &value[2], &value[3], &value[4], &value[5]);
	      
	      for (j=0; j<6; j++)
		{
		  if (calibfile == 1) matrix1[j][i] = value[j];
		  if (calibfile == 2) matrix2[j][i] = value[j];
		  if (calibfile == 3) matrix3[j][i] = value[j];
		  if (calibfile == 4) matrix4[j][i] = value[j];
		}
	    }
	  
	  fscanf(stream, "%i\n", &nr_points);

	  fclose(stream);

/*

	  stream = fopen(filename, "r");
	  
	  //if (repdetails == 1) 
	  printf("Read matrixfile: %s\n", filename);
	  
	  fscanf(stream, "%f %f %f %f %f %f\n", &value[0], &value[1], &value[2], &value[3], &value[4], &value[5]);
	      
	  for (j=0; j<6; j++)
	  {
	      if (calibfile == 1) matrix1level[j] = value[j];
	      if (calibfile == 2) matrix2level[j] = value[j];
	      if (calibfile == 3) matrix3level[j] = value[j];
	      if (calibfile == 4) matrix4level[j] = value[j];
	  }
	  
	  fclose(stream);
*/
	  matrixcount++;
      }
      else
      {
	  printf("ERROR: Missing matrixfile %s\n", filename);
	  
	  for (j=0; j<6; j++)
	  {
	      if (calibfile == 1) matrix1[j][0] = -1.0E10;
	      if (calibfile == 2) matrix2[j][0] = -1.0E10;
	      if (calibfile == 3) matrix3[j][0] = -1.0E10;
	      if (calibfile == 4) matrix4[j][0] = -1.0E10;
	  }
      }
  }
/*
  for (j=0; j<6; j++)
  {
      sumsig1[j] = 0.0;
      for (i=0; i<nPtsScan; i++)
	  sumsig1[j] += matrix4[j][i];
  }
*/

  for (j=0; j<6; j++)
      smooth_signal(matrix4[j], nPtsScan, bgsubsmoothfactor, fftw_f, fftw_b, matrix4_s[j]);

/*
  for (j=0; j<6; j++)
  {
      sumsig2[j] = 0.0;
      for (i=0; i<nPtsScan; i++)
	  sumsig2[j] += matrix4_s[j][i];

      printf("*** SUM %i: %lf  -  %lf\n", j, sumsig1[j], sumsig2[j]);
  }
*/
  for (i=0; i<nPtsScan; i++)
  {
      //printf("%10.6lf %10.6lf %10.6lf %10.6lf\n", matrix1[0][i], matrix2[0][i], matrix3[0][i], matrix4_s[0][i]);
      
      for (j=0; j<6; j++)
      {
	  matrix1[j][i] -= matrix4_s[j][i];
	  matrix2[j][i] -= matrix4_s[j][i];
	  matrix3[j][i] -= matrix4_s[j][i];
	  //matrix1level[j] -= matrix4level[j];
	  //matrix2level[j] -= matrix4level[j];
	  //matrix3level[j] -= matrix4level[j];
      }

      //printf("%10.6lf %10.6lf %10.6lf\n", matrix1[0][i], matrix2[0][i], matrix3[0][i]);
  }



  for (j=0; j<6; j++)
  {
      matrix1level[j] = 0.0;
      matrix2level[j] = 0.0;
      matrix3level[j] = 0.0;
      
      for (i=0; i<nPtsScan; i++)
      {
	  matrix1level[j] += matrix1[j][i];
	  matrix2level[j] += matrix2[j][i];
	  matrix3level[j] += matrix3[j][i];
      }

      matrix1level[j] /= (double)nPtsScan;
      matrix2level[j] /= (double)nPtsScan;
      matrix3level[j] /= (double)nPtsScan;
  }
	  

  
/*
  matrix1level[0] = matrix4level[0];
  matrix1level[1] = matrix4level[1];
  matrix2level[2] = matrix4level[2];
  matrix2level[3] = matrix4level[3];
  matrix3level[4] = matrix4level[4];
  matrix3level[5] = matrix4level[5];
*/
  if (matrixcount < 4)
  {
      fprintf(streamlog, "Correction %s: Not enough matrix files found\n", filename);
      printf("     Not enough matrix files found (star couldn't be locked?!)\n");
  }
  else
  {
      sprintf(scriptnrtext, "%i", scriptnrall);
      strcpy(filename, ProcDir);
      strcat(filename, "phot");
      //strcpy(filename, "matrixfiles/phot");
      strcat(filename, scriptnrtext);
      strcat(filename, ".dat");
      
      
      stream = fopen(filename, "w");

      fprintf(stream, "%f %f  %f %f  %f %f\n", matrix1level[0], matrix1level[1], matrix1level[2], matrix1level[3], matrix1level[4], matrix1level[5]);
      fprintf(stream, "%f %f  %f %f  %f %f\n", matrix2level[0], matrix2level[1], matrix2level[2], matrix2level[3], matrix2level[4], matrix2level[5]);
      fprintf(stream, "%f %f  %f %f  %f %f\n", matrix3level[0], matrix3level[1], matrix3level[2], matrix3level[3], matrix3level[4], matrix3level[5]);

  
      printf("   Matrix1: (%f %f)   %f  %f    %f  %f \n", matrix1level[0], matrix1level[1], matrix1level[2], matrix1level[3], matrix1level[4], matrix1level[5]);
      printf("   Matrix2:   %f  %f  (%f %f)   %f  %f \n", matrix2level[0], matrix2level[1], matrix2level[2], matrix2level[3], matrix2level[4], matrix2level[5]);
      printf("   Matrix3:   %f  %f    %f  %f  (%f %f)\n", matrix3level[0], matrix3level[1], matrix3level[2], matrix3level[3], matrix3level[4], matrix3level[5]);

      fclose(stream);
  }
    

  free_dmatrix(corr, 0, 5, 0, nPtsScan-1);
  free_dmatrix(matrix1, 0, 5, 0, nPtsScan-1);
  free_dmatrix(matrix2, 0, 5, 0, nPtsScan-1);
  free_dmatrix(matrix3, 0, 5, 0, nPtsScan-1);
  free_dmatrix(matrix4, 0, 5, 0, nPtsScan-1);
  free_dmatrix(matrix4_s, 0, 5, 0, nPtsScan-1);

  return matrixcount;  
}




int generate_photfiles_avg(int scriptnrall, int *ch_baseline, FILE *streamrepcorr, FILE *streamlog, int objid)
{
  int i, j, j2, calibfile, matrixcount, count[4][6];
  char  scriptnrtext[10], calibfiletext[10], filename[100];
  FILE *stream;
  float value[6], value_, calibtag; 
  double **corr, **matrix1, **matrix2, **matrix3, **matrix4;
  double matrixavg[6];
  double matrix1level[6], matrix2level[6], matrix3level[6], matrix4level[6];
  //float scattering[6], maxscattering;
  int nr_points;
  double corravg[3], corrmin[3], corrmax[3], A1, A2, B1, B2;
  float c;


  corr = dmatrix(0, 6, 0, nPtsScan-1);
  matrix1 = dmatrix(0, 6, 0, nPtsScan-1);
  matrix2 = dmatrix(0, 6, 0, nPtsScan-1);
  matrix3 = dmatrix(0, 6, 0, nPtsScan-1);
  matrix4 = dmatrix(0, 6, 0, nPtsScan-1);

  matrixcount = 0;

  for (calibfile=1; calibfile<=4; calibfile++)
  {
      i = scriptnrall - 4 + calibfile;
      
      sprintf(scriptnrtext, "%i", i);
      sprintf(calibfiletext, "%i", calibfile);
      strcpy(filename, ProcDir);
      strcat(filename, "matrixavg");
      //strcpy(filename, "matrixfiles/matrixavg");
      strcat(filename, scriptnrtext);
      strcat(filename, ".m");
      strcat(filename, calibfiletext);

      for (j=0; j<6; j++)
	matrixavg[j] = 0;

      if (file_exist(filename))
      {
	  stream = fopen(filename, "r");
	  
	  //if (repdetails == 1) 
	  printf("Read matrixfile: %s\n", filename);
	  
	  fscanf(stream, "%f %f %f %f %f %f\n", &value[0], &value[1], &value[2], &value[3], &value[4], &value[5]);
	      
	  for (j=0; j<6; j++)
	  {
	      if (calibfile == 1) matrix1level[j] = value[j];
	      if (calibfile == 2) matrix2level[j] = value[j];
	      if (calibfile == 3) matrix3level[j] = value[j];
	      if (calibfile == 4) matrix4level[j] = value[j];
	  }
	  
	  fclose(stream);

	  matrixcount++;
      }
      else
      {
	  printf("ERROR: Missing matrixfile %s\n", filename);
	  
	  for (j=0; j<6; j++)
	  {
	      if (calibfile == 1) matrix1[j][0] = -1.0E10;
	      if (calibfile == 2) matrix2[j][0] = -1.0E10;
	      if (calibfile == 3) matrix3[j][0] = -1.0E10;
	      if (calibfile == 4) matrix4[j][0] = -1.0E10;
	  }
      }
  }


  for (j=0; j<6; j++)
  {
      matrix1level[j] -= matrix4level[j];
      matrix2level[j] -= matrix4level[j];
      matrix3level[j] -= matrix4level[j];
  }

/*
  matrix1level[0] = matrix4level[0];
  matrix1level[1] = matrix4level[1];
  matrix2level[2] = matrix4level[2];
  matrix2level[3] = matrix4level[3];
  matrix3level[4] = matrix4level[4];
  matrix3level[5] = matrix4level[5];
*/
  if (matrixcount < 4)
  {
      fprintf(streamlog, "Correction %s: Not enough matrix files found\n", filename);
      printf("     Not enough matrix files found (star couldn't be locked?!)\n");
  }
  else
  {
      sprintf(scriptnrtext, "%i", scriptnrall);
      strcpy(filename, ProcDir);
      strcat(filename, "phot");
      //strcpy(filename, "matrixfiles/phot");
      strcat(filename, scriptnrtext);
      strcat(filename, ".dat");
      
      
      stream = fopen(filename, "w");

      fprintf(stream, "%f %f  %f %f  %f %f\n", matrix1level[0], matrix1level[1], matrix1level[2], matrix1level[3], matrix1level[4], matrix1level[5]);
      fprintf(stream, "%f %f  %f %f  %f %f\n", matrix2level[0], matrix2level[1], matrix2level[2], matrix2level[3], matrix2level[4], matrix2level[5]);
      fprintf(stream, "%f %f  %f %f  %f %f\n", matrix3level[0], matrix3level[1], matrix3level[2], matrix3level[3], matrix3level[4], matrix3level[5]);

  
      printf("   Matrix1: (%f %f)   %f  %f    %f  %f \n", matrix1level[0], matrix1level[1], matrix1level[2], matrix1level[3], matrix1level[4], matrix1level[5]);
      printf("   Matrix2:   %f  %f  (%f %f)   %f  %f \n", matrix2level[0], matrix2level[1], matrix2level[2], matrix2level[3], matrix2level[4], matrix2level[5]);
      printf("   Matrix3:   %f  %f    %f  %f  (%f %f)\n", matrix3level[0], matrix3level[1], matrix3level[2], matrix3level[3], matrix3level[4], matrix3level[5]);

      fclose(stream);
  }
    

  free_dmatrix(corr, 0, 6, 0, nPtsScan-1);
  free_dmatrix(matrix1, 0, 6, 0, nPtsScan-1);
  free_dmatrix(matrix2, 0, 6, 0, nPtsScan-1);
  free_dmatrix(matrix3, 0, 6, 0, nPtsScan-1);
  free_dmatrix(matrix4, 0, 6, 0, nPtsScan-1);

  return matrixcount;  
}




int generate_photfiles_old(int scriptnrall, int *ch_baseline, FILE *streamrepcorr, FILE *streamlog, int objid)
{
  int i, j, j2, calibfile, matrixcount, count[4][6];
  char  scriptnrtext[10], calibfiletext[10], filename[100];
  FILE *stream;
  float value[6], value_, calibtag; 
  double **corr, **matrix1, **matrix2, **matrix3, **matrix4;
  double matrixavg[6];
  double matrix1level[6], matrix2level[6], matrix3level[6];
  //float scattering[6], maxscattering;
  int nr_points;
  double corravg[3], corrmin[3], corrmax[3], A1, A2, B1, B2;
  float c;


  corr = dmatrix(0, 6, 0, nPtsScan-1);
  matrix1 = dmatrix(0, 6, 0, nPtsScan-1);
  matrix2 = dmatrix(0, 6, 0, nPtsScan-1);
  matrix3 = dmatrix(0, 6, 0, nPtsScan-1);
  matrix4 = dmatrix(0, 6, 0, nPtsScan-1);

  matrixcount = 0;

  for (calibfile=1; calibfile<=4; calibfile++)
    {
      i = scriptnrall - 4 + calibfile;
      
      sprintf(scriptnrtext, "%i", i);
      sprintf(calibfiletext, "%i", calibfile);
      strcpy(filename, ProcDir);
      strcat(filename, "matrix");
      //strcpy(filename, "matrixfiles/matrix");
      strcat(filename, scriptnrtext);
      strcat(filename, ".m");
      strcat(filename, calibfiletext);

      for (j=0; j<6; j++)
	matrixavg[j] = 0;

      if (file_exist(filename))
	{
	  stream = fopen(filename, "r");
	  
	  //if (repdetails == 1) 
	  printf("Read matrixfile: %s\n", filename);
	  
	  for (i=0; i<nPtsScan; i++)
	    {
	      fscanf(stream, "%i %f %f %f %f %f %f\n", &j2, &value[0], &value[1], &value[2], &value[3], &value[4], &value[5]);
	      
	      for (j=0; j<6; j++)
		{
		  if (calibfile == 1) matrix1[j][i] = value[j];
		  if (calibfile == 2) matrix2[j][i] = value[j];
		  if (calibfile == 3) matrix3[j][i] = value[j];
		  if (calibfile == 4) matrix4[j][i] = value[j];

		  matrixavg[j] += value[j];
		  //printf("Matrix: %i -> %f %f %f %f (%s)\n", calibfile, matrix1[j][i], matrix2[j][i], matrix3[j][i], matrix4[j][i], filename);
		}
	    }
	  
	  fscanf(stream, "%i\n", &nr_points);

	  fclose(stream);

	  //if (maxscattering > threshold_scattering)
	  if ((nr_points < 5) && (calibfile < 4))
	  {
	      //fprintf(streamlog, "Correction %s: Matrixfile was rejected because scattering is above threshold (%f > %f)!!!\n", filename, maxscattering, threshold_scattering);
	      //printf("     Matrixfile was rejected because scattering is above threshold (%f > %f)!!!\n", maxscattering, threshold_scattering);

	      fprintf(streamlog, "Correction %s: Matrixfile was rejected because two less scans contain star light (#scans = %i)!!!\n", filename, nr_points);
	      printf("     Matrixfile %s was rejected because two less scans contain star light (#scans = %i)!!!\n", filename, nr_points);

	      for (j=0; j<6; j++)
	      {
		  if (calibfile == 1) matrix1[j][0] = -1.0E10;
		  if (calibfile == 2) matrix2[j][0] = -1.0E10;
		  if (calibfile == 3) matrix3[j][0] = -1.0E10;
		  if (calibfile == 4) matrix4[j][0] = -1.0E10;
	      }
	  }
	  else
	    {
	      /*
	      matrixavg_min = 1e10;
	      for (j=0; j<6; j++)
		if (matrixavg[j] < matrixavg_min) matrixavg_min = matrixavg[j];
	      
	      matrixavg_max = 1e10;
	      for (j=0; j<6; j++)
		if (matrixavg[j] < matrixavg_max) matrixavg_max = matrixavg[j];
	      
	      if (matrixavg_max >= 3.0*matrixavg_min)
	      */
	      
	      matrixcount++;
	    }
	}
      else
	{
	  printf("ERROR: Missing matrixfile %s\n", filename);

	  for (j=0; j<6; j++)
	    {
	      if (calibfile == 1) matrix1[j][0] = -1.0E10;
	      if (calibfile == 2) matrix2[j][0] = -1.0E10;
	      if (calibfile == 3) matrix3[j][0] = -1.0E10;
	      if (calibfile == 4) matrix4[j][0] = -1.0E10;
	    }
	}
    }

  //calculating the correction factor

  if (matrixcount < 4)
  {
      fprintf(streamlog, "Correction %s: Not enough matrix files found\n", filename);
      printf("     Not enough matrix files found (star couldn't be locked?!)\n");
  }
  else
  {
      sprintf(scriptnrtext, "%i", scriptnrall);
      strcpy(filename, ProcDir);
      strcat(filename, "phot");
      //strcpy(filename, "matrixfiles/phot");
      strcat(filename, scriptnrtext);
      strcat(filename, ".dat");
      
      
      stream = fopen(filename, "w");
      
      
      if (ch_baseline[0] == 0)
	printf("North Delayed Case\n");
      else
	printf("South Delayed Case\n");
      
      
      if ((matrix1[0][0] != -1.0E10) && (matrix2[0][0] != -1.0E10) && (matrix3[0][0] != -1.0E10) && (matrix4[0][0] != -1.0E10))
	{
	  printf("CREATE CORRECTION FILE:\n");
	  
	  for (j=0; j<3; j++)
	    {
	      corrmin[j] = 1.0E10;
	      corrmax[j] = -1.0E10;
	      corravg[j] = 0.0;
	    }
	  
	  for (j=0; j<6; j++)
	    {
	      matrix1level[j] = 0.0;
	      matrix2level[j] = 0.0;
	      matrix3level[j] = 0.0;
	      
	      count[1][j] = 0;
	      count[2][j] = 0;
	      count[3][j] = 0;
	    }

	  for (i=0; i<nPtsScan; i++)
	  {
	      for (j=0; j<3; j++)
	      {
		  if (ch_baseline[0] == 1)
		  {
		      //*** South Delayed Case ABC
		      switch (j)
			{
			case 0:
			    //A1 = -(matrix3[0][i]-matrix1[0][i]);
			    //B1 = -(matrix2[0][i]-matrix1[0][i]);
			    //A2 = -(matrix3[1][i]-matrix1[1][i]);
			    //B2 = -(matrix2[1][i]-matrix1[1][i]);
			    A1 = matrix3[0][i];
			    B1 = matrix2[0][i];
			    A2 = matrix3[1][i];
			    B2 = matrix2[1][i];
			    matrix1level[0] += matrix1[0][i];//-matrix4[0][i];
			    matrix1level[1] += matrix1[1][i];//-matrix4[1][i];
			    matrix3level[0] += A1;
			    matrix2level[0] += B1;
			    matrix3level[1] += A2;
			    matrix2level[1] += B2;
			    break;
			  
			case 1:
			    //A1 = -(matrix3[2][i]-matrix2[2][i]);
			    //B1 = -(matrix1[2][i]-matrix2[2][i]);
			    //A2 = -(matrix3[3][i]-matrix2[3][i]);
			    //B2 = -(matrix1[3][i]-matrix2[3][i]);
			    A1 = matrix3[2][i];
			    B1 = matrix1[2][i];
			    A2 = matrix3[3][i];
			    B2 = matrix1[3][i];
			    matrix2level[2] += matrix2[2][i];//-matrix4[2][i];
			    matrix2level[3] += matrix2[3][i];//-matrix4[3][i];
			    matrix3level[2] += A1;
			    matrix1level[2] += B1;
			    matrix3level[3] += A2;
			    matrix1level[3] += B2;
			    break;
			  
			case 2:
			    //A1 = -(matrix1[4][i]-matrix3[4][i]);
			    //B1 = -(matrix2[4][i]-matrix3[4][i]);
			    //A2 = -(matrix1[5][i]-matrix3[5][i]);
			    //B2 = -(matrix2[5][i]-matrix3[5][i]);
			    A1 = matrix1[4][i];
			    B1 = matrix2[4][i];
			    A2 = matrix1[5][i];
			    B2 = matrix2[5][i];
			    matrix3level[4] += matrix3[4][i];//-matrix4[4][i];
			    matrix3level[5] += matrix3[5][i];//-matrix4[5][i];
			    matrix1level[4] += A1;
			    matrix2level[4] += B1;
			    matrix1level[5] += A2;
			    matrix2level[5] += B2;
			    break;
			}
		    }
		  
		  if (ch_baseline[0] == 0)
		    {
		      //*** North Delayed Case BAC
		      switch (j)
			{
			case 0:
			    //A1 = -(matrix3[0][i]-matrix2[0][i]);
			    //B1 = -(matrix1[0][i]-matrix2[0][i]);
			    //A2 = -(matrix3[1][i]-matrix2[1][i]);
			    //B2 = -(matrix1[1][i]-matrix2[1][i]);
			    A1 = matrix3[0][i];
			    B1 = matrix1[0][i];
			    A2 = matrix3[1][i];
			    B2 = matrix1[1][i];
			    matrix2level[2] += matrix2[0][i];//-matrix4[0][i]; //2
			    matrix2level[3] += matrix2[1][i];//-matrix4[1][i]; //2
			    matrix3level[0] += A1;
			    matrix2level[0] += B1; //2
			    matrix3level[1] += A2;
			    matrix2level[1] += B2; //2
			    break;
			  
			case 1:
			    //A1 = -(matrix3[2][i]-matrix1[2][i]);
			    //B1 = -(matrix2[2][i]-matrix1[2][i]);
			    //A2 = -(matrix3[3][i]-matrix1[3][i]);
			    //B2 = -(matrix2[3][i]-matrix1[3][i]);
			    A1 = matrix3[2][i];
			    B1 = matrix2[2][i];
			    A2 = matrix3[3][i];
			    B2 = matrix2[3][i];
			    matrix1level[0] += matrix1[2][i];//-matrix4[2][i]; //1
			    matrix1level[1] += matrix1[3][i];//-matrix4[3][i]; //1
			    matrix3level[2] += A1;
			    matrix1level[2] += B1; //1
			    matrix3level[3] += A2;
			    matrix1level[3] += B2; //1
			    break;
			  
			case 2:
			    //A1 = -(matrix2[4][i]-matrix3[4][i]);
			    //B1 = -(matrix1[4][i]-matrix3[4][i]);
			    //A2 = -(matrix2[5][i]-matrix3[5][i]);
			    //B2 = -(matrix1[5][i]-matrix3[5][i]);
			    A1 = matrix2[4][i];
			    B1 = matrix1[4][i];
			    A2 = matrix2[5][i];
			    B2 = matrix1[5][i];
			    matrix3level[4] += matrix3[4][i];//-matrix4[4][i];
			    matrix3level[5] += matrix3[5][i];//-matrix4[5][i];
			    matrix1level[4] += A1; //1
			    matrix2level[4] += B1; //2
			    matrix1level[5] += A2; //1
			    matrix2level[5] += B2; //2
			    break;
			}
		    }
		}
	    }
	  
	  
	  for (j=0; j<6; j++)
	  {
	      matrix1level[j] /= nPtsScan;
	      matrix2level[j] /= nPtsScan;
	      matrix3level[j] /= nPtsScan;
	  }

	  fprintf(stream, "%f %f  %f %f  %f %f\n", matrix1level[0], matrix1level[1], matrix1level[2], matrix1level[3], matrix1level[4], matrix1level[5]);
	  fprintf(stream, "%f %f  %f %f  %f %f\n", matrix2level[0], matrix2level[1], matrix2level[2], matrix2level[3], matrix2level[4], matrix2level[5]);
	  fprintf(stream, "%f %f  %f %f  %f %f\n", matrix3level[0], matrix3level[1], matrix3level[2], matrix3level[3], matrix3level[4], matrix3level[5]);

/*
	  for (j=0; j<3; j++)
	    {
	      calc_correction(j, matrix1level, matrix2level, matrix3level, &c, &calibtag);

	      corravg[j] = c;
	    }

	  printf("CORRFACTOR: %f %f %f\n", corravg[0],  corravg[1],  corravg[2]);
      
	  fprintf(streamrepcorr, "%i %f %f %f %f\n", scriptnrall, 1.0-objid/50.0, corravg[0], corravg[1], corravg[2]);
	  	      
	  for (i=0; i<nPtsScan; i++)
	    {
	      for (j=0; j<3; j++)
		if (corr[j][i] == -1.0) corr[j][i] = corravg[j];
	      
	      value_ = 0;
	      if (i == 0) value_ = corravg[0];
	      if (i == 1) value_ = corravg[1];
	      if (i == 2) value_ = corravg[2];
	      if (i == 3) value_ = corrmin[0];
	      if (i == 4) value_ = corrmin[1];
	      if (i == 5) value_ = corrmin[2];
	      if (i == 6) value_ = corrmax[0];
	      if (i == 7) value_ = corrmax[1];
	      if (i == 8) value_ = corrmax[2];
	      if (i == 9) value_ = 0;//corrcount[0];
	      if (i == 10) value_ = 0;//corrcount[1];
	      if (i == 11) value_ = 0;//corrcount[2];
	      if (i == 12) value_ = 0;//corrcountrej[0];
	      if (i == 13) value_ = 0;//corrcountrej[1];
	      if (i == 14) value_ = 0;//corrcountrej[2];
	      if (i == 15) value_ = matrix1level[0];
	      if (i == 16) value_ = matrix1level[1];
	      if (i == 17) value_ = matrix1level[2];
	      if (i == 18) value_ = matrix1level[3];
	      if (i == 19) value_ = matrix1level[4];
	      if (i == 20) value_ = matrix1level[5];
	      if (i == 21) value_ = matrix2level[0];
	      if (i == 22) value_ = matrix2level[1];
	      if (i == 23) value_ = matrix2level[2];
	      if (i == 24) value_ = matrix2level[3];
	      if (i == 25) value_ = matrix2level[4];
	      if (i == 26) value_ = matrix2level[5];
	      if (i == 27) value_ = matrix3level[0];
	      if (i == 28) value_ = matrix3level[1];
	      if (i == 29) value_ = matrix3level[2];
	      if (i == 30) value_ = matrix3level[3];
	      if (i == 31) value_ = matrix3level[4];
	      if (i == 32) value_ = matrix3level[5];
	      
	      fprintf(stream, "%i %f %f %f %f\n", i, corr[0][i], corr[1][i], corr[2][i], value_);
	    }
*/
	}
      else
	{
	  for (j=0; j<6; j++)
	    {
	      matrix1level[j] = 0.0;
	      matrix2level[j] = 0.0;
	      matrix3level[j] = 0.0;
	    }
	}
  
      fclose(stream);
  
      printf("   Matrix1: (%f %f)   %f  %f    %f  %f \n", matrix1level[0], matrix1level[1], matrix1level[2], matrix1level[3], matrix1level[4], matrix1level[5]);
      printf("   Matrix2:   %f  %f  (%f %f)   %f  %f \n", matrix2level[0], matrix2level[1], matrix2level[2], matrix2level[3], matrix2level[4], matrix2level[5]);
      printf("   Matrix3:   %f  %f    %f  %f  (%f %f)\n", matrix3level[0], matrix3level[1], matrix3level[2], matrix3level[3], matrix3level[4], matrix3level[5]);
    }
  

  free_dmatrix(corr, 0, 6, 0, nPtsScan-1);
  free_dmatrix(matrix1, 0, 6, 0, nPtsScan-1);
  free_dmatrix(matrix2, 0, 6, 0, nPtsScan-1);
  free_dmatrix(matrix3, 0, 6, 0, nPtsScan-1);
  free_dmatrix(matrix4, 0, 6, 0, nPtsScan-1);

  return matrixcount;  
}
  
/*
int generate_kappamatrix(int scriptnrall, int *ch_baseline, FILE *streammatrix, FILE *streamlog, int objid)
{
    double **kappa, **invkappa;
    double matrixavg[6];
    int calibfile;
    
    
    kappa = dmatrix(0, 5, 0, 5);
    invkappa = dmatrix(0, 5, 0, 5);

    for (calibfile=1; calibfile<4; calibfile++)
    {
	i = scriptnrall - 4 + calibfile;
      
	sprintf(scriptnrtext, "%i", i);
	sprintf(calibfiletext, "%i", calibfile);
	strcpy(filename, "matrixavg");
	//strcpy(filename, "matrixfiles/matrixavg");
	strcat(filename, scriptnrtext);
	strcat(filename, ".m");
	strcat(filename, calibfiletext);

	for (j=0; j<6; j++)
	    matrixavg[j] = 0;

	if (file_exist(filename))
	{
	    stream = fopen(filename, "r");
	  
	    //if (repdetails == 1) 
	    printf("Read matrixfile: %s\n", filename);
	  
	    fscanf(stream, "%f %f %f %f %f %f\n", &matrixavg[0], &matrixavg[1], &matrixavg[2], &matrixavg[3], &matrixavg[4], &matrixavg[5]);
	      
	    fclose(stream);


	    if (calibfile == 1)
	    {
		kappa[0][0] = matrixavg[2];
		kappa[0][1] = matrixavg[3];
		kappa[1][0] = matrixavg[2];
		kappa[1][1] = matrixavg[3];
	    }

	    if (calibfile == 2)
	    {
		kappa[2][2] = matrixavg[2];
		kappa[2][3] = matrixavg[3];
		kappa[3][2] = matrixavg[2];
		kappa[3][3] = matrixavg[3];
	    }

	    if (calibfile == 3)
	    {
		kappa[4][4] = matrixavg[2];
		kappa[4][5] = matrixavg[3];
		kappa[5][4] = matrixavg[2];
		kappa[5][5] = matrixavg[3];
	    }
	}
	else
	{
	    printf("ERROR: Missing matrixfile %s\n", filename);
	    
	    for (j=0; j<6; j++)
	    {
		if (calibfile == 1) matrix1[j][0] = -1.0E10;
		if (calibfile == 2) matrix2[j][0] = -1.0E10;
		if (calibfile == 3) matrix3[j][0] = -1.0E10;
		if (calibfile == 4) matrix4[j][0] = -1.0E10;
	    }
	}
    }
    
}
*/
  

//
//*** Reads the calibration information from the four pre-processed phot files
//

int find_photfile(char *filename, long filenr, double *matrix1level, double *matrix2level, double *matrix3level)
{
     char  scriptnrtext[10], filename2[100], temptext[100];
     FILE *stream, *stream2;
     float value[3], value_;
     int i, j, j2, scriptnrall, objid;
     int found;

     //*** search the corr-file, which comes in the filenumbers after the datafile
     stream = fopen(filename, "r");
     found = 0;
     
     while (feof(stream) == 0)
     {
	 fscanf(stream, "%i %i %s\n", &scriptnrall, &objid, &temptext);
	 
	 if ((scriptnrall > filenr) && (found == 0))
	 {
	     found = 1;

	     sprintf(scriptnrtext, "%i", scriptnrall);
	     strcpy(filename2, ProcDir);
	     strcat(filename2, "phot");
	     //strcpy(filename2, "matrixfiles/phot");
	     strcat(filename2, scriptnrtext);
	     strcat(filename2, ".dat");

	     if (file_exist(filename2) == 1)
	     {
		 stream2 = fopen(filename2, "r");

		 printf("Read Matrix: %s\n", filename2);

		 fscanf(stream2, "%lf %lf  %lf %lf  %lf %lf\n", &matrix1level[0], &matrix1level[1], &matrix1level[2], &matrix1level[3], &matrix1level[4], &matrix1level[5]);
		 fscanf(stream2, "%lf %lf  %lf %lf  %lf %lf\n", &matrix2level[0], &matrix2level[1], &matrix2level[2], &matrix2level[3], &matrix2level[4], &matrix2level[5]);
		 fscanf(stream2, "%lf %lf  %lf %lf  %lf %lf\n", &matrix3level[0], &matrix3level[1], &matrix3level[2], &matrix3level[3], &matrix3level[4], &matrix3level[5]);

		 fclose(stream2);
	     }
	     else
	     {
		 matrix1level[0] = -1.0e10;
		 matrix2level[0] = -1.0e10;
		 matrix3level[0] = -1.0e10;
	     }
	     
	 }
     }
     
     fclose(stream);

     return found;
}


int find_matrix4(char *filename, long filenr, double *matrix4[])
{
     char  scriptnrtext[10], filename2[100], temptext[100];
     FILE *stream, *stream2;
     float value[6];
     int i, j, j2, scriptnrall, objid;
     int found = 0;

     if (file_exist(filename) == 0) return 0;

     //*** search the matrix-file, which comes in the filenumbers after the datafile
     stream = fopen(filename, "r");
     
     while (feof(stream) == 0)
       {
	 fscanf(stream, "%i %i %s\n", &scriptnrall, &objid, &temptext);

	 if ((scriptnrall > filenr) && (found == 0))
	   {
	     found = scriptnrall;
	     
	     sprintf(scriptnrtext, "%i", scriptnrall);
	     strcpy(filename2, ProcDir);
	     strcat(filename2, "matrix");
	     //strcpy(filename2, "matrixfiles/matrix");
	     strcat(filename2, scriptnrtext);
	     strcat(filename2, ".m4");

	     if (file_exist(filename2) == 1)
	       {
		 stream2 = fopen(filename2, "r");
		 
		 if (repdetails == 1) printf("Read matrixfile: %s\n", filename2);
		 
		 for (j=0; j<=255; j++)
		 {
     		     fscanf(stream2, "%i %f %f %f %f %f %f\n", &j2, &value[0], &value[1], &value[2], &value[3], &value[4], &value[5]);
		     
		     for (i=0; i<6; i++)
			 matrix4[i][j] = value[i];
		 }
		 
		 fclose(stream2);
	       }
	     else
	       for (i=0; i<6; i++)
		 matrix4[0][0] = -1.0E10;
	   }
       }

     fclose(stream);

     return found;
}


int find_matrixavg4(char *filename, long filenr, double *matrix4)
{
     char  scriptnrtext[10], filename2[100], temptext[100];
     FILE *stream, *stream2;
     float value[6];
     int i, j, j2, scriptnrall, objid;
     int found = 0;

     if (file_exist(filename) == 0) return 0;

     //*** search the matrix-file, which comes in the filenumbers after the datafile
     stream = fopen(filename, "r");
     
     while (feof(stream) == 0)
       {
	 fscanf(stream, "%i %i %s\n", &scriptnrall, &objid, &temptext);

	 if ((scriptnrall > filenr) && (found == 0))
	   {
	     found = scriptnrall;
	     
	     sprintf(scriptnrtext, "%i", scriptnrall);
	     strcpy(filename2, ProcDir);
	     strcat(filename2, "matrixavg");
	     //strcpy(filename2, "matrixfiles/matrixavg");
	     strcat(filename2, scriptnrtext);
	     strcat(filename2, ".m4");
	     
	     if (file_exist(filename2) == 1)
	     {
		 //printf("READING: %s\n", filename2);

		 stream2 = fopen(filename2, "r");
		 
		 if (repdetails == 1) printf("Read matrixfile: %s\n", filename2);

		 fscanf(stream2, "%f %f %f %f %f %f\n", &value[0], &value[1], &value[2], &value[3], &value[4], &value[5]);

		 for (i=0; i<6; i++)
		     matrix4[i] = value[i];
		 
		 fclose(stream2);
	     }
	     else
		 for (i=0; i<6; i++)
		     matrix4[0] = -1.0E10;
	   }
       }

     fclose(stream);

     return found;
}



void calc_correction(int channel, double *matrix1level, double *matrix2level, double *matrix3level, float *corr, float *calibtag)
{
  double c;

  c = -1.0;

  //printf("_CHANNEL: %i  %f\n", channel, c);

  switch (channel)
    {
    case 0:
      if (((matrix3level[0]*matrix2level[0]) < 0.0) || (matrix3level[1]*matrix2level[1] < 0.0))
	c = -2.0;
      else
	if (((matrix3level[0]*matrix2level[0]) != 0.0) && (matrix3level[1]*matrix2level[1] != 0.0))
	  c = (matrix3level[0] + matrix3level[1] + matrix2level[0] + matrix2level[1])/(2*(sqrt(matrix3level[0]*matrix2level[0]) + sqrt(matrix3level[1]*matrix2level[1])));
      break;
      
    case 1:
      if (((matrix3level[2]*matrix1level[2]) < 0.0) || (matrix3level[3]*matrix1level[3] < 0.0))
	c = -2.0;
      else
	if (((matrix3level[2]*matrix1level[2]) != 0.0) && (matrix3level[3]*matrix1level[3] != 0.0))
	  c = (matrix3level[2] + matrix3level[3] + matrix1level[2] + matrix1level[3])/(2*(sqrt(matrix3level[2]*matrix1level[2]) + sqrt(matrix3level[3]*matrix1level[3])));
      break;
      
    case 2:
      if (((matrix1level[4]*matrix2level[4]) < 0.0) || (matrix1level[5]*matrix2level[5] < 0.0))
	c = -2.0;
      else
	if (((matrix1level[4]*matrix2level[4]) != 0.0) && (matrix1level[5]*matrix2level[5] != 0.0))
	  c = (matrix1level[4] + matrix1level[5] + matrix2level[4] + matrix2level[5])/(2*(sqrt(matrix1level[4]*matrix2level[4]) + sqrt(matrix1level[5]*matrix2level[5])));
      break;
    }

  
  if (((matrix2level[0]/matrix3level[0] > matrixscat_threshold) || (matrix3level[0]/matrix2level[0] > matrixscat_threshold)) ||
      ((matrix2level[1]/matrix3level[1] > matrixscat_threshold) || (matrix3level[1]/matrix2level[1] > matrixscat_threshold)) ||
      ((matrix1level[2]/matrix3level[2] > matrixscat_threshold) || (matrix3level[2]/matrix1level[2] > matrixscat_threshold)) ||
      ((matrix1level[3]/matrix3level[3] > matrixscat_threshold) || (matrix3level[3]/matrix1level[3] > matrixscat_threshold)) ||
      ((matrix1level[4]/matrix2level[4] > matrixscat_threshold) || (matrix2level[4]/matrix1level[4] > matrixscat_threshold)) ||
      ((matrix1level[5]/matrix2level[5] > matrixscat_threshold) || (matrix2level[5]/matrix1level[5] > matrixscat_threshold)))
  {
      printf("   Flux ratio between different matrix files above threshold!\n"); 
      c = 1.0;
      *calibtag -= 1.0;
    }

  if (c == 0.0) 
    {
      c = 1.0;
      *calibtag -= 10.0;
    }

  if (c > 2.0) 
    { 
      printf("   Correction factor run out of limit (%f)!\n", c); 
      c = 1.0;
      *calibtag -= 1.0;
    };

  if (c < 1.0) 
    { 
      printf("   Correction factor run out of limit (%f)!\n", c); 
      c = 1.0; 
      *calibtag -= 1.0;
    };

  *corr = c;
}



//
//*** Reads the calibration information from the four pre-processed matrix files
//
/*
int find_matrixfile(char *filename, long filenr, double *matrix1[], double *matrix2[], double *matrix3[], double *matrix4[])
{
     char  scriptnrtext[10], calibfiletext[10], filename2[100];
     FILE *stream, *stream2;
     float value[6];
     int i, j, j2, scriptnrall, calibfile, objid;
     int found[5];


     //*** search the matrix-file, which comes in the filenumbers after the datafile
     stream = fopen(filename, "r");
     
     for (i=1; i<=4; i++)
       found[i] = 0;
     
     while (feof(stream) == 0)
       {
	 fscanf(stream, "%i %i %i\n", &scriptnrall, &calibfile, &objid);

	 if ((scriptnrall > filenr) && (found[calibfile] == 0))
	   {
	     found[calibfile] = scriptnrall;
	     
	     sprintf(scriptnrtext, "%i", scriptnrall);
	     sprintf(calibfiletext, "%i", calibfile);
	     strcpy(filename2, "matrix");
	     //strcpy(filename2, "matrixfiles/matrix");
	     strcat(filename2, scriptnrtext);
	     strcat(filename2, ".m");
	     strcat(filename2, calibfiletext);

	     if (file_exist(filename2) == 1)
	       {
		 stream2 = fopen(filename2, "r");
		 
		 if (repdetails == 1) printf("Read matrixfile: %s\n", filename2);
		 
		 for (j=0; j<=255; j++)
		   {
     		     fscanf(stream2, "%i %f %f %f %f %f %f\n", &j2, &value[0], &value[1], &value[2], &value[3], &value[4], &value[5]);
		     
		     for (i=0; i<6; i++)
		       {
			 if (calibfile == 1) matrix1[i][j] = value[i];
			 if (calibfile == 2) matrix2[i][j] = value[i];
			 if (calibfile == 3) matrix3[i][j] = value[i];
			 if (calibfile == 4) matrix4[i][j] = value[i];
		       }
		   }

		 fclose(stream2);
	       }
	     else
	       for (i=0; i<6; i++)
		 {
		   if (calibfile == 1) matrix1[0][0] = -1.0E10;
		   if (calibfile == 2) matrix2[0][0] = -1.0E10;
		   if (calibfile == 3) matrix3[0][0] = -1.0E10;
		   if (calibfile == 4) matrix4[0][0] = -1.0E10;
		 }
	   }
       }

     fclose(stream);


     return (found[1] + found[2] + found[3] + found[4]);
}
*/

float calc_biascorr(int channel, float amp, float weight)
{
  FILE *streambias;
  float chisqr, minchisqr, minchisqrparam, v, a, vis, param, noiseparam;

  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "biascorr.dat");
  streambias = fopen(ProcFilename, "r");

  minchisqr = 1.0e10;
  while (feof(streambias) == 0)
    {
	fscanf(streambias, "%f %f\n", &noiseparam, &param);

	chisqr = (noiseparam - weight/amp)*(noiseparam - weight/amp);
	
	if (chisqr < minchisqr)
	{
	    minchisqr = chisqr;
	    minchisqrparam = param;
	}
    }

  fclose(streambias);

  //solve equation
  minchisqr = 1.0e10;
  for (v=1.0; v>=0.0; v-=0.01)
  {
      a = 1.0 - ((1.0-v) + exp(minchisqrparam*(1.0-v)*(1.0-v)) - 1.0);
      
      chisqr = (a-amp)*(a-amp);

      if (chisqr < minchisqr)
      {
	  minchisqr = chisqr;
	  vis = v;
      }
  }

  return vis;
}

float fit_biascorr(float *param2, float *noiseparam)
{
  FILE *streamsim;
  int channel, nr, i;
  float amp, fitamp, weight, noise, noise2, noise3;
  float factor, nrfactor;
  double amp2[100], fitamp2[100], noiseparam2[100];
  double param, chisqr;

  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "simbias.dat");
  streamsim = fopen(ProcFilename, "r");

  nr = -1;
  while (feof(streamsim) == 0)
    {
	fscanf(streamsim, "%i %f %f %f %f %f %f\n", &channel, &amp, &fitamp, &weight, &noise, &noise2, &noise3);

	if (fitamp > 0.0)
	{
	    nr++;
	    amp2[nr] = amp;
	    fitamp2[nr] = fitamp;
	    noiseparam2[nr] = noise2;
	}
    }

  fclose(streamsim);

  //determine factor
  factor = 0.0;
  nrfactor = 0.0;

  for (i=0; i<=nr; i++)
      if (amp2[i] > 0.99)
      {
	  factor += fitamp2[i];
	  nrfactor++;
      }

  factor /= nrfactor;

  for (i=0; i<=nr; i++)
      fitamp2[i] /= factor;

  for (i=0; i<=nr; i++)
      fitamp2[i] -= amp2[i];

  //write new file
  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "simbias2.dat");
  streamsim = fopen(ProcFilename, "w");

  for (i=0; i<=nr; i++)
      fprintf(streamsim, "%i %f %f %f %f %f %f\n", 0, amp2[i], fitamp2[i], 0.0, 0.0, 0.0, 0.0);

  fclose(streamsim);

  //calc avg. noiseparameter (=noise/genfringeamp)
  for (i=0; i<=nr; i++)
      (*noiseparam) += noiseparam2[i];

  (*noiseparam) /= (nr+1);

  //fit residual data
  fit_exp_x2(amp2, fitamp2, nr, &param, &chisqr);

  printf("\n\nBIAS-FIT for Noise=%f: f=%f mit ChiSqr=%f\n\n\n", (*noiseparam), param, chisqr);

  *param2 = (float)param;
}

#undef calibavg
#undef calibavgerror
#undef calibset_vis
#undef calibset_visscat
#undef calibset_nr
