/* convert.c                                 */
/* Interferometry Data Reduction Software    */
/* by Stefan Kraus                           */
/* 2002-05                                   */
/*                                           */
/* copyright laws apply                      */


#include "exchange.h"

void convert2fitsoi(char *filename_in, char *filename_out)
{
    oi_array array[max_nights];
    oi_target targets;
    oi_wavelength wave[max_nights];
    oi_vis vis;
    oi_vis2 vis2;
    oi_t3 t3;
    
    char filename[FLEN_FILENAME];
    FILE *fp, *fp2;
    int i, irec, iwave, itarg, status, found;
    int nr_arrays;
    int nr_ins;
    int nr_targets;
    fitsfile *fptr;
    
    char sta_tel0[5], sta_tel1[5], sta_tel2[5];
    double sta_tel0_0, sta_tel0_1, sta_tel0_2;
    double sta_tel1_0, sta_tel1_1, sta_tel1_2;
    double sta_tel2_0, sta_tel2_1, sta_tel2_2;
    char arr_name[20];
    char tar_star[30];
    double tar_ra, tar_dec, ins_wavelength;
    char ins_combiner[30], ins_name[20];
    
    int scriptnrall, objid, count;
    char vis_date[10];
    double vis_mjd, vis_time, vis_int_time;
    
    char nr_alloc_arr[max_vispoints][20];
    char nr_alloc_ins[max_vispoints][30];
    char nr_alloc_date[max_vispoints][12];
    double nr_alloc_mjd[max_vispoints];
    double nr_alloc_time[max_vispoints];
    double nr_alloc_int_time[max_vispoints];

    int dataint[3], olddataint[3];
    float data[24], olddata[24];

    int end_table, end_file, first_item;

    int nr_arraydim;
    int arraydim_vis2[100], arraydim_cp[100];

    int channel;


    //*** Read ARRAY-CONFIGURATION INFO
    strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "info.dat");
    fp2 = fopen(ProcFilename, "r");
    
    nr_arrays = 0;
    nr_ins = 0;
    nr_targets = 0;
    strcpy(array[0].arrname, "");

    while (feof(fp2) == 0)
    {
	fscanf(fp2, "%i %i %s %lf %lf  %s %lf %lf %lf   %s %s %lf  %s %s %s %s   %lf %lf %lf   %lf %lf %lf   %lf %lf %lf\n", &scriptnrall, &objid, &tar_star, &tar_ra, &tar_dec, &vis_date, &vis_time, &vis_mjd, &vis_int_time, &ins_name, &ins_combiner, &ins_wavelength, &arr_name, &sta_tel0, &sta_tel1, &sta_tel2, &sta_tel0_0, &sta_tel0_1, &sta_tel0_2, &sta_tel1_0, &sta_tel1_1, &sta_tel1_2, &sta_tel2_0, &sta_tel2_1, &sta_tel2_2);

	//if (objid == targetid)
	{
	    strcpy(nr_alloc_arr[scriptnrall], arr_name);
	    strcpy(nr_alloc_ins[scriptnrall], ins_name);
	    strcpy(nr_alloc_date[scriptnrall], vis_date);

	    nr_alloc_mjd[scriptnrall] = vis_mjd;
	    nr_alloc_time[scriptnrall] = vis_time;
	    nr_alloc_int_time[scriptnrall] = vis_int_time;

	    //ARRAY INFO
	    found = 0;
	    for (i=0;i<nr_arrays;i++)
		if (strcasecmp(arr_name, array[i].arrname) == 0)
		    found = 1;

	    if (found == 0)
	    {
		strcpy(array[nr_arrays].arrname, arr_name);
		array[nr_arrays].revision = 1;
		strcpy(array[nr_arrays].frame, "GEOCENTRIC");
		array[nr_arrays].arrayx = cos(obs_lat*DToR);
		array[nr_arrays].arrayy = cos(obs_lon*DToR);
		array[nr_arrays].arrayz = sin(obs_lat*DToR);

		array[nr_arrays].nelement = 3;  //not necessary in lastest release
		array[nr_arrays].elem = malloc(array[nr_arrays].nelement*sizeof(element));
	    
		array[nr_arrays].elem[0].sta_index = 0;
		strcpy(array[nr_arrays].elem[0].tel_name, "A");
		strcpy(array[nr_arrays].elem[0].sta_name, sta_tel0);
		array[nr_arrays].elem[0].staxyz[0] = sta_tel0_0;
		array[nr_arrays].elem[0].staxyz[1] = sta_tel0_1;
		array[nr_arrays].elem[0].staxyz[2] = sta_tel0_2;

		array[nr_arrays].elem[1].sta_index = 1;
		strcpy(array[nr_arrays].elem[1].tel_name, "B");
		strcpy(array[nr_arrays].elem[1].sta_name, sta_tel1);
		array[nr_arrays].elem[1].staxyz[0] = sta_tel1_0;
		array[nr_arrays].elem[1].staxyz[1] = sta_tel1_1;
		array[nr_arrays].elem[1].staxyz[2] = sta_tel1_2;

		array[nr_arrays].elem[2].sta_index = 2;
		strcpy(array[nr_arrays].elem[2].tel_name, "C");
		strcpy(array[nr_arrays].elem[2].sta_name, sta_tel2);
		array[nr_arrays].elem[2].staxyz[0] = sta_tel2_0;
		array[nr_arrays].elem[2].staxyz[1] = sta_tel2_1;
		array[nr_arrays].elem[2].staxyz[2] = sta_tel2_2;

		array[nr_arrays].elem[0].diameter = 0.45;
		array[nr_arrays].elem[1].diameter = 0.45;
		array[nr_arrays].elem[2].diameter = 0.45;

		nr_arrays++;
	    }


	    //TARGET INFO
	    found = 0;
	    for (i=0;i<nr_targets;i++)
		if (strcasecmp(tar_star, targets.targ[i].target) == 0)
		    found = 1;

	    if (found == 0)
	    {
		targets.ntarget = nr_targets+1;
		if (nr_targets == 0)
		    targets.targ = malloc(targets.ntarget*sizeof(target));
		else
		    targets.targ = realloc(targets.targ, targets.ntarget*sizeof(target));

		strcpy(targets.targ[nr_targets].target, tar_star);
		targets.targ[nr_targets].target_id = objid;
		targets.targ[nr_targets].raep0 = tar_ra * 15.0;
		targets.targ[nr_targets].decep0 = tar_dec;
		targets.targ[nr_targets].equinox = 2000.0;
		targets.targ[nr_targets].ra_err = 0.0;
		targets.targ[nr_targets].dec_err = 0.0;
		targets.targ[nr_targets].sysvel = 0.0;
		strcpy(targets.targ[nr_targets].veltyp, "");
		strcpy(targets.targ[nr_targets].veldef, "");
		targets.targ[nr_targets].pmra = 0.0;
		targets.targ[nr_targets].pmdec = 0.0;
		targets.targ[nr_targets].pmra_err = 0.0;
		targets.targ[nr_targets].pmdec_err = 0.0;
		targets.targ[nr_targets].parallax = 0.0;
		targets.targ[nr_targets].para_err = 0.0;
		strcpy(targets.targ[nr_targets].spectyp, "");

		targets.revision = 1;

		nr_targets++;
	    }


	    //WAVELENGTH INFO
	    found = 0;
	    for (i=0;i<nr_ins;i++)
		if (strcasecmp(ins_name, wave[i].insname) == 0)
		    found = 1;

	    if (found == 0)
	    {
		wave[nr_ins].nwave = 1;
		strcpy(wave[nr_ins].insname, ins_name);

		wave[nr_ins].eff_wave = malloc(wave[nr_ins].nwave*sizeof(float));
		wave[nr_ins].eff_band = malloc(wave[nr_ins].nwave*sizeof(float));

		wave[nr_ins].eff_wave[0] = ins_wavelength*1.0e-6;
		wave[nr_ins].eff_band[0] = wavelength_bandwidth*1.0e-6;

		wave[nr_ins].revision = 1;

		nr_ins++;
	    }
	}
    }

    printf("\n*** Writing OI-FITS File %s:\n", filename_out);
    printf("  - Found %i telescope configuration(s):\n", nr_arrays);
    for (i=0; i<nr_arrays; i++)
	printf("         %s\n", array[i].arrname);
    printf("  - Found %i target(s):\n", nr_targets);
    for (i=0; i<nr_targets; i++)
	printf("         %s (ID %i)\n", targets.targ[i].target, targets.targ[i].target_id);
    printf("  - Found %i spectral channel(s):\n", nr_ins);
    for (i=0; i<nr_ins; i++)
	printf("         %s  Wavel. %lf  Bandw. %lf\n", wave[i].insname, wave[i].eff_wave[0]*1.0e6, wave[i].eff_band[0]*1.0e6);

    fclose(fp2);

    printf("\n");

    
    /* Write out FITS file */
    status = 0;
    fits_create_file(&fptr, filename_out, &status);
    if (status) 
    {
	fits_report_error(stderr, status);
	exit(1);
    }
    write_oi_target(fptr, targets, &status);
    for (i=0;i<nr_arrays;i++)
	write_oi_array(fptr, array[i], 1, &status);
    for (i=0; i<nr_ins; i++)
	write_oi_wavelength(fptr, wave[i], 1, &status);
    


    //*** Read VIS/CP INFO
    for (channel = 0; channel <= 3; channel++)
    {
	//determine how large the arrays have to be
	fp = fopen(filename_in, "r");
	
	end_table = 0;
	end_file = 0;
	first_item = 1;
	nr_arraydim = 0;
	count = 0;

	while ((feof(fp) == 0) || (end_file == 0))
	{
	    if (feof(fp) == 0)
	    {
		fscanf(fp, streamampstring1, &dataint[0], &data[0], &dataint[1], &data[1], &data[2], &data[3], &data[4], &data[5], &data[6], &data[7], &data[8], &data[9], &data[10], &data[11], &data[12], &data[13], &data[14], &data[15], &data[16], &data[17], &dataint[2]);
		fscanf(fp, streamampstring2, &data[18], &data[19], &data[20], &data[21], &data[22], &data[23]);
	    }
	    else
		end_file = 1;

	    count++;
	    //dataint[0] = count;


	    if (count > 1)
	    {
		if (strcasecmp(nr_alloc_date[dataint[0]], nr_alloc_date[olddataint[0]]) != 0) end_table = 1;
		if (strcasecmp(nr_alloc_arr[dataint[0]], nr_alloc_arr[olddataint[0]]) != 0) end_table = 1;
		if (strcasecmp(nr_alloc_ins[dataint[0]], nr_alloc_ins[olddataint[0]]) != 0) end_table = 1;
	    }
	
	    for (i=0; i<3; i++)
		olddataint[i] = dataint[i];
	
	    for (i=0; i<24; i++)
		olddata[i] = data[i];

	    if ((end_table == 1) || (end_file == 1) || (first_item == 1))
	    {
		if (first_item == 0)
		{
		    end_table = 0;
		}

		if (end_file == 0)
		{
		    //start new table
		    nr_arraydim++;
		    arraydim_vis2[nr_arraydim] = 0;
		    arraydim_cp[nr_arraydim] = 0;
		}
	    }

	    if (end_file == 0)
	    {
		if (channel < 3)
		{
		    //VIS2: add various baselines
		    //for (i=0; i<3; i++)
		    i = channel;

		    if (data[1+i*3] > 0.0)
			arraydim_vis2[nr_arraydim]++;
		}
		else
		{
		    //CP
		    if (data[16] > -4.0)
			arraydim_cp[nr_arraydim]++;
		}
	    }

	    first_item = 0;
	}
    
	fclose(fp);

	

	fp = fopen(filename_in, "r");

	end_table = 0;
	end_file = 0;
	first_item = 1;
	nr_arraydim = 0;
	count = 0;

	while ((feof(fp) == 0) || (end_file == 0))
	{
	    if (feof(fp) == 0)
	    {
		fscanf(fp, streamampstring1, &dataint[0], &data[0], &dataint[1], &data[1], &data[2], &data[3], &data[4], &data[5], &data[6], &data[7], &data[8], &data[9], &data[10], &data[11], &data[12], &data[13], &data[14], &data[15], &data[16], &data[17], &dataint[2]);
		fscanf(fp, streamampstring2, &data[18], &data[19], &data[20], &data[21], &data[22], &data[23]);
	    }
	    else
		end_file = 1;

	    count++;
	
	    if (count > 1)
	    {
		if (strcasecmp(nr_alloc_date[dataint[0]], nr_alloc_date[olddataint[0]]) != 0) end_table = 1;
		if (strcasecmp(nr_alloc_arr[dataint[0]], nr_alloc_arr[olddataint[0]]) != 0) end_table = 1;
		if (strcasecmp(nr_alloc_ins[dataint[0]], nr_alloc_ins[olddataint[0]]) != 0) end_table = 1;
	    }
	    
	    if ((end_table == 1) || (end_file == 1) || (first_item == 1))
	    {
		//write full table
		if (first_item == 0)
		{
		    printf("  Configuration: %s   %s   %s\n", nr_alloc_date[olddataint[0]], nr_alloc_arr[olddataint[0]], nr_alloc_ins[olddataint[0]]);
		    
		    if (channel < 3)
		    {
			if (vis2.numrec > 0)
			{
			    write_oi_vis2(fptr, vis2, 1, &status);
			    printf("     Writing %i Vis2 measurement(s)\n", (int)vis2.numrec);
			}
		    }
		    else
		    {
			if (t3.numrec > 0)
			{
			    write_oi_t3(fptr, t3, 1, &status);
			    printf("     Writing %i CP measurement(s)\n", (int)t3.numrec);
			}
		    }

		    printf("\n");

		    end_table = 0;
		}

		if (end_file == 0)
		{
		    //start new table
		    nr_arraydim++;

		    if (channel < 3)
		    {
			strcpy(vis2.date_obs, nr_alloc_date[dataint[0]]);
			strcpy(vis2.arrname, nr_alloc_arr[dataint[0]]);
			strcpy(vis2.insname, nr_alloc_ins[dataint[0]]);
			vis2.numrec = 0;
			vis2.revision = 1;
			vis2.nwave = 1;
		    }
		    else
		    {
			strcpy(t3.date_obs, nr_alloc_date[dataint[0]]);
			strcpy(t3.arrname, nr_alloc_arr[dataint[0]]);
			strcpy(t3.insname, nr_alloc_ins[dataint[0]]);
			t3.numrec = 0;
			t3.revision=1;
			t3.nwave = 1;
		    }
		}
	    }

	    for (i=0; i<3; i++)
		olddataint[i] = dataint[i];
	
	    for (i=0; i<24; i++)
		olddata[i] = data[i];
	
	
	    if (end_file == 0)
	    {
		//VIS2: add various baselines
		i = channel;
		if (channel < 3)
		    //for (i=0; i<3; i++)
		    if (data[1+i*3] > 0.0)
		    {
			vis2.numrec++;
		    
			if (vis2.numrec == 1)
			    vis2.record = malloc(arraydim_vis2[nr_arraydim]*sizeof(oi_vis2_record));
		    
			vis2.record[vis2.numrec-1].target_id = dataint[1];
			vis2.record[vis2.numrec-1].time = nr_alloc_time[dataint[0]];
			vis2.record[vis2.numrec-1].mjd = nr_alloc_mjd[dataint[0]];
			vis2.record[vis2.numrec-1].int_time = nr_alloc_int_time[dataint[0]];
			vis2.record[vis2.numrec-1].vis2data = malloc(1*sizeof(double));
			vis2.record[vis2.numrec-1].vis2data[0] = data[1+i*3];
			vis2.record[vis2.numrec-1].vis2err = malloc(1*sizeof(double));
			vis2.record[vis2.numrec-1].vis2err[0] = data[2+i*3];
			vis2.record[vis2.numrec-1].ucoord = data[10+i*2]/100.0;
			vis2.record[vis2.numrec-1].vcoord = data[11+i*2]/100.0;
			
			if (i==0)
			{
			    vis2.record[vis2.numrec-1].sta_index[0] = 0; //A-B
			    vis2.record[vis2.numrec-1].sta_index[1] = 1; //A-B
			}
			
			if (i==1)
			{
			    vis2.record[vis2.numrec-1].sta_index[0] = 0; //A-C
			    vis2.record[vis2.numrec-1].sta_index[1] = 2; //A-C
			}
			
			if (i==2)
			{
			    vis2.record[vis2.numrec-1].sta_index[0] = 1; //B-C
			    vis2.record[vis2.numrec-1].sta_index[1] = 2; //B-C
			}
			
			vis2.record[vis2.numrec-1].flag = malloc(1*sizeof(char));
			vis2.record[vis2.numrec-1].flag[0] = FALSE;
		    }
	    
	    
		//CP
		if (channel == 3)
		    if (data[16] > -4.0)
		    {
			t3.numrec++;

			if (t3.numrec == 1)
			    t3.record = malloc(arraydim_cp[nr_arraydim]*sizeof(oi_t3_record));
		
			t3.record[t3.numrec-1].target_id = dataint[1];
			t3.record[t3.numrec-1].time = nr_alloc_time[dataint[0]];
			t3.record[t3.numrec-1].mjd = nr_alloc_mjd[dataint[0]];
			t3.record[t3.numrec-1].int_time = nr_alloc_int_time[dataint[0]];

			t3.record[t3.numrec-1].t3amp = malloc(1*sizeof(double));
			t3.record[t3.numrec-1].t3amp[0] = 0.0;
			t3.record[t3.numrec-1].t3amperr = malloc(1*sizeof(double));
			t3.record[t3.numrec-1].t3amperr[0] = 0.0;


			t3.record[t3.numrec-1].t3phi = malloc(1*sizeof(double));
			t3.record[t3.numrec-1].t3phi[0] = data[16]*RToD;
			t3.record[t3.numrec-1].t3phierr = malloc(1*sizeof(double));
			t3.record[t3.numrec-1].t3phierr[0] = data[17]*RToD;
			t3.record[t3.numrec-1].u1coord = data[10+0*2]/100.0; //A-B
			t3.record[t3.numrec-1].v1coord = data[11+0*2]/100.0; //A-B
			t3.record[t3.numrec-1].u2coord = data[10+2*2]/100.0; //B-C
			t3.record[t3.numrec-1].v2coord = data[11+2*2]/100.0; //B-C
			t3.record[t3.numrec-1].sta_index[0] = 0;
			t3.record[t3.numrec-1].sta_index[1] = 1;
			t3.record[t3.numrec-1].sta_index[2] = 2;

			t3.record[t3.numrec-1].flag = malloc(1*sizeof(char));
			t3.record[t3.numrec-1].flag[0] = FALSE;
		    }
	    }
	
	    first_item = 0;
	}
    
	fclose(fp);
    }
    

    if (status) 
    {
	/* Error occurred - delete partially-created file and exit */
	fits_delete_file(fptr, &status);
	exit(1);
    } 
    else
	fits_close_file(fptr, &status);
}
