/* fit.c                                     */
/* Interferometry Data Reduction Software    */
/* by Stefan Kraus                           */
/* 2002-05                                   */
/*                                           */
/* copyright laws apply                      */


#include <fftw.h>
#include "fitfuncs.c"

/* SINC-ENVELOPE FIT */

void func(float x, float a[], float *y, float dyda[], int na)
{
	float y_;
	float temp1, temp2, temp3;
       
	
	//SINC FIT
	y_ = sinc((x-a[3])/a[2]);
	y_ = abs2(y_);
	*y = a[1]*y_;

	dyda[1] = y_;

	temp1 = abs2(1/a[2]);
	temp2 = abs2(sin((x-a[3])/a[2])/(x-a[3]));
	dyda[2] = temp2*sign(a[2])*a[1]-sign(sin((x-a[3])/a[2])*x)*temp1*cos((x-a[3])/a[2])*a[1];

	temp1 = abs2(1/(a[3]-x));
	temp2 = abs2(sin((x-a[3])/a[2])/(x-a[3]));
	temp3 = abs2(sin((a[3]-x)/a[2])*a[2]/(a[3]-x));
	dyda[3] = cos((a[3]-x)/a[2])*temp1*sign(sin((a[3]-x)/a[2])*a[2])*x*a[1]-temp3*x*a[1]/(a[3]-x);
}



double func_(float x, float a[])
{
        float y_;

        y_ = sinc((x-a[3])/a[2]);
        y_ = abs2(y_)*a[1];
        return y_;
}


void fit_envelope(int channel, double *envelope, double centerPos, int fit_window, float *a, int topbottom, int fitenv_varwid)
{
    const int NPT = 100;
    const int MA = 3;


    int i,*ia,iter,itst,k,mfit=MA;
    int points;
    float alamda,chisq,ochisq,*x,*y,*sig,**covar,**alpha, maxvalue;
    float extrem;
    float gues[MA+1];
    FILE *fp1, *fp2;

    gues[0] = 0.0;
    gues[1] = 0.4;
    gues[2] = 36.0;
    gues[3] = 0.0;


	ia=ivector(1,MA);
	x=vector(1,NPT);
	y=vector(1,NPT);
	sig=vector(1,NPT);
	covar=matrix(1,MA,1,MA);
	alpha=matrix(1,MA,1,MA);

	errormsg = 0;


	points = 0;
	extrem = 0;
	maxvalue = -1.0e10;
        for (i=0;i<nPtsScan;i++)
      	  if ((envelope[i] != 0) && (abs2(i-centerPos)<fit_window))
	    {
	      points++;
	      x[points] = i-centerPos;
	      y[points] = abs2(envelope[i]);
	      if (y[points] > maxvalue) maxvalue = y[points];
	      if (abs2(y[points]) > extrem) extrem = abs2(y[points]);
	      sig[points] = 1.0+1.0/y[points];
	    }

	if (channel == 0) gues[2] = fitenv_fixwid0;
	if (channel == 1) gues[2] = fitenv_fixwid1;
	if (channel == 2) gues[2] = fitenv_fixwid2;

	for (i=1;i<=mfit;i++) ia[i]=1;
	ia[3]=0;
	ia[2]=fitenv_varwid;
	for (i=1;i<=MA;i++) a[i]=gues[i];


	for (iter=1;iter<=2;iter++) 
	  {
	    alamda = -1;
	    mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func,&alamda);
	    k=1;
	    itst=0;
	    for (;;) {
	      k++;
	      ochisq=chisq;
	      mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func,&alamda);
	      if (chisq > ochisq)
		itst=0;
	      else if (fabs(ochisq-chisq) < 0.1)
		itst++;
	      if (itst < 4) continue;
	      alamda=0.0;
	      mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func,&alamda);
	      
	      break;
	    }
	  }
	
	if (a[1] > maxvalue) a[1] = maxvalue;
	

	//Recompose the function
	if ((repfit == 1) && (channel == 0))// && ((topbottom == 1) || (topbottom == -1)))
	  {
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "env1.dat");
	      fp1 = fopen(ProcFilename,"w");
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "env2.dat");
	      fp2 = fopen(ProcFilename,"w");
	    
	    for (i=1;i<=points;i++)
	      {
		fprintf(fp1, "%f %f\n", (float)x[i]*OPD_slowfringe/(float)nPtsScan, y[i]);
		fprintf(fp2, "%f %f\n", (float)x[i]*OPD_slowfringe/(float)nPtsScan, -y[i]);
	      }	    

	    fclose(fp1);
	    fclose(fp2);
	    

	    //if (topbottom < 0)
	    strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "fitenv1.dat");
	    fp1 = fopen(ProcFilename, "w");
	    //else
	    strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "fitenv2.dat");
	    fp2 = fopen(ProcFilename, "w");
	    
	    for (i=1;i<=nPtsScan;i++)
	      {
		fprintf(fp1, "%f %f\n", (float)(i-centerPos)*OPD_slowfringe/(float)nPtsScan, -func_((float)i-centerPos, a));
		fprintf(fp2, "%f %f\n", (float)(i-centerPos)*OPD_slowfringe/(float)nPtsScan, func_((float)i-centerPos, a));
	      }

	    fclose(fp1);
	    fclose(fp2);
	  }


	free_matrix(alpha,1,MA,1,MA);
	free_matrix(covar,1,MA,1,MA);
	free_vector(sig,1,NPT);
	free_vector(y,1,NPT);
	free_vector(x,1,NPT);
	free_ivector(ia,1,MA);

	if ((a[1] == gues[1]) && (a[2] == gues[2])) errormsg = 1;


	if (errormsg != 0)
	  {
	    a[1] = -1.0; //amp
	    a[2] = -1.0; //wid
	    a[3] = -1.0; //pos

	    a[0] = -1.0;
	  }
	else
	  {
	    a[1] = abs2(a[1]); // /0.96;
	    a[2] = abs2(a[2]);
	    a[3] = a[3];
	    
	    a[0] = chisq;
	  }

}

void fit_phase(int channel, double *data, int centerPos, int fit_window, float *phase, float *freq)
{
  int i, maxi;
  double max, amp;
  int nn[3];
  fftw_complex nj[fit_window], Nk[fit_window];
  fftw_plan plan;


  nn[0] = fit_window;
  nn[1] = 2;
  nn[2] = 0;

  if (((centerPos-fit_window/2) > 0) && ((centerPos+fit_window/2) < nPtsScan))
    {      
      for (i=0; i<fit_window; i++)
	{
	  nj[i].re = data[centerPos+i-fit_window/2];
	  nj[i].im = 0;
	}

      //*** Forward transformation
      plan = fftw_create_plan(fit_window, FFTW_FORWARD, FFTW_ESTIMATE);
      fftw_one(plan, nj, Nk);
      fftw_destroy_plan(plan);
     
      //find peak
      max = -1.0e10;
      for (i=0; i<fit_window/2; i++)
	{
	  amp = sqrt(Nk[i].re*Nk[i].re + Nk[i].im*Nk[i].im);
	 
	  if (amp > max)
	    {
	      max = amp;
	      maxi = i;
	    }
	}

      *freq = maxi;
      *phase = atan2(Nk[maxi].im, Nk[maxi].re);

    }
}



/* GAUSSIAN FIT */


void func_gauss(float x, float a[], float *y, float dyda[], int na)
{
  float fac, ex, arg;

  //GAUSSIAN FIT
  arg=(x-a[3])/a[2];
  ex=exp(-arg*arg);
  fac=a[1]*ex*2.0*arg;
  *y = a[1]*ex;
  dyda[1]=ex;
  dyda[3]=fac/a[2];
  dyda[2]=fac*arg/a[2];
}


double func__gauss(float x, float a[])
{
  return a[1]*exp(-((x-a[3])/a[2])*((x-a[3])/a[2]));
}



void fit_gauss(double *envelope, int points, double *amp, double *wid, double *pos, double *fitquality)
{
    const int NPT = 100;
    const int MA = 3;

    int i,*ia,iter,itst,k,mfit=MA;
    float alamda,chisq,ochisq,*x,*y,*sig,**covar,**alpha;
    float extremmin, extremmax, extremmaxx;
    float xmin, xmax, ymin, ymax;
    float a[MA+1];
    float gues[MA+1];

    gues[0] = 0.0;
    gues[1] = 0.001;
    gues[2] = 1000.0;
    gues[3] = 4000.0;
    
	ia=ivector(1,MA);
	x=vector(1,points);
	y=vector(1,points);
	sig=vector(1,points);
	covar=matrix(1,MA,1,MA);
	alpha=matrix(1,MA,1,MA);


	errormsg = 0;


	extremmin = 1;
	extremmax = 0;
        for (i=0;i<points;i++)
	  {
	      x[i] = i;
	      y[i] = envelope[i];
	      if (abs2(y[points]) < extremmin) extremmin = abs2(y[points]);
	      if (abs2(y[points]) > extremmax) 
		{
		  extremmax = abs2(y[points]);
		  extremmaxx = points;
		}
	      sig[i] = 1;
	  }

	gues[3] = extremmaxx;

	//substract offset
        for (i=1;i<=points;i++)
	  y[i] -= extremmin;


	for (i=1;i<=mfit;i++) ia[i]=1;
	for (i=1;i<=MA;i++) a[i]=gues[i];

	for (iter=1;iter<=2;iter++) {
		alamda = -1;
		mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func_gauss,&alamda);
		k=1;
		itst=0;
		for (;;) {
			k++;
			ochisq=chisq;
			mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func_gauss,&alamda);
			if (chisq > ochisq)
				itst=0;
			else if (fabs(ochisq-chisq) < 0.1)
				itst++;
	                if (itst < 4) continue;
			alamda=0.0;
			mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func_gauss,&alamda);

			break;
		}
	}


	if (doplot2 == 1)
	  {
	    xmin = 1.0e10;
	    xmax = -1.0e10;
	    ymin = 1.0e10;
	    ymax = -1.0e10;

	    for (i=1;i<=points;i++)
	      {
		printf("(%f  %f)", x[i], y[i]);

		if (x[i] < xmin) xmin = x[i];
		if (x[i] > xmax) xmax = x[i];
		if (y[i] < ymin) ymin = y[i];
		if (y[i] > ymax) ymax = y[i];
	      }

	    cpgline(points, x, y);

	    cpgsci(1);
	    cpgenv(-1.0, nPtsScan, xmin, xmax, ymin, ymax);
	  }	      
	    

	free_matrix(alpha,1,MA,1,MA);
	free_matrix(covar,1,MA,1,MA);
	free_vector(sig,1,nPtsScan);
	free_vector(y,1,nPtsScan);
	free_vector(x,1,nPtsScan);
	free_ivector(ia,1,MA);

	if (abs2(a[3]) < 0) errormsg = 1;
	if ((a[1] == gues[1]) && (a[2] == gues[2])) errormsg = 1;


	if (errormsg != 0)
	  {
	    *amp = -1.0;
	    *wid = 0.0;
	    *pos = -1.0;

	    *fitquality = -1.0;
	  }
	else
	  {
	    *amp = abs2(a[1]);
	    *wid = a[2];
	    *pos = a[3];
	    
	    *fitquality = chisq;
	  }

}


void func_gauss2(float x, float a[], float *y, float dyda[], int na)
{
  float fac, ex, arg;

  //GAUSSIAN FIT
  arg=(x-a[3])/a[2];
  ex=exp(-arg*arg);
  fac=a[1]*ex*2.0*arg;
  *y = a[1]*ex;
  dyda[1]=ex;
  dyda[3]=fac/a[2];
  dyda[2]=fac*arg/a[2];
}


double func__gauss2(float x, float a[])
{
  return (a[1]*exp(-((x-a[3])/a[2])*((x-a[3])/a[2]))+a[4]);
}


void fit_gauss2(int channel, double *data, int points, float *result)
{
    const int NPT = 100;
    const int MA = 3;


	int i,*ia,iter,itst,k,mfit=MA;
	float alamda,chisq,ochisq,*x,*y,*sig,**covar,**alpha;
	float extremmin, extremmax, extremmaxx;
	float minleft, minright, maxright;
	float a[MA+1];
	float gues[MA+1];
	FILE *fp, *stream;
	float plotx[nPtsScan], ploty[nPtsScan];
	float expected_pos, window, area;

	gues[0] = 0.0;
	gues[1] = 1.0;
	gues[2] = 2.0;
	gues[3] = 30.0;
	

	if (channel == 2)
	  expected_pos = 60;
	else
	  expected_pos = 30;

	window = 20;


	ia=ivector(1,MA);
	x=vector(1,points);
	y=vector(1,points);
	sig=vector(1,points);
	covar=matrix(1,MA,1,MA);
	alpha=matrix(1,MA,1,MA);


	errormsg = 0;

	minleft = 1.0e10;
	minright = 1.0e10;
	maxright = 0.0;
	for (i=0;i<points;i++)
	  {
	    if ((i <= 20) && (i > 4) && (data[i] < minleft)) minleft = data[i];
	    if ((i >= points-20) && (data[i] < minright)) minright = data[i];
	    if ((i >= points-20) && (data[i] > maxright)) maxright = data[i];
	  }
	minright = (maxright+minright)/2.0;
	
	points = -1;
        for (i=0;i<nPtsScan/2;i++)
        //if ((i > expected_pos-window) && (i < expected_pos+window))
	{
	    points++;
	    x[points] = points;
	    y[points] = data[i]-(minleft+((minright-minleft)*i/(nPtsScan/2-1)));
	    sig[points] = 1.0;
	}


	extremmax = 0.0;
	extremmin = 0.0;
        for (i=20;i<points;i++)
	{
	    if (y[i] < extremmin) extremmin = y[i];
	    
	    if (abs2(y[i]) > extremmax) 
	    {
		extremmax = abs2(y[i]);
		extremmaxx = i;
	    }
	}

	gues[1] = extremmax-extremmin;
	gues[3] = extremmaxx;
	//gues[4] = 0.0;

	for (i=1;i<=mfit;i++) ia[i]=1;
	for (i=1;i<=MA;i++) a[i]=gues[i];

	//printf("GUESS: Ampl: %f  Width: %f  Pos: %f  Offs: %f\n", gues[1], gues[2], gues[3], gues[4]);

	for (iter=1;iter<=2;iter++) {
		alamda = -1;
		mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func_gauss2,&alamda);
		k=1;
		itst=0;
		for (;;) {
			k++;
			ochisq=chisq;
			mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func_gauss2,&alamda);
			if (chisq > ochisq)
				itst=0;
			else if (fabs(ochisq-chisq) < 0.1)
				itst++;
	                if (itst < 4) continue;
			alamda=0.0;
			mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func_gauss2,&alamda);

			break;
		}
	}

	if (abs2(a[1]) <= 0.0) errormsg = 1;
	if (abs2(a[3]) < 20.0) errormsg = 1;
	if ((a[1] == gues[1]) && (a[2] == gues[2]) && (a[3] == gues[3])) errormsg = 1;


	if (errormsg != 0)
	  {
	    result[1] = -1.0;
	    result[2] = 0.0;
	    result[3] = -1.0;
	    result[4] = -1.0;

	    result[0] = -1.0;
	  }
	else
	  {
	    if (doplot2 == 1)
	      {
		if (repfiles == 1)
		{
		    strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "powspecfit0.dat");
		    if (channel == 0) stream = fopen(ProcFilename, "w");

		    strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "powspecfit1.dat");
		    if (channel == 1) stream = fopen(ProcFilename, "w");

		    strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "powspecfit2.dat");
		    if (channel == 2) stream = fopen(ProcFilename, "w");
		}

		cpgsci(8);
		for (i=0; i<=nPtsScan/2; i++)
		  {
		    plotx[i] = i;
		    
		    ploty[i] = func__gauss2(i, a)+(minleft+((minright-minleft)*i/(nPtsScan/2-1)));

		    if (repfiles == 1) fprintf(stream,"%f %f\n", (float)i/((float)nPtsScan/2.0)*(FFT_maxfreq-FFT_minfreq) + FFT_minfreq, ploty[i]);
		  }

		if (repfiles == 1) fclose(stream);
		cpgline(nPtsScan/2-1, plotx, ploty);
	      }

	    area = 0.0;

	    for (i=0; i<=nPtsScan/2; i++)
	      area += func__gauss2(i, a);

	    result[1] = a[1];
	    result[2] = a[2];
	    result[3] = a[3];
	    result[4] = area;
	    
	    result[0] = chisq;
	  }
}


#undef MA


//FUNC: exp(f*x^2)-1

#define MA1 1

void func_exp_x2(float x, float a[], float *y, float dyda[], int na)
{
    *y = exp(a[1]*x*x) - 1.0;
    dyda[1] = x*x*exp(a[1]*x*x);
}

double func__exp_x2(float x, float a[])
{
  return (exp(a[1]*x*x)-1.0);
}


void fit_exp_x2(double *data1, double *data2, int points, double *param, double *fitquality)
{
    FILE *fp;
    int i,*ia,iter,itst,k,mfit=MA1;
    float alamda,chisq,ochisq,*x,*y,*sig,**covar,**alpha;
    float xmin, xmax, ymin, ymax, x_;
    static float a[MA1+1];
    static float gues[MA1+1]={0.0, 1.0};

    ia=ivector(1,MA1);
    x=vector(1,points);
    y=vector(1,points);
    sig=vector(1,points);
    covar=matrix(1,MA1,1,MA1);
    alpha=matrix(1,MA1,1,MA1);

    errormsg = 0;

    for (i=0;i<points;i++)
    {
	x[i+1] = 1.0-data1[i];
	y[i+1] = data2[i];
	sig[i+1] = 1.0;
	
	//printf("Point: %i   %f  %f\n", i, (float)x[i], (float)y[i]);
    }

    for (i=1;i<=mfit;i++) ia[i]=1;
    for (i=1;i<=MA1;i++) a[i]=gues[i];


    for (iter=1;iter<=2;iter++) {
	alamda = -1;
	mrqmin(x,y,sig,points,a,ia,MA1,covar,alpha,&chisq,func_exp_x2,&alamda);
	k=1;
	itst=0;
	for (;;) {
			k++;
			ochisq=chisq;
			mrqmin(x,y,sig,points,a,ia,MA1,covar,alpha,&chisq,func_exp_x2,&alamda);
			if (chisq > ochisq)
				itst=0;
			else if (fabs(ochisq-chisq) < 0.1)
				itst++;
	                if (itst < 2) continue;
			alamda=0.0;
			mrqmin(x,y,sig,points,a,ia,MA1,covar,alpha,&chisq,func_exp_x2,&alamda);

			break;
		}
	}


	//Recompose the function	
        strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "simbias3.dat");
	fp = fopen(ProcFilename,"w");
	
	for (x_=0.0; x_<=1.0; x_ += 0.01)
	  fprintf(fp, "%f %f\n", x_, func__exp_x2(1.0-x_, a));
	
	fclose(fp);

	if (doplot2 == 1)
	  {
	    xmin = 1.0e10;
	    xmax = -1.0e10;
	    ymin = 1.0e10;
	    ymax = -1.0e10;

	    for (i=1;i<=points;i++)
	      {
		printf("(%f  %f)", x[i], y[i]);

		if (x[i] < xmin) xmin = x[i];
		if (x[i] > xmax) xmax = x[i];
		if (y[i] < ymin) ymin = y[i];
		if (y[i] > ymax) ymax = y[i];
	      }

	    cpgline(points, x, y);

	    cpgsci(1);
	    cpgenv(-1.0, nPtsScan, xmin, xmax, ymin, ymax);
	  }	      
	    

	free_matrix(alpha,1,MA1,1,MA1);
	free_matrix(covar,1,MA1,1,MA1);
	free_vector(sig,1,nPtsScan);
	free_vector(y,1,nPtsScan);
	free_vector(x,1,nPtsScan);
	free_ivector(ia,1,MA1);

	*param = a[1];
	*fitquality = chisq;
}

#undef MA1
