/* fitfuncs.c                                */
/* Interferometry Data Reduction Software    */
/* by Stefan Kraus                           */
/* 2002-05                                   */
/*                                           */
/* copyright laws apply                      */


int errormsg;

float abs2(float x)
{
  if (x < 0)
    return -x;
  else
    return x;
}



void gaussj(float **a, int n, float **b, int m)
{
	int *indxc,*indxr,*ipiv;
	int i,icol,irow,j,k,l,ll;
	float big,dum,pivinv,temp;

	indxc=ivector(1,n);
	indxr=ivector(1,n);
	ipiv=ivector(1,n);
	for (j=1;j<=n;j++) ipiv[j]=0;
	for (i=1;i<=n;i++) 
	{
		big=0.0;
		for (j=1;j<=n;j++)
			if (ipiv[j] != 1)
				for (k=1;k<=n;k++) 
				{
					if (ipiv[k] == 0) 
					{
						if (fabs(a[j][k]) >= big) 
						{
							big=fabs(a[j][k]);
							irow=j;
							icol=k;
						}
					}
					else
					{
					  if (ipiv[k] > 1)
					    {
//printf("gaussj: Singular Matrix-1\n");
					      //nrerror("gaussj: Singular Matrix-1");
					    }
					}
				}
		
		++(ipiv[icol]);
		if (irow != icol) 
		{
			for (l=1;l<=n;l++)
			  {
			    temp = a[irow][l];
			    a[irow][l] = a[icol][l];
			    a[icol][l] = temp;
			  }
			for (l=1;l<=m;l++)
			  {
			    temp = b[irow][l];
			    b[irow][l] = b[icol][l];
			    b[icol][l] = temp;
			  }
		}
		indxr[i]=irow;
		indxc[i]=icol;
		if (a[icol][icol] == 0.0) 
		  {
		    errormsg = 1;
//printf("gaussj: Singular Matrix-2\n");
		    //nrerror("gaussj: Singular Matrix-2");
		  }
		pivinv=1.0/a[icol][icol];
		a[icol][icol]=1.0;
		for (l=1;l<=n;l++) a[icol][l] *= pivinv;
		for (l=1;l<=m;l++) b[icol][l] *= pivinv;
		for (ll=1;ll<=n;ll++)
			if (ll != icol) {
				dum=a[ll][icol];
				a[ll][icol]=0.0;
				for (l=1;l<=n;l++) a[ll][l] -= a[icol][l]*dum;
				for (l=1;l<=m;l++) b[ll][l] -= b[icol][l]*dum;
			}
	}

	for (l=n;l>=1;l--) 
	{
		if (indxr[l] != indxc[l])
			for (k=1;k<=n;k++)
			  {
			    temp = a[k][indxr[l]];
			    a[k][indxr[l]] = a[k][indxc[l]];
			    a[k][indxc[l]] = temp;
			  }
	}
	free_ivector(ipiv,1,n);
	free_ivector(indxr,1,n);
	free_ivector(indxc,1,n);
}


void covsrt(float **covar, int ma, int ia[], int mfit)
{
	int i,j,k;
	float swap;
	float temp;

	for (i=mfit+1;i<=ma;i++)
		for (j=1;j<=i;j++) covar[i][j]=covar[j][i]=0.0;
	k=mfit;
	for (j=ma;j>=1;j--) {
		if (ia[j]) {
			for (i=1;i<=ma;i++)
			  {
			    temp = covar[i][j];
			    covar[i][j] = covar[i][k];
			    covar[i][k] = temp;
			  }
			for (i=1;i<=ma;i++)
			  {
			    temp = covar[j][i];
			    covar[j][i] = covar[k][i];
			    covar[k][i] = temp;
			  }
			k--;
		}
	}
}

void mrqcof(float x[], float y[], float sig[], int ndata, float a[], int ia[],
	int ma, float **alpha, float beta[], float *chisq,
	void (*mrqcoffuncs)(float, float [], float *, float [], int))
{
	int i,j,k,l,m,mfit=0;
	float ymod,wt,sig2i,dy,*dyda;

	dyda=vector(1,ma);
	for (j=1;j<=ma;j++)
		if (ia[j]) mfit++;
	for (j=1;j<=mfit;j++) {
		for (k=1;k<=j;k++) alpha[j][k]=0.0;
		beta[j]=0.0;
	}
	*chisq=0.0;
//printf("MRQCOF: a = %f   %f\n", a[1], a[2]);
	for (i=1;i<=ndata;i++) {
		(*mrqcoffuncs)(x[i],a,&ymod,dyda,ma);
		sig2i=1.0/(sig[i]*sig[i]);
		dy=y[i]-ymod;
		for (j=0,l=1;l<=ma;l++) {
			if (ia[l]) {
				wt=dyda[l]*sig2i;
				for (j++,k=0,m=1;m<=l;m++)
					if (ia[m]) alpha[j][++k] += wt*dyda[m];
				beta[j] += dy*wt;
			}
		}
		*chisq += dy*dy*sig2i;
	}
	for (j=2;j<=mfit;j++)
		for (k=1;k<j;k++) alpha[k][j]=alpha[j][k];
	free_vector(dyda,1,ma);
//printf("MRQCOFEND: a = %f   %f\n", a[1], a[2]);
}


void mrqmin(float x[], float y[], float sig[], int ndata, float a[], int ia[],
	int ma, float **covar, float **alpha, float *chisq, 
	void (*mrqminfuncs)(float, float [], float *, float [], int),
        float *alamda)
{
	int j,k,l,m;
	static int mfit;
	static float ochisq,*atry,*beta,*da,**oneda;

//printf("MRQMIN: a = %f   %f\n", a[1], a[2]);

	if (*alamda < 0.0) {
		atry=vector(1,ma);
		beta=vector(1,ma);
		da=vector(1,ma);
		for (mfit=0,j=1;j<=ma;j++)
			if (ia[j]) mfit++;
		oneda=matrix(1,mfit,1,1);
		*alamda=0.001;
		mrqcof(x,y,sig,ndata,a,ia,ma,alpha,beta,chisq,mrqminfuncs);
		ochisq=(*chisq);
		for (j=1;j<=ma;j++) atry[j]=a[j];
	}
//printf("MRQMIN-START1\n");
	for (j=0,l=1;l<=ma;l++) {
		if (ia[l]) {
			for (j++,k=0,m=1;m<=ma;m++) {
				if (ia[m]) {
					k++;
					covar[j][k]=alpha[j][k];
				}
			}
			covar[j][j]=alpha[j][j]*(1.0+(*alamda));
//printf("MRQMIN-START2\n");
			oneda[j][1]=beta[j];
//printf("MRQMIN-START3\n");
		}
	}
//printf("MRQMIN-MIDDLE %i\n",mfit);
	gaussj(covar,mfit,oneda,1);
//printf("MRQMIN-MIDDLE1\n");
	for (j=1;j<=mfit;j++) da[j]=oneda[j][1];
//printf("MRQMIN-MIDDLE2\n");
	if (*alamda == 0.0) {
//printf("MRQMIN-MIDDLE2a\n");
		covsrt(covar,ma,ia,mfit);
//printf("MRQMIN-MIDDLE2b %f %i\n", oneda, mfit);
		free_matrix(oneda,1,mfit,1,1);
//printf("MRQMIN-MIDDLE2c\n");
		free_vector(da,1,ma);
		free_vector(beta,1,ma);
		free_vector(atry,1,ma);
		return;
	}
//printf("MRQMIN-MIDDLE3\n");
	for (j=0,l=1;l<=ma;l++)
		if (ia[l]) atry[l]=a[l]+da[++j];
//printf("MRQMIN-MIDDLE4\n");
	mrqcof(x,y,sig,ndata,atry,ia,ma,covar,da,chisq,mrqminfuncs);
	if (*chisq < ochisq) {
		*alamda *= 0.1;
		ochisq=(*chisq);
		for (j=0,l=1;l<=ma;l++) {
			if (ia[l]) {
				for (j++,k=0,m=1;m<=ma;m++) {
					if (ia[m]) {
						k++;
						alpha[j][k]=covar[j][k];
					}
				}
				beta[j]=da[j];
				a[l]=atry[l];
			}
		}
	} else {
		*alamda *= 10.0;
		*chisq=ochisq;
	}
//printf("MRQMINEND: a = %f   %f\n", a[1], a[2]);
}


float sign(float x)
{
  if (x < 0)
    return -1.0;
  else
    return 1.0;
}

