/* idrs.c                                    */
/* Interferometry Data Reduction Software    */
/* by Stefan Kraus                           */
/* 2002-05                                   */
/*                                           */
/* copyright laws apply                      */


/* Configuration files:                      */
/*   idrs.ini  (Lists the files to process)  */
/*   calib.ini (Lists the calib, targets     */
/*              and diameters for            */
/*              stellar readius estimation)  */
/*   objid.ini (Translates the IOTA-StarID   */
/*            into an internal ID-number)    */
/*   idrs.cfg                                */

// order visibilities, uv-coordinates: baselines AB, AC, BC


/* Attention: PowSpec doesn't work properly  */
/* free_dvector(y_raw_double2,...) caused segmentation faults */


//*** GENERAL SETTINGS
int repdetails;              //ReportDetailsScreen
#define repfit            0  //writes fit-details raw-fringes on disc
#define repfiles          0  //writes details into files: matrix.dat, powspec.dat, powspecavg.dat
int fitampmethod;            //VisEstimationMethod, method to fit the fringe-amplitude: 0 = via envelope-fitting, 1 = via powerspectrum, 2 = via wavelet
#define simfringe         0  //simulate fringes
#define dobiascorr        0  //1 = simple bias correction, 2 = corrbias-file correction
int Viscalibmethod;          //VisCalibrationMethod
int CPcalibmethod;           //CPCalibrationMethod
int CPfreqclosureThreshold;  //CPFreqClosureThreshold
int doplot0;                 //Plot0
int doplot1;                 //Plot1
int doplot2;                 //Plot2
int doplot3;                 //Plot3
int doplotbg;                //Plotbg
int doplotwait;              //PlotWait
int addcaliberror;           //VisErrorAddCalibrationError
int adddelayerror;
float adderrorVis;           //VisErrorAddSys
float adderrorCP;            //CPErrorAddSys

char ProcDir[300], ProcFilename[300];

int cpg_window0, cpg_window1, cpg_window2, cpg_window3, cpg_windowbg;
#define PGPLOT_window  "/xwindow"
#define PS1_window  "idrs1.ps/CPS"
#define PS2_window  "idrs2.ps/CPS"
#define PS3_window  "idrs3.ps/CPS"
#define PSBG_window  "idrs.bg.ps/CPS"
#define PSPC_window  "idrs.pc.ps/CPS"

int autogetobjid = 1;
int targetid;
int channel_fastfringe;
double fringefreq[6];

#define createbiascorr                0   //necessary: simfringe=1; dophotcorr=0
#define simfringe_N_RUNSmax           200
#define createbiascorr_interv_amp     0.05
#define createbiascorr_interv_noise   0.05

#define bininterval                   0.01 //[min]

//PHOTOMETRIC CORRECTION
int dobackgroundsub;            //VisSubBackground

int dophotcorr;                 //VisPhotometricCorrection
float photcorrsmoothfactor1;    //VisPhotometricCorrectionSmooth1
float photcorrsmoothfactor2;    //VisPhotometricCorrectionSmooth2
float bgsubsmoothfactor;        //VisSubBackgroundSmooth

//FILTER GROUPS
int dofilteramp;                //VisFilterAmp
float filterampthreshold;       //VisFilterAmpThreshold

int dofilterscat;               //VisFilterScatteringGroups
float filtervisscatthreshold;   //VisFilterScatteringGroups
#define filterover1vis   100000.0   //filters all targetid-groups with have more than this threshold of measurements above vis=1.0
float filtercperror;            //CPFilterLargeErrorsThreshold
float filterviserror;           //VisFilterLargeErrorsThreshold

char NameScan[300];


#define minvisnumber                 10   //minimal number of visibility estimation within a data file to use this data file for averaging

  

//*** FIT-PARAMETERS
//* CWT
float cwt_filter_threshold_power;          //CWTPowerThreshold
float cwt_filter_threshold_filter;         //CWTPowerThreshold
#define algCWT                       0     //0: fast CWT using FFT,  1: conventional CWT
#define avgCWT                       0     //1: produce averaged CWT output files
#define cwt_narrowband               0     //1: narrow-band data, 0=broad bandwidth filter

int cwt_subtract_background;               //CWTSubBackground
int cwt_background_border;                 //CWTBackgroundBorder

int cwt_threshold_area_min;                //CWTThresholdAreaMin
int cwt_threshold_area_max;                //CWTThresholdAreaMax
int cwt_threshold_pos_intpeak_spatial;     //CWTThresholdIntPeakSpatial
int cwt_threshold_pos_intpeak_freq;        //CWTThresholdIntPeakScale
int cwt_threshold_size_peak_freq01;        //CWTThresholdSizePeakScaleCh0
int cwt_threshold_size_peak_freq2;         //CWTThresholdSizePeakScaleCh1Ch2
float cwt_threshold_SignalToNoise;         //CWTThresholdSignal2Noise

float VisFringeTrackerSelection = 0;       //0.4 original: 2.0
float FringeTrackerthresholdRatio = 0.4;   //0.4 original: 2.0

#define avg_range_low         0.5
#define avg_range_high        1.5


//* Envelope Fitting algorithm
#define fitviskde    1          //1=uses KDE to get the average amplitude and width
                                //0=uses mean and root variance 
#define fitenv_fixwid0  51.5
#define fitenv_fixwid1  56.0
#define fitenv_fixwid2  30.0

#define fitenv_max_vis  1.05    //the maximal value for the uncorrected amplitude to accept

#define fit_window_slow   220   //window-size to fit envelope for the slow fringe
#define fit_window_fast   110   //window-size to fit envelope for the fast fringe
int CPWindowWidth=32;


//* Powerspectrum algorithm
int powmaxrange=10;             //window-size for power-spectrum amp. estimation; that's the range to each size: LamVir: 8
#define powexpfreq        30    //expected frequency of the fringe peak
#define powexprange       2     //allowed deviation of the expected frequency of this amount



//* Matrix File Reduction
#define matrixscat_threshold    4.0    //threshold for the ratio between the flux in different matrixfiles
#define matrix_threshold        2.0    //threshold to avoid that a lost star-matrixfiles will be used for intensity estimation
#define OPD_slowfringe          50.0   //optical path delay for the slow fringe = 1/2*fast fringe [mu m]
#define FFT_minfreq             0.1256 //frequency of the left side in the power spectrum [1/(mu m)]=2*PI/50mu m
#define FFT_maxfreq             16.085 //frequency of the left side in the power spectrum [1/(mu m)]=2*PI/(50mu m) * (256/2)



//*** DIMENSIONS
#define max_nights       14
#define max_calibsets    200   //max. number of object-observation-sets

#define nPtsScan         256
#define nPtsWavelets     128   //nr of scale-points in the wavelet
#define N_RUNSmax        500


#define wavelength_bandwidth   0.3 //micron; for fits-oi header

#define model_binary    1    //for model.c: 0=single star, 1=binary star

#include <cpgplot.h>
#include <stdlib.h>

#include "nrutil_.c"
#include "nrutil.c"
#include "sort.c"
#include "trackerdefines.h"
#include "general.c"
#include <math.h>
#include <stdio.h>
#include "string.h"
#include "mathfunctions.c"
#include "fit.c"
#include "model.c"
#include "convert.c"
#include <time.h>
#include "trackerfunctions.c"
#include "ptracker_ed.h"
#include "idrslib.c"
#include "sepcolors.c"
#include "calib.c"
#include "kdeinc.c"
#include "kstinc.c"
#include "powspec.c"
#include <dfftw.h>
#include "wavelet.c"
#include "histo.c"


#define N_RUNSmax      500

// SCRIPT: [Command 0=single files, 1=adding] [FileDir] [FileFrom] [FileTo] [ScanFrom] [ScanTo]


void filter_envelope(double *data, double *envelope, int nrdata, int topdown)
{
  int i;

  for (i=0; i<nrdata; i++)
    envelope[i] = 0;

  for (i=2; i<nrdata-2; i++)
    {
      //if (topdown == 1)
      //{
      if ( (data[i-1] < data[i]) && (data[i+1] < data[i]) &&
	   (data[i-2] < data[i-1]) && (data[i+2] < data[i-1]) &&
	   (data[i] > 0) )
	envelope[i] = data[i];
      //}
      //else
      //{
      if ( (data[i-1] >= data[i]) && (data[i+1] >= data[i]) &&
	   (data[i-2] >= data[i-1]) && (data[i+2] >= data[i-1]) &&
	   (data[i] < 0) )
	envelope[i] = abs2(data[i]);
      //}
    }
}


int main(int argc, char *argv[])
{
  int i, m, i_run, i_run2;
  int packetFound[3];
  int nDFTSmooth = 11; 
  int nDFT; 
  int nFringe = 40; 
  int nFringe2 = 15;
  
  int useSymmetry = 1; 
  int templateType = 2; 
  int samplesPerFringePeriod = 8; 
  int nPeriodsInDFTWindow = 1; 

  float *y_raw; 
  double *y_raw_double0, *y_raw_double1, *y_raw_double2, *envelope;
  double *filtered0, *filtered1, *filtered2;
  double *power0, *power1, *power2;
  double *poweravg0, *poweravg1, *poweravg2; 
  double *poweravg0bg, *poweravg1bg, *poweravg2bg; 
  int powspecavgcount[3];
  double powsum[3], powsumbg[3], powmax[3], powsum1, powsum2, powminleft, powminright;
  int powfreq;
  int powmaxpos[3];
  float weight[3]; 
  float centerBin[3], centerPhase[3]; 
  float centerBincp;
  double closurePhase[N_RUNSmax];
  float cpmean[max_nights], cpscat[max_nights], mean, variance, mean1, mean2, variance1, variance2;
  double jd;
  double wavelength;
  double fitamp[3], fitampsqr[3], fitwid[3], fitpos[3], fitquality[3], fitprob[3], waveletamp;
  double **corr, **matrix4, *matrixavg4;
  double sky[6];
  double singlescan[6];
  double tel0baseline[3], tel1baseline[3], tel2baseline[3];
  FILE *stream[3], *stream2, *streamscript, *streamamp, *streamamp2, *streamphotidx, *streamraw, *streamlog, *streamsre, *streamampcorr, *streamvis, *streamvis2, *streamvis3, *streamvis4, *streambias, *streamborders, *streamkdeamp, *streamkdewid, *streamsim, *streambaseline, *streaminfo, *streambin, *streamdelay, *streamtransf;
  FILE *streammatrix, *streamrepcorr, *streampow, *streamcp, *streammedian;
  int usescan;
  
  double matrix1level[6], matrix2level[6], matrix3level[6];
  float corrfactor[3];
  
  int scriptdirnr, scriptnr, scriptnrall, scriptnrall2, scriptnrall2proc, scriptmin, scriptmax, j, N_RUNS, fixeddelay, calibfile, scriptscanfrom, scriptscanto, scanfrom, scanto, redstepfrom, redstepto, redstepobs, scriptnrallfound;
  int objid, channel, withinlimits[3], refused_outoflimits[3], refused_nofringe[3];
  int fitsuccess;
  float param[4], param2[4], cpphase[3], cpfreq[3], parampowspec[5];
  long int scannr, fitsuccessnr, fiterrornr, fitsuccessnrchannel[3], fitprobequal0[3], fitprobnot0[3];
  char DataDir[300], text_[300], filename[300], filename2[300], filename3[300], filename4[300], scriptnrtext[10], filenameamp[300], channeltext[10], char_;
  double *outkde, scalemin, scalemax, ustart[3], uend[3], vstart[3], vend[3], umin, umax, vmin, vmax;
  double biaslog, biaslog2;
  float parambias, noiseparambias;
  int biascount;
  float uavg[3], vavg[3], sreamp[3], sreamperror[3], sreampprob[3];
  char  temptext[300], temptext2[300], temptext3[300], temptext4[300];
  char  scriptfile[300], configfile[300];
  float kernelwidth;
  int redstep;
  float vis[3], viserror[3], avg_calib[3][max_nights], avg_caliberror[3][max_nights];
  float newjd, oldjd;
  int caliblist[max_targets], srelist[max_targets], is_target, ch_baseline[6];
  float nr_calibs[3], target_diameter[max_targets];
  int nr_night, n, nr_target;
  float basel_min, basel_max;
  float diameter, diametererror, b, v, leastsqr;
  float calibtag=0, biascorr[3], biascorr_norm[3];
  int biascorrcount[3];
  
  float vispoints[max_targets][max_vispoints][3];
  int vispoints_nr[max_targets], plot_target;
  float calibset_vis[3][max_calibsets], calibset_visscat[3][max_calibsets], calibset_jd[max_calibsets], calibset_cp[max_calibsets], calibset_cpscat[max_calibsets], calibset_nr[3][max_calibsets];
  int calibset_from[max_calibsets], calibset_to[max_calibsets];
  int calibset_objid[max_calibsets], calibset_night[max_calibsets];
  int nr_calibset;
  
  float calib_level, calib_level_l, calib_level_r, calib_level_ljd, calib_level_rjd;
  float calib_level3[3];
  int found, splitdatafile, splitdatafile2, splitdatafrom, splitdatato;
  char outpraw_dir[300];
  int outpraw_filenr, outpraw_scannr;
  int tempint, tempint2;
  long templong;
  
  float plotx[nPtsScan], ploty[nPtsScan], ploty2[nPtsScan], ymin, ymax;
  float wavelet[npad][nPtsWavelets];
  float avgwavelet0[npad][nPtsWavelets];
  float avgwavelet1[npad][nPtsWavelets];
  float avgwavelet2[npad][nPtsWavelets];
  float *fitampdata;
  float sum, sumsqr;
  float errorplus, errorminus;
  fftw_plan cwt_p, cwt_p2;
  fftw_plan fftw_photcorr_f, fftw_photcorr_b;


  float binV2[3], binV2err[3], binCP, binCPerr, binuavg[3], binvavg[3], binJD, oldobjid, startjd, w[3];
  int count, binscriptnrall;
  double cpdata[100], cpdataerr[100];
  double delaymjd;
  float delayu, delayv;
  double delaydiff, delaydiffmin;
  float delay, delaymin, delaymax, delayumin, delayvmin;
  float delaymin_channel[3];

  float sepRA, sepDEC;

  int expfreq, exprange;
  int powmaxrangechannel;

  double cpcalibrator;
  float cpfreq_threshold;


  adddelayerror = 1;
  corrfactor[0]=0.0;
  corrfactor[1]=0.0;
  corrfactor[2]=0.0;

  
  if (argc <= 1)
  {
      printf("Interferometry Data Reduction Software\n(c) 2002-05 by Stefan Kraus\n    at  University of Massachusetts at Amherst, Amherst MA 01002, USA\n    and Max-Planck-Institute for Radioastronomy (MPIfR), Auf dem Huegel 69, D-53121 Bonn, Germany\n\nUsage: idrs [option]\noption:\n    ''all''=performs all data reduction steps\n    ''vis''=starts with visibility estimation (skips processing matrixfiles)\n    ''cp''=starts with closure phase estimation (skips visibility fitting)\n    ''filter''=starts with filtering and makes the final visibility/CP calibration\n    ''final ([objid])''=final calibration ([objid] for stellar radius estimation)\n    ''-i [filename]''=declaring IniFile (default: idrs.ini)\n    ''splitdata [dirfrom] [dirto] [fileno. from] [fileno. to]'' (Splits data files in 50:50; Headerfiles are not modified!)\n\n");
      return 0;
  }
  
  found = 0;
  for (i=1; i<argc; i++) if (strcasecmp(argv[i], "-c") == 0) found = i;
  if (found >= 1)
      strcpy(configfile, argv[found+1]);
  else 
      strcpy(configfile, "idrs.cfg");

  if (read_ConfigFile(configfile) == 0) return 0;


  //*** Check Parameters
  //SRE
  found = 0;
  for (i=1; i<argc; i++) if (strcasecmp(argv[i], "final") == 0) found = i;
  if (found >= 1)
    {
      if (argc-1 > found)
	plot_target = atoi(argv[found+1]);
      else
	plot_target = 0;
    }

  //SCRIPTFILE
  found = 0;
  for (i=1; i<argc; i++) if (strcasecmp(argv[i], "-i") == 0) found = i;
  if (found >= 1)
    strcpy(scriptfile, argv[found+1]);
  else 
    strcpy(scriptfile, "idrs.ini");
  
  
  //PGPLOT: open xwindow
  if (doplot0 > 0)
    {
      if (doplot0 == 1)
	  cpg_window0 = cpgopen(PGPLOT_window);
      else
	  cpg_window0 = cpgopen(PSPC_window);
    }

  if (doplot1 > 0)
  {
      if (doplot1 == 1)
	  cpg_window1 = cpgopen(PGPLOT_window);
      else
	  cpg_window1 = cpgopen(PS1_window);
      cpgask(doplotwait);
      cpgsubp(2, 3);
  }
  
  if (doplot2 > 0)
  {
      if (doplot2 == 1)
	  cpg_window2 = cpgopen(PGPLOT_window);
      else
	  cpg_window2 = cpgopen(PS2_window);
      cpgask(doplotwait);
      cpgsubp(1, 3);
  }
  
  if (doplot3 > 0)
  {
      if (doplot3 == 1)
	  cpg_window3 = cpgopen(PGPLOT_window);
      else
	  cpg_window3 = cpgopen(PS3_window);
      cpgask(0);
      cpgsubp(1, 3);
  }
  
  if (doplotbg > 0)
  {
      if (doplotbg == 1)
	  cpg_windowbg = cpgopen(PGPLOT_window);
      else
	  cpg_windowbg = cpgopen(PSBG_window);
      cpgask(0);
      cpgsubp(3, 6);
  }
  
  if (file_exist(scriptfile) == 1)
      streamscript = fopen(scriptfile,"r");
  else
  {
      printf("ERROR: Ini-File %s not found!\n", scriptfile);
      return 0;
  }
  fscanf(streamscript, "ProcDir: %s\n", ProcDir);
  fclose(streamscript);

  //create ProcDir
  CreatePath(ProcDir);
  
  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "idrs.log");
  streamlog = fopen(ProcFilename, "w");

  if (createbiascorr == 1)
  {
      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "biascorr.dat");
      streambias = fopen(ProcFilename, "w");
  }
  
  redstepfrom = 0;
  
  if (simfringe == 0)
      redstepto = 2;
  else
      redstepto = 1;

  if (simfringe == 1)
  {
      genFringeAmplitude = 0.0;
      ADDITIVE_NOISE = 0.0;
  }

  splitdatafrom = 0;
  found = 0;
  redstepobs = 32000;
  for (i=1; i<argc; i++) 
    {
      if (strcasecmp(argv[i], "splitdata") == 0) 
	{
	  splitdatafile = atoi(argv[i+1]);
	  splitdatafile2 = atoi(argv[i+2]);
	  splitdatafrom = atoi(argv[i+3]);
	  splitdatato = atoi(argv[i+4]);
	}

      if (strcasecmp(argv[i], "all") == 0) found = i;
      if (strcasecmp(argv[i], "vis") == 0)
	{
	  found = i;
	  redstepfrom=1;
	}
      if (strcasecmp(argv[i], "cp") == 0)
	{
	  found = i;
	  redstepfrom=2;
	}
      if (strcasecmp(argv[i], "obs") == 0)
	{
	  found = i;
	  redstepfrom=0;
	  redstepobs = atoi(argv[i+1]);
	}
    }

  if (found == 0)
  {
      printf("IDRS was called with an unknown argument!.\nPlease call idrs without argument to get a list of all supported arguments.\n");
      return 0;
  }

  printf("Processing Inifile: %s\n", scriptfile);

  if (found >= 1)
    {
      kernelwidth = 50.0;
      
      // allocate vectors using the Numerical Recipes vector design
      y_raw_double0 = dvector(0,nPtsScan-1);
      y_raw_double1 = dvector(0,nPtsScan-1);
      y_raw_double2 = dvector(0,nPtsScan-1);
      filtered0 = dvector(0,nPtsScan-1);
      filtered1 = dvector(0,nPtsScan-1);
      filtered2 = dvector(0,nPtsScan-1);
      power0 = dvector(0,nPtsScan-1);
      power1 = dvector(0,nPtsScan-1);
      power2 = dvector(0,nPtsScan-1);
      poweravg0 = dvector(0,nPtsScan-1);
      poweravg1 = dvector(0,nPtsScan-1);
      poweravg2 = dvector(0,nPtsScan-1);
      poweravg0bg = dvector(0,nPtsScan-1);
      poweravg1bg = dvector(0,nPtsScan-1);
      poweravg2bg = dvector(0,nPtsScan-1);
      envelope = dvector(0,nPtsScan-1);
      y_raw = vector(1,nPtsScan);
      
      cwt_p = fftw_create_plan(npad, FFTW_FORWARD, FFTW_ESTIMATE);
      cwt_p2 = fftw_create_plan(npad, FFTW_BACKWARD, FFTW_ESTIMATE);

      fftw_photcorr_f = fftw_create_plan(3*nPtsScan, FFTW_FORWARD, FFTW_ESTIMATE);
      fftw_photcorr_b = fftw_create_plan(3*nPtsScan, FFTW_BACKWARD, FFTW_ESTIMATE);
      

      corr = dmatrix(0, 3, 0, nPtsScan-1);
      matrix4 = dmatrix(0, 5, 0, nPtsScan-1);
      matrixavg4 = dvector(0, 6);
      
      nDFT = roundOdd((float) nPeriodsInDFTWindow*samplesPerFringePeriod);

      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "corrrep.dat");
      streamrepcorr = fopen(ProcFilename, "w");
      if (repfiles == 1)
      {
	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "matrix.dat");
	  streammatrix = fopen(ProcFilename, "w");

	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "kdeamp.dat");
	  streamkdeamp = fopen(ProcFilename, "w");

	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "kdewid.dat");
	  streamkdewid = fopen(ProcFilename, "w");
      }

      for (redstep = redstepfrom; redstep <= redstepto; redstep++)
	{
	  if (redstep == 0) 
	  {
	      printf("*** PROCESSING THE MATRIX FILES ***\n");
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "phot.idx");
	      streamphotidx = fopen(ProcFilename, "w");
	  }
	  
	  if (redstep == 1) 
	    {
	      fclose(streamrepcorr);
	      
	      printf("\n\n***  PROCESSING THE DATA FILES  ***\n\n");
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "ampnocp.dat");
	      streamamp = fopen(ProcFilename, "w");
	      
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "amp.dat");
	      if ((redstepobs < 32000) && (file_exist(ProcFilename) == 1))
	      {
		  streamamp2 = fopen(ProcFilename, "r");

		  while (feof(streamamp2) == 0)
		  {
		      fscanf(streamamp2, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
		      fscanf(streamamp2, streamampstring2, &biascorr[0], &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);
		      
		      printf(streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
		      printf(streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], wavelength*1.0e6); //corrfactor[2]);
		      if (scriptnrall < redstepobs)
		      {
			  fprintf(streamamp, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
			  fprintf(streamamp, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], wavelength*1.0e6); //corrfactor[2]);
		      }
		  }
		  
		  fclose(streamamp2);
	      }
	    }

	  if (redstep == 2)
	  {
	      printf("\n\n***  PROCESSING THE DATA FILES  ***\n\n");
	      printf("***   COMPUTING CLOSURE-PHASES  ***\n\n");
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "cp.dat");
	      streamcp = fopen(ProcFilename, "w");
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "amp.dat");
	      streamamp = fopen(ProcFilename, "w");
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "ampnocp.dat");
	      streamamp2 = fopen(ProcFilename, "r");
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "baseline.dat");
	      streambaseline = fopen(ProcFilename, "w");
	  }
	
          strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "info.dat");
	  streaminfo = fopen(ProcFilename, "w");

	  streamscript = fopen(scriptfile,"r");

	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "pow.dat");
	  streampow = fopen(ProcFilename, "w");
	  if (repfiles == 1)
	  {
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "simbias.dat");
	      streamsim = fopen(ProcFilename, "w");
	  }

	  fscanf(streamscript, "ProcDir: %s\n", ProcDir);

	  scriptdirnr = 0;
	  scriptnrall = -1;
	  scriptnrall2proc = 1;
	  scriptnrallfound = 0;
	  biaslog = 0.0;
	  biaslog2 = 0.0;
	  for (channel=0; channel<3; channel++)
	    {
	      fitprobequal0[channel] = 0;
	      fitprobnot0[channel] = 0;
	    }
	  
	  //READS OUTPRAW.INI
	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "idrs_raw.ini");
	  if (file_exist(ProcFilename) == 1)
	  {
	      stream2 = fopen(ProcFilename, "r");
	      
	      fscanf(stream2, "%s %s %i %i", &temptext, &outpraw_dir, &outpraw_filenr, &outpraw_scannr);
	      
	      fclose(stream2);
	  }
	  else
	      printf("Skip writing raw data (no file '%s')\n", ProcFilename);
	  
	  while (feof(streamscript) == 0)
	    {
	      scriptdirnr++;
	      
	      //read whole line in text_
	      j = 0;
	      for (;;)
		{
		  text_[j] = getc(streamscript);
		  if (text_[j] == 10) break;
		  if (feof(streamscript) != 0) break;
		  j++;
		}

		    
	      if (j > 10)
	      {
		  //all files?
		  sscanf(text_, "Dir: %s  %s %s", &DataDir, &temptext, &temptext2);

		  if (strcasecmp(temptext2, "all") == 0)
		  { 
		      scriptmin = 0;
		      scriptmax = 0;
		  }
		  else
		      sscanf(text_, "Dir: %s  %s %i %s %i   %s %s", &DataDir, &temptext, &scriptmin, &temptext, &scriptmax, &temptext, &temptext2);


		  //all scans?
		  sscanf(text_, "Dir: %s  %s %i %s %i   %s %s", &DataDir, &temptext, &temptext3, &temptext, &temptext4, &temptext, &temptext2);
		  
		  if (strcasecmp(temptext2, "all") == 0)
		  {
		      scriptscanfrom = 0; 
		      scriptscanto = 0; 
		  }
		  else
		  {
		      if (strcasecmp(temptext2, "skip10") == 0)
		      {
			  scriptscanfrom = 11; 
			  scriptscanto = 0; 
		      }
		      else
		      {
			  if (strcasecmp(temptext2, "skip20") == 0)
			  {
			      scriptscanfrom = 21; 
			      scriptscanto = 0; 
			  }
			  else
			      sscanf(text_, "Dir: %s  %s %i %s %i   %s %i %s %i", &DataDir, &temptext, &temptext3, &temptext, &temptext4, &temptext, &scriptscanfrom, &temptext, &scriptscanto);
		      }
		  }

		  if (scriptmax == 0)
		      scriptmax = 32000;

		  if (redstepobs < 32000)
		  {
		      if (redstep == 0)
		      {
			  scriptmin = 0;
			  scriptmax = 32000;
			  printf("Reduce all matrix files!");
		      }
		      else
		      {
			  scriptmin = redstepobs;
			  scriptmax = redstepobs;
			  printf("Reduce only specific science files!");
		      }
		  }

		  if (file_exist(DataDir) == 0)
		  {
		      printf("ERROR: DataDir %s, as specified in Ini-File %s not found!\n", DataDir, scriptfile);
		      return 0;
		  }


		  for (scriptnr = scriptmin; scriptnr <= scriptmax; scriptnr++)
		  {
		      if (createbiascorr == 1) scriptnr = scriptmin;

		      if (redstepobs < 32000)
			  scriptnrall = redstepobs;
		      else
			  scriptnrall++;

		      sprintf(scriptnrtext, "%i", scriptnr);
		      strcpy(filename, DataDir);
		      strcat(filename, "iota");
		      strcat(filename, scriptnrtext);
		      
		      strcpy(filename2, filename);
		      strcat(filename2, ".header");

		      strcat(filename, ".data");
		      
		      for (i=0; i<nPtsScan; i++)
			{
			  poweravg0[i] = 0.0;
			  poweravg1[i] = 0.0;
			  poweravg2[i] = 0.0;
			  poweravg0bg[i] = 10000.0;
			  poweravg1bg[i] = 10000.0;
			  poweravg2bg[i] = 10000.0;
			}
		      
		      for (i=0; i<3; i++) powspecavgcount[i]=0;
		      
		      if (file_exist(filename2) == 1)
		      {
			  if (scriptnrall > scriptnrallfound) scriptnrallfound = scriptnrall;

			  read_header(filename2, ProcDir, &N_RUNS, &fixeddelay, &calibfile, &objid, &jd, &ustart[0], &uend[0], &vstart[0], &vend[0], &ch_baseline[0], &wavelength, &tel0baseline[0], &tel1baseline[0], &tel2baseline[0], &fringefreq[0], scriptnrall, streaminfo);

			  if ((fabs(fringefreq[0]) > fabs(fringefreq[2])) && (fabs(fringefreq[0]) > fabs(fringefreq[4]))) channel_fastfringe = 2;
			  if ((fabs(fringefreq[2]) > fabs(fringefreq[0])) && (fabs(fringefreq[2]) > fabs(fringefreq[4]))) channel_fastfringe = 1;
			  if ((fabs(fringefreq[4]) > fabs(fringefreq[0])) && (fabs(fringefreq[4]) > fabs(fringefreq[2]))) channel_fastfringe = 0;


			  if (redstep == 2)
			      fprintf(streambaseline, "%f  %f %f %f  %f %f %f  %f %f %f\n", (float)(jd-2452500.0), (float)tel0baseline[0], (float)tel0baseline[1], (float)tel0baseline[2], (float)tel1baseline[0], (float)tel1baseline[1], (float)tel1baseline[2], (float)tel2baseline[0], (float)tel2baseline[1], (float)tel2baseline[2]);

			  powfreq = 30;
			  
			  if (objid == 0)
			    {
			      printf("   Missing Object-ID for %s! (Please check objid.ini!)\n", filename2);
			      fprintf(streamlog, "   Missing Object-ID for %s!\n", filename2);
			    }
			      
			  
			  if (scriptscanfrom == 0) 
			    scanfrom = 1;
			  else
			    scanfrom = scriptscanfrom;
			  
			  if (scriptscanto == 0)
			    scanto = N_RUNS;
			  else
			    scanto = scriptscanto;
			}

		      if (simfringe == 1) N_RUNS = N_RUNSmax;


		      //*** Write raw-signal into file
		      if ((strcasecmp(outpraw_dir, DataDir) == 0) && (outpraw_filenr == scriptnr) && (simfringe == 0))
			if ((file_exist(filename)) && ((redstep == 1) || (redstep == 2)))
			{
			    strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "phot.idx");

			    if (dophotcorr == 1)
				if (find_photfile(ProcFilename, scriptnrall, matrix1level, matrix2level, matrix3level) == 0)
				{
				    fprintf(streamlog, "   Missing corr files (File #%i, %s)\n", scriptnrall, filename);
				    printf("   Missing corr files (File #%i)\n", scriptnrall);
				} 
			    
			    printf("\n\n   WRITE RAW-DATA IN FILE 'sigraw.dat' and 'rawfringes.dat' for scan #%i in the following data file!\n", outpraw_scannr);
		    
			    stream2 = fopen(filename,"r");
			    strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "sigraw.dat");
			    streamraw = fopen(ProcFilename,"w");
			    
			    for (i_run = 1; i_run <= N_RUNS; i_run++) 
			      {
				for (i=0;i<nPtsScan;i++)
				  { 
				    read_singlescan(stream2, singlescan);
				    
				    if ((i_run == outpraw_scannr) || (simfringe == 1))
				      fprintf(streamraw, "%i %f %f %f %f %f %f\n", i, singlescan[0], singlescan[1], singlescan[2], singlescan[3], singlescan[4], singlescan[5]);
				  }
			      }
			    
			    fclose(streamraw);
			    fclose(stream2);
			    
				  
			    stream2 = fopen(filename,"r");

			    strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "sigfringes.dat");
			    streamraw = fopen(ProcFilename,"w");

			    strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "powspec.dat");
			    streammedian = fopen(ProcFilename, "w");

			      
			    for (i_run = 1; i_run <= N_RUNS; i_run++) 
			    {
				usescan = 1;
				if (dophotcorr == 1)
				    usescan = read_scan_photcorr(stream2, y_raw_double0, y_raw_double1, y_raw_double2, matrix1level, matrix2level, matrix3level, matrix4, matrixavg4, ch_baseline, fftw_photcorr_f, fftw_photcorr_b);
				else
				    read_scan(stream2, y_raw_double0, y_raw_double1, y_raw_double2, matrix4, matrixavg4, ch_baseline, fftw_photcorr_f, fftw_photcorr_b);

				if ((i_run == outpraw_scannr) || (simfringe == 1))
				{
				    if (VisFringeTrackerSelection == 0)
				    {
					packetFound[0] = usescan;
					packetFound[1] = usescan;
					packetFound[2] = usescan;
					centerBin[0] = 0.0;
					centerBin[1] = 0.0;
					centerBin[2] = 0.0;
				    }
				    else
				    {
					packetFound[0] = ptracker_ed(y_raw_double0, 0, nPtsScan-1, &nDFT, nPeriodsInDFTWindow, nDFTSmooth, nFringe, templateType, useSymmetry, FringeTrackerthresholdRatio, &weight[0], &centerPhase[0]);
					centerBin[0] = (centerPhase[0]+PI)/(2*PI)*nPtsScan;
					
					packetFound[1] = ptracker_ed(y_raw_double1, 0, nPtsScan-1, &nDFT, nPeriodsInDFTWindow, nDFTSmooth, nFringe, templateType, useSymmetry, FringeTrackerthresholdRatio, &weight[1], &centerPhase[1]);
					centerBin[1] = (centerPhase[1]+PI)/(2*PI)*nPtsScan;
					
					packetFound[2] = ptracker_ed(y_raw_double2, 0, nPtsScan-1, &nDFT, nPeriodsInDFTWindow, nDFTSmooth, nFringe2, templateType, useSymmetry, FringeTrackerthresholdRatio, &weight[2], &centerPhase[2]);
					centerBin[2] = (centerPhase[2]+PI)/(2*PI)*nPtsScan;
				    }

				    for (i=0;i<nPtsScan;i++)
				      fprintf(streamraw, "%f %f  %f %f  %f %f\n", (float)(i-centerBin[0])*OPD_slowfringe/(float)nPtsScan, y_raw_double0[i], (float)(i-centerBin[1])*OPD_slowfringe/(float)nPtsScan, y_raw_double1[i], (float)(i-centerBin[2])*2.0*OPD_slowfringe/(float)nPtsScan, y_raw_double2[i]);

				    printf("centerBin: %f %f %f\n", centerBin[0], centerBin[1], centerBin[2]);

				    calc_powspec(y_raw_double0, power0);
				    calc_powspec(y_raw_double1, power1);
				    calc_powspec(y_raw_double2, power2);
				    
				    //writes the powerspectrum in a file
				    for (i=0; i<nPtsScan/2; i++)
				      fprintf(streammedian, "%f %f %f %f\n", (float)i/((float)nPtsScan/2.0)*(FFT_maxfreq-FFT_minfreq)+FFT_minfreq, (float)power0[i], (float)power1[i], (float)power2[i]);
				    
				}
			    }
			    
			    fclose(streammedian);
			    fclose(streamraw);
			    fclose(stream2);
			}

		      
			      
		      //*** Measuring the levels from the Matrixfiles
		      if ((redstep == 0) && (file_exist(filename)) && (calibfile > 0) && (simfringe == 0))
		      {
			  printf("\n*** PROCESSING MATRIXFILE %i: %s (%i Scans/ch.)\n", calibfile, filename, N_RUNS);
			  
			  generate_matrixfiles(filename, ch_baseline, N_RUNS, scriptnrall, calibfile);
			  
			  if (calibfile == 4)
			  {
			      if (generate_photfiles_avg(scriptnrall, ch_baseline, streamrepcorr, streamlog, objid) == 4)
				  fprintf(streamphotidx, "%i %i %s\n", scriptnrall, objid, filename);
			  }
		      }
		      
		      if ((redstep == 2) && (scriptnrall2proc == 1))
		      {
			  if (redstepobs < 32000)
			  {
			      while (feof(streamamp2) == 0)
			      {
				  fscanf(streamamp2, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
				  fscanf(streamamp2, streamampstring2, &biascorr[0], &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);
				  
				  if (scriptnrall < redstepobs)
				  {
				      fprintf(streamamp, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
				      fprintf(streamamp, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], corrfactor[2]);				      
				  }
			      }
			  }
			  else
			  {
			      fscanf(streamamp2, streamampstring1, &scriptnrall2, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
			      fscanf(streamamp2, streamampstring2, &biascorr_norm[0], &biascorr_norm[1], &biascorr_norm[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);
			  }
			    
			  for (i=0; i<3; i++)
			      fitamp[i] = sreamp[i];
			    
			  scriptnrall2proc = 0;
		      }


			      
		      //*** Processing the datafiles
		      if ((((calibfile == 0) && (redstep > 0)) && (file_exist(filename))) || (simfringe == 1))
		      {
			    
			  //if (scriptnr == scriptmin)
			  {
			      fitsuccessnr = 0;
			      fitsuccessnrchannel[0] = 0;
			      fitsuccessnrchannel[1] = 0;
			      fitsuccessnrchannel[2] = 0;
			      fiterrornr = 0;
			      refused_outoflimits[0] = 0;
			      refused_outoflimits[1] = 0;
			      refused_outoflimits[2] = 0;
			      refused_nofringe[0] = 0;
			      refused_nofringe[1] = 0;
			      refused_nofringe[2] = 0;
			      
			      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "amp0.tmp");
			      stream[0] = fopen(ProcFilename, "w");

			      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "amp1.tmp");
			      stream[1] = fopen(ProcFilename, "w");

			      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "amp2.tmp");
			      stream[2] = fopen(ProcFilename, "w");
			      
			      scannr = 0;
			  }
			  
			  
			  printf("\n*** PROCESSING FILE: %s (=#%i; %i Scans/ch.)", filename, scriptnrall, N_RUNS);

			  if (calibfile != 0) 
			    printf(" - MATRIXFILE %i\n", calibfile);
			  else
			    printf("\n");
			  

			  //*** Reading Calibration Data
			  if ((redstep == 1) || (redstep == 2))
			  {
			      for (channel=0; channel<3; channel++)
			      {
				  biascorr[channel] = 0.0;
				  biascorrcount[channel] = 0.0;
			      }


			      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "phot.idx");
			      if (find_photfile(ProcFilename, scriptnrall, matrix1level, matrix2level, matrix3level) == 0)
			      {
				  fprintf(streamlog, "   Missing corr files (File #%i, %s)\n", scriptnrall, filename);
				  printf("   Missing corr files (File #%i)\n", scriptnrall);
			      }
			      else
			      {
				  if (repfiles == 1)
				      fprintf(streammatrix, "%i %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f ", scriptnrall, matrix1level[0], matrix1level[1], matrix1level[2], matrix1level[3], matrix1level[4], matrix1level[5], matrix2level[0], matrix2level[1], matrix2level[2], matrix2level[3], matrix2level[4], matrix2level[5], matrix3level[0], matrix3level[1], matrix3level[2], matrix3level[3], matrix3level[4], matrix3level[5], -objid/10.0);
			      }
			  }


			  for (i=0; i<3; i++)
			  {
			      fitamp[i] = 0.0;
			      fitampsqr[i] = 0.0;
			      fitwid[i] = 0.0;
			  }		
			  
			  for (channel=0; channel<3; channel++)
			  {
			      biascorr[channel] = 0.0;
			      biascorrcount[channel] = 0;
			  }			
			  
			  if (simfringe == 0)
			  {
			      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "phot.idx");
			      if (find_matrix4(ProcFilename, scriptnrall, matrix4) == 0)
			      {
				  fprintf(streamlog, "   Missing matrix4 (File #%i, %s)\n", scriptnrall, filename);
				  if (repfiles == 1) fprintf(streammatrix, "\n");
				  printf("   Missing matrix4 files (File #%i)\n", scriptnrall);
				  
			      }

			      if (find_matrixavg4(ProcFilename, scriptnrall, matrixavg4) == 0)
			      {
				  fprintf(streamlog, "   Missing matrixavg4 (File #%i, %s)\n", scriptnrall, filename);
				  if (repfiles == 1) fprintf(streammatrix, "\n");
				  printf("   Missing matrixavg4 files (File #%i)\n", scriptnrall);
				  
			      }
			      else
				  if (repfiles == 1)
				      fprintf(streammatrix, "%f %f %f %f %f %f\n", matrix4[0], matrix4[1], matrix4[2], matrix4[3], matrix4[4], matrix4[5]);
			  }
			  
			  
			  stream2 = fopen(filename,"r");
			  
			  i_run2 = 0;
			  if (simfringe == 1)
			  {
			      N_RUNS = simfringe_N_RUNSmax; //N_RUNSmax;
			      
			      if (createbiascorr == 1)
			      {
				  biascorr[0] = 0.0;
				  biascorr[1] = 0.0;
				  biascorr[2] = 0.0;
				  biascorrcount[0] = 0;
				  biascorrcount[1] = 0;
				  biascorrcount[2] = 0;
				  				  
				  if (genFringeAmplitude >= 0.99)
				  {
				      genFringeAmplitude = 0.0;
				      ADDITIVE_NOISE += createbiascorr_interv_noise*2.0;//1.0*(float)i_run/(float)N_RUNS;

				      if (repfiles == 1)
					  fclose(streamsim);
				      
				      fit_biascorr(&parambias, &noiseparambias);

				      fprintf(streambias, "%f %f\n", noiseparambias, parambias);

				      if (repfiles == 1)
				      {
					  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "simbias.dat");
					  streamsim = fopen(ProcFilename, "w");
				      }
				  }

				  genFringeAmplitude += createbiascorr_interv_amp;

				  
				  if (genFringeAmplitude <= 1.0)
				      printf("*** Calculate BIAS correction: Ampl: %f  Noise-Level: %f\n", genFringeAmplitude, ADDITIVE_NOISE);
			      }
			  }

			  //Set avgwavelet to null
			  if (avgCWT == 1)
			    for (i=0; i<nPtsScan; i++)
			      for (j=0; j<nPtsWavelets; j++)
				{
				  avgwavelet0[i][j] = 0.0;
				  avgwavelet1[i][j] = 0.0;
				  avgwavelet2[i][j] = 0.0;
				}

			  for (channel=0; channel<3; channel++)
			  {
			      biascorr[channel] = 0.0;
			      biascorrcount[channel] = 0;
			  }

			  for (i_run = 1; i_run <= N_RUNS; i_run++) 
			  {
			      sprintf(NameScan, "%s - Scan #%i\n", filename, i_run);
			      
			      scannr++;
			      usescan = 1;
			      
			      /* generate test data */
			      if (simfringe == 1)
				{
				  if (createbiascorr == 0)
				  {
				      genFringeAmplitude = (float)(scriptnrall-1) / (float)(scriptmax-scriptmin);
				      ADDITIVE_NOISE = 0.1*2.0;//1.0*(float)i_run/(float)N_RUNS;
				      
				      if (i_run == 1) printf("Noise: %f   Amp: %f        %i\n", ADDITIVE_NOISE/2.0, genFringeAmplitude, powspecavgcount[0]);
				  }
				  else
				  {
				      if (ADDITIVE_NOISE > 1.0)
				      {
					fclose(streambias);
					printf("\n\nDone!\n");
					
					return 0;
				      }
				  }
			      
				  genFringeFrequency = 60.0;
				  generateIdealPacket(y_raw, 1, nPtsScan);
		 		  
				  for (i=1;i<=nPtsScan;i++)
				  {
				      y_raw_double0[i-1] = (double) y_raw[i]-0.3;
				      y_raw_double1[i-1] = (double) y_raw[i]-0.3;
				    }


				  genFringeFrequency = 60.0; //60
				  generateIdealPacket(y_raw, 1, nPtsScan);
		 		  
				  for (i=1;i<=nPtsScan;i++)
				      y_raw_double2[i-1] = (double) y_raw[i]-0.3;
			      }
			      else
			      {
				  if (redstep == 1)
				  {
				      if (dophotcorr == 1)
				      {
					  usescan = read_scan_photcorr(stream2, y_raw_double0, y_raw_double1, y_raw_double2, matrix1level, matrix2level, matrix3level, matrix4, matrixavg4, ch_baseline, fftw_photcorr_f, fftw_photcorr_b);
				      }
				      else
					  read_scan(stream2, y_raw_double0, y_raw_double1, y_raw_double2, matrix4, matrixavg4, ch_baseline, fftw_photcorr_f, fftw_photcorr_b);
				  }
				  else
				      read_scan2(stream2, y_raw_double0, y_raw_double1, y_raw_double2, ch_baseline);
			      }

			      
			      if (((i_run >= scanfrom) && (i_run <= scanto)) || (simfringe == 1))
			      {
				  if (VisFringeTrackerSelection == 0)
				  {
				      packetFound[0] = usescan;
				      packetFound[1] = usescan;
				      packetFound[2] = usescan;
				      centerBin[0] = 0.0;
				      centerBin[1] = 0.0;
				      centerBin[2] = 0.0;
				  }
				  else
				  {
				      //*** Find Fringe using Fringe Tracker
				      packetFound[0] = ptracker_ed(y_raw_double0, 0, nPtsScan-1, &nDFT, nPeriodsInDFTWindow, nDFTSmooth, nFringe, templateType, useSymmetry, FringeTrackerthresholdRatio, &weight[0], &centerPhase[0]);
				      centerBin[0] = (centerPhase[0]+PI)/(2*PI)*nPtsScan;
				      
				      packetFound[1] = ptracker_ed(y_raw_double1, 0, nPtsScan-1, &nDFT, nPeriodsInDFTWindow, nDFTSmooth, nFringe, templateType, useSymmetry, FringeTrackerthresholdRatio, &weight[1], &centerPhase[1]);
				      centerBin[1] = (centerPhase[1]+PI)/(2*PI)*nPtsScan;
				      
				      packetFound[2] = ptracker_ed(y_raw_double2, 0, nPtsScan-1, &nDFT, nPeriodsInDFTWindow, nDFTSmooth, nFringe2, templateType, useSymmetry, FringeTrackerthresholdRatio, &weight[2], &centerPhase[2]);
				      centerBin[2] = (centerPhase[2]+PI)/(2*PI)*nPtsScan;
				    }

				  if (redstep == 1)
				    for (channel=0; channel<3; channel++)
					if (weight[channel] < 100.0)
					{
					    biascorr[channel] += weight[channel];
					    biascorrcount[channel]++;
					}

				  
				  
                                  if ((fitampmethod == 2) && (redstep == 1))
				  {
				      if (packetFound[0] == 1)
				      {
					  if (doplot2 == 1)
					  {
					      cpgslct(cpg_window2);
					      
					      cpgsubp(1, 1);
					      cpgeras;
					      cpgsubp(2, 2);
					  }
					  
					  waveletamp = cwavelettrans(cwt_p, cwt_p2, 0, centerBin[0], y_raw_double0, &wavelet[0][0], filtered0, objid);
					  
					  if (waveletamp > 0.0)
					  {
					      fitamp[0] += waveletamp;
					      fitampsqr[0] += waveletamp*waveletamp;
					      fitsuccessnrchannel[0]++;
					      fprintf(stream[0], "%f\n", (float)waveletamp);
					      
					      if (avgCWT == 1)
						  for (i=0; i<nPtsScan; i++)
						      for (j=0; j<nPtsWavelets; j++)
							  if ((i+(centerBin[0]-128) < nPtsScan) && (i+(centerBin[0]-128) > 0))
							      avgwavelet0[i][j] += wavelet[(int)(i+(centerBin[0]-128))][j];
					  }
					  else
					      refused_outoflimits[0]++;
				      }
				      else
				      {
					  refused_nofringe[0]++;
					  waveletamp = -9.0;
				      }

				      //if ((strcasecmp(outpraw_dir, DataDir) == 0) && (outpraw_filenr == scriptnr) && (simfringe == 0))
				      //write_wavelet(wavelet);
				      
				      printf("Scan #%4i - Fringe-Power: %12.2f",  i_run, waveletamp);

				      if (packetFound[1] == 1)
				      {
					  if (doplot2 == 1)
					  {
					      cpgslct(cpg_window2);

					      cpgsubp(1, 1);
					      cpgeras;
					      cpgsubp(2, 2);
					  }
					  
					  waveletamp = cwavelettrans(cwt_p, cwt_p2, 1, centerBin[1], y_raw_double1, &wavelet[0][0], filtered1, objid);
					  
					  if (waveletamp > 0.0)
					  {
					      fitamp[1] += waveletamp;
					      fitampsqr[1] += waveletamp*waveletamp;
					      fitsuccessnrchannel[1]++;
					      fprintf(stream[1], "%f\n", (float)waveletamp);

					      if (avgCWT == 1)
						for (i=0; i<nPtsScan; i++)
						  for (j=0; j<nPtsWavelets; j++)
						    if ((i+(centerBin[0]-128) < nPtsScan) && (i+(centerBin[0]-128) > 0))
						      avgwavelet1[i][j] += wavelet[(int)(i+(centerBin[0]-128))][j];
					  }
					  else
					  {
					      refused_outoflimits[1]++;
					  }
				      }
				      else
				      {
					  refused_nofringe[1]++;
					  waveletamp = -9.0;
				      }

				      printf("  %12.2f", waveletamp);


				      if (packetFound[2] == 1)
				      {
					  if (doplot2 == 1)
					  {
					      cpgslct(cpg_window2);
					      
					      cpgsubp(1, 1);
					      cpgeras;
					      cpgsubp(2, 2);
					  }
					  
					  waveletamp = cwavelettrans(cwt_p, cwt_p2, 2, centerBin[2], y_raw_double2, &wavelet[0][0], filtered2, objid);
					  
					  if (waveletamp > 0.0)
					  {
					      fitamp[2] += waveletamp;
					      fitampsqr[2] += waveletamp*waveletamp;
					      fitsuccessnrchannel[2]++;
					      fprintf(stream[2], "%f\n", (float)waveletamp);
					      if (avgCWT == 1)
						  for (i=0; i<nPtsScan; i++)
						      for (j=0; j<nPtsWavelets; j++)
							  if ((i+(centerBin[0]-128) < nPtsScan) && (i+(centerBin[0]-128) > 0))
							      avgwavelet2[i][j] += wavelet[(int)(i+(centerBin[0]-128))][j];
					  }
					  else
					      refused_outoflimits[2]++;

				      }
				      else
				      {
					  refused_nofringe[2]++;
					  waveletamp = -9.0;
				      }

				      printf("  %12.2f", waveletamp);
				      
				      printf("\n");
				  }
				  
				  
				  if (simfringe == 1)
				  {
				      centerBin[0] = centerBin[2];
				      centerBin[1] = centerBin[2];
				  }
				  
				  				  calc_powspec(y_raw_double0, power0);
				  calc_powspec(y_raw_double1, power1);
				  calc_powspec(y_raw_double2, power2);

				  
				  for (channel=0; channel<3; channel++)
				  {
				      if (packetFound[channel] == 1)
				      {
					  powspecavgcount[channel]++;

					  switch (channel)
					  {
					      case 0:
						  for (i=0; i<nPtsScan/2; i++)
						      poweravg0[i] += power0[i];
						  break;
						  
					      case 1:
						  for (i=0; i<nPtsScan/2; i++)
						      poweravg1[i] += power1[i];
						  break;
						  
					      case 2:
						  for (i=0; i<nPtsScan/2; i++)
						      poweravg2[i] += power2[i];
						  break;
					  }
				      }
				  }
				  
		                  //*** Calculate ClosurePhase
		                  if (redstep == 2)
				  {
				      centerBincp = (find_peak(y_raw_double0, 4) +  find_peak(y_raw_double1, 4) + find_peak(y_raw_double2, 2))/3.0;
				      
					if (centerBincp > 0)
					{
					  fit_phase(0, y_raw_double0, (int)centerBincp, CPWindowWidth, &cpphase[0], &cpfreq[0]);
					  fit_phase(1, y_raw_double1, (int)centerBincp, CPWindowWidth, &cpphase[1], &cpfreq[1]);
					  fit_phase(2, y_raw_double2, (int)centerBincp, CPWindowWidth, &cpphase[2], &cpfreq[2]);

					  //printf(" Phase: median: %f  %f  %f\n", cpphase[0], cpphase[1], cpphase[2]); 

					  if (i_run == scanfrom) 
					  {
					      printf("Baseline-Def.: ");
					      if (ch_baseline[0] == 0) printf("A-");
					      if (ch_baseline[0] == 1) printf("B-");
					      if (ch_baseline[0] == 2) printf("C-");
					      if (ch_baseline[1] == 0) printf("A ");
					      if (ch_baseline[1] == 1) printf("B ");
					      if (ch_baseline[1] == 2) printf("C ");
					      if (ch_baseline[2] == 0) printf("A-");
					      if (ch_baseline[2] == 1) printf("B-");
					      if (ch_baseline[2] == 2) printf("C-");
					      if (ch_baseline[3] == 0) printf("A ");
					      if (ch_baseline[3] == 1) printf("B ");
					      if (ch_baseline[3] == 2) printf("C ");
					      if (ch_baseline[4] == 0) printf("A-");
					      if (ch_baseline[4] == 1) printf("B-");
					      if (ch_baseline[4] == 2) printf("C-");
					      if (ch_baseline[5] == 0) printf("A: Sign convention ");
					      if (ch_baseline[5] == 1) printf("B: Sign convention ");
					      if (ch_baseline[5] == 2) printf("C: Sign convention ");
					  }
					  

					  //make CP close
					  cpphase[1] = -cpphase[1];

					  if (fringefreq[4] < 0.0) { cpphase[0] = -cpphase[0]; cpfreq[0] = -cpfreq[0]; }
					  if (fringefreq[2] < 0.0) { cpphase[1] = -cpphase[1]; cpfreq[1] = -cpfreq[1]; }
					  if (fringefreq[0] < 0.0) { cpphase[2] = -cpphase[2]; cpfreq[2] = -cpfreq[2]; }

					  //apply sign convention
					  //BA -> AB
					  if ((ch_baseline[0] == 1) && (ch_baseline[1] == 0))
					    {
					      cpphase[0] = -cpphase[0];
					      if (i_run == scanfrom) printf("-phi0  ");
					    }
					  if ((ch_baseline[2] == 1) && (ch_baseline[3] == 0))
					    {
					      cpphase[1] = -cpphase[1];
					      if (i_run == scanfrom) printf("-phi1  ");
					    }
					  if ((ch_baseline[4] == 1) && (ch_baseline[5] == 0))
					    {
					      cpphase[2] = -cpphase[2];
					      if (i_run == scanfrom) printf("-phi2  ");
					    }

					  //CB -> BC
					  if ((ch_baseline[0] == 2) && (ch_baseline[1] == 1))
					    {
					      cpphase[0] = -cpphase[0];
					      if (i_run == scanfrom) printf("-phi0  ");
					    }
					  if ((ch_baseline[2] == 2) && (ch_baseline[3] == 1))
					    {
					      cpphase[1] = -cpphase[1];
					      if (i_run == scanfrom) printf("-phi1  ");
					    }
					  if ((ch_baseline[4] == 2) && (ch_baseline[5] == 1))
					    {
					      cpphase[2] = -cpphase[2];
					      if (i_run == scanfrom) printf("-phi2  ");
					    }

					  //AC -> CA
					  if ((ch_baseline[0] == 0) && (ch_baseline[1] == 2))
					    {
					      cpphase[0] = -cpphase[0];
					      if (i_run == scanfrom) printf("-phi0  ");
					    }
					  if ((ch_baseline[2] == 0) && (ch_baseline[3] == 2))
					    {
					      cpphase[1] = -cpphase[1];
					      if (i_run == scanfrom) printf("-phi1  ");
					    }
					  if ((ch_baseline[4] == 0) && (ch_baseline[5] == 2))
					    {
					      cpphase[2] = -cpphase[2];
					      if (i_run == scanfrom) printf("-phi2  ");
					    }

					  if (i_run == scanfrom) printf("\n");

					  cpfreq[2] = -cpfreq[2];
                                          

					  for (i=0; i<3; i++)
					    {
					      while (cpphase[i] < 0.0) cpphase[i] += 2.0*PI;
					      while (cpphase[i] > 2.0*PI) cpphase[i] -= 2.0*PI;
					    }
					  
					  //*** check for frequency closure
					  if ((abs(cpfreq[0]+cpfreq[1]+cpfreq[2]) <= CPfreqclosureThreshold) && ((cpfreq[0] != 0.0) || (cpfreq[1] != 0.0) || (cpfreq[2] != 0.0)) && ((isnan(cpphase[0]) == 0.0) && (isnan(cpphase[1]) == 0.0) && (isnan(cpphase[2]) == 0.0)))
					  {
					      i_run2++;
					      
					      closurePhase[i_run2] = (cpphase[0]+cpphase[1]+cpphase[2]);
					      
					      while (closurePhase[i_run2] < 0.0) closurePhase[i_run2] += 2.0*PI;
					      while (closurePhase[i_run2] >= 2.0*PI) closurePhase[i_run2] -= 2.0*PI;
					      
					      fprintf(streamcp, "%f  %f  %f  %f  %f\n", (float)scriptnrall+(float)i_run/(float)N_RUNS, cpphase[0], cpphase[1], cpphase[2], closurePhase[i_run2]);
					  }
					}
				    }
				  
				  if ((redstep == 1) && ((fitampmethod == 0) || ((algCWT == 0) && (fitampmethod == 2)) || ((fitampmethod == 1) && (doplot1 == 1)) ))
				    {
				      if (fitampmethod == 0)
					{
					  //*** Checks, if all data points are within reasonable limits
					  withinlimits[0] = 10;
					  withinlimits[1] = 10;
					  withinlimits[2] = 10;
					  for (i=0; i<nPtsScan; i++)
					    {
					      if (abs(y_raw_double0[i]) > fitenv_max_vis)
						{
						  y_raw_double0[i] = 0.0;
						  withinlimits[0]--;
						}
					      
					      if (abs(y_raw_double1[i]) > fitenv_max_vis)
						{
						  y_raw_double1[i] = 0.0;
						  withinlimits[1]--;
						}
					      
					      if (abs(y_raw_double2[i]) > fitenv_max_vis)
						{
						  y_raw_double2[i] = 0.0;
						  withinlimits[2]--;
						}
					    }
					}
				      

				      //*** Calculate Amplitude
				      for (channel = 0; channel < 3; channel++)
					{
					  fitsuccess = 0;
					  

					  if (fitampmethod == 0)
					  {
					      if (withinlimits[channel] < 1)
					      {
						  refused_outoflimits[channel]++;
						  if (repdetails == 1) printf("   Scan#%4i, Channel%2i: Out of limit\n", i_run, channel);
					      }
					      
					      if (packetFound[channel] == 0) refused_nofringe[channel]++;

					  
					      if ((packetFound[channel] == 1) && (centerBin[channel] > nPtsScan*0.1) && (centerBin[channel] < nPtsScan*0.9) && (withinlimits[channel] > 0))
						{
						  //positive values
						  if (channel == 0) filter_envelope(y_raw_double0, envelope, nPtsScan, 1);
						  if (channel == 1) filter_envelope(y_raw_double1, envelope, nPtsScan, 1);
						  if (channel == 2) filter_envelope(y_raw_double2, envelope, nPtsScan, 1);
						  
						  //one run with fixed sinc-width
						  if (channel == channel_fastfringe)
						    fit_envelope(channel, envelope, centerBin[channel], fit_window_fast, param2, (channel+1), 0);
						  else
						    fit_envelope(channel, envelope, centerBin[channel], fit_window_slow, param2, (channel+1), 0);
						  
						  //one run with varying sinc-width
						  if (channel == channel_fastfringe)
						      fit_envelope(channel, envelope, centerBin[channel], fit_window_fast, param, (channel+1), 0); //1
						  else
						      fit_envelope(channel, envelope, centerBin[channel], fit_window_slow, param, (channel+1), 0); //1
						  
						  if ((param2[0] < param[0]) && (param2[0] > 0))
						    for (i=0; i<=3; i++)
						      param[i] = param2[i];
						  
					      
					          if ((param[0] >= 0.0) && (param[0] < 0.15) && (param[1] < fitenv_max_vis) && (((channel != 2) && (param[2] < 100.0)) || ((channel == channel_fastfringe) && (param[2] < 80.0))))
						  {
						      param[1] = sqr(param[1]);

						      fitsuccess = 1;
						      
						      fitsuccessnr++;
						      fitsuccessnrchannel[channel]++;
						      
						      //old-fashioned bias correction
						      if ((dobiascorr == 1)) // && (createbiascorr == 0) && ((1/weight[channel]) > 0.15))
							{
							  //printf("Corrected BIAS on channel %i with factor %f\n", channel, exp((1/weight[channel])/0.6*0.3));
							  param[1] /= exp((1.0/weight[channel]/2.0)/0.6*0.3);
							  param2[1] /= exp((1.0/weight[channel]/2.0)/0.6*0.3);
							}
						      
						      
						      if (repdetails == 1) printf("   Fitted Amp:%f  Wid:%f  ChiSqr:%f (Channel %i)\n", param[1], param[2], param[0], channel);
						      
						      biaslog += sqr(param[1]-genFringeAmplitude);
						      biaslog2 += param[1]-genFringeAmplitude;
						      
						      if ((repfiles == 1) && (channel == 0))
							{
							  fprintf(streamkdeamp, "%f\n", param[1]);
							  fprintf(streamkdewid, "%f\n", param[2]);
							}
						      
						      fitamp[channel] += param[1];
						      fitampsqr[channel] += param[1]*param[1];
						      fitwid[channel] += param[2];
						      
						      if (repdetails == 1) printf("Dir#%i File#%i Scan#%i  %f %f | %f %f | %f %f | %f %f\n", scriptdirnr, scriptnr, i_run, param[1], param2[1], param[2], param2[2], param[3], param2[4], param[0], param2[0]);						      
						      
						      fprintf(stream[channel], "%f\n",  param[1]);
						    }
						  else
						    {
						      param[1] = -1.0;
						      fiterrornr++;
						    }
						} //if packetfound
					    } //if fitampmethod

				      //PGPLOT: plot data
				        if (doplot1 == 1)
					{
					  cpgslct(cpg_window1);
					  
					  if (channel == 0)
					  {
					      cpgsubp(1, 1);
					      cpgeras;
					      cpgsubp(2, 3);
					  }
					  
					  //plot data
					  cpgsci(1);
					  ymin = 1e10;
					  ymax = 1e-10;
					  for (i=0; i<nPtsScan; i++)
					  {
					      plotx[i] = i;
					      if (channel == 0) ploty[i] = y_raw_double0[i];
					      if (channel == 1) ploty[i] = y_raw_double1[i];
					      if (channel == 2) ploty[i] = y_raw_double2[i];

					      if (ploty[i] > ymax) ymax = ploty[i];
					      if (ploty[i] < ymin) ymin = ploty[i];
					    }
					  
					  cpgsci(1);
					  cpgenv(-1.0, nPtsScan, -1.5, 1.5, 0, 1);
					  if (channel == 0) cpglab("", "Visibility", "Channel 0");
					  if (channel == 1) cpglab("", "Visibility", "Channel 1");
					  if (channel == 2) cpglab("", "Visibility", "Channel 2");
					  
					  if ((withinlimits[channel] > 0) || (redstep == 2))
					    cpgsci(1);
					  else
					    cpgsci(8);
					  
					  
					  cpgline(nPtsScan-1, plotx, ploty);

					  if ((fitampmethod == 0) || (fitampmethod == 2))
					  {
					      //plot envelope and fit
					      if (fitsuccess == 1)
					      {
						  cpgsci(2);
						  
						  for (i=0; i<nPtsScan; i++)
						      ploty[i] = envelope[i];
						  
						  cpgline(nPtsScan-1, plotx, ploty);
						  
						  cpgsci(3);
						  
						  
						  for (i=0; i<nPtsScan; i++)
						      ploty[i] = func_(i-centerBin[channel], param);
						  cpgline(nPtsScan-1, plotx, ploty);
						  for (i=0; i<nPtsScan; i++)
						      ploty[i] = -ploty[i];
						  cpgline(nPtsScan-1, plotx, ploty);	
					      }
					      
					      //plot fringe-tracker result
					      if (packetFound[channel] == 1)
					      {
						  cpgsci(5);
						  
						  plotx[0] = centerBin[channel];
						  plotx[1] = centerBin[channel];
						  ploty[0] = -100.0;
						  ploty[1] = 100.0;
						  
						  cpgline(2, plotx, ploty);
					      }
					  }

					  //plot powspec
					  cpgsci(1);
					  ymin = 1e10;
					  ymax = 1e-10;
					  for (i=0; i<=nPtsScan/2; i++)
					    {
					      plotx[i] = i;
					      
					      if (channel == 0) ploty[i] = power0[i];
					      if (channel == 1) ploty[i] = power1[i];
					      if (channel == 2) ploty[i] = power2[i];
					      
					      if (ploty[i] > ymax) ymax = ploty[i];
					      if (ploty[i] < ymin) ymin = ploty[i];
					    }
					  
					  cpgenv(-1.0, nPtsScan/2, 0.0, ymax*1.1, 0, 1);
					  if (channel == 0) cpglab("", "Power", "Channel 0");
					  if (channel == 1) cpglab("", "Power", "Channel 1");
					  if (channel == 2) cpglab("", "Power", "Channel 2");
					  
					  cpgline(nPtsScan/2-1, plotx, ploty);
					  cpgask(doplotwait);
					} //for channel
				      
				      if (doplot2 == 1)
					{
					  cpgslct(cpg_window2);
					  cpgsubp(1, 1);
					  cpgeras;
					  cpgsubp(1, 3);
					  cpgask(doplotwait);
					}
					} //if i_run in border
				    } //for channel
				} //if redstep
			    } //for i_run
	  
			    fclose(stream2);
			    
			    //generating KDE and fit gaussian on KDE to get averaged amplitude
			    //if (scriptnr == scriptmax)
			      {
				fclose(stream[0]);
				fclose(stream[1]);
				fclose(stream[2]);
			      }

			      //printf("%i %i\n", scriptnr, scriptmax);

			    //if (scriptnr == scriptmax)
			      {
				//write into log
				if (redstep == 1)
				{
				    if (fitampmethod == 2)
				    {
					fitsuccessnr = fitsuccessnrchannel[0] + fitsuccessnrchannel[1] + fitsuccessnrchannel[2];
					fiterrornr = refused_outoflimits[0] + refused_outoflimits[1] + refused_outoflimits[2] + refused_nofringe[0] + refused_nofringe[1] + refused_nofringe[2];
					
					printf("\n#Scans total:  %4i     #Scans kept: %4i     #Scans rejected: %4i\n", 3*scannr, fitsuccessnr, fiterrornr);
					printf("(refused: NoFringeFound: %4i %4i %4i       OutOfLimit: %4i %4i %4i)\n\n", refused_outoflimits[0], refused_outoflimits[1], refused_outoflimits[2], refused_nofringe[0], refused_nofringe[1], refused_nofringe[2]);
				    }
					
				    fprintf(streamlog, "%45s SCANS%4i FITSUCC%5i REFFIT%5i OUTOFLIMIT%5i/%5i/%5i NOFRINGE%5i/%5i/%5i BASELINES: ", filename, N_RUNS, fitsuccessnr, fiterrornr, refused_outoflimits[0], refused_outoflimits[1], refused_outoflimits[2], refused_nofringe[0], refused_nofringe[1], refused_nofringe[2]);
				    for (n=0; n<3; n++)
				      {
					fprintf(streamlog, " ch%i=", n);
					if (ch_baseline[0+n*2] == 0) fprintf(streamlog, "A");
					if (ch_baseline[0+n*2] == 1) fprintf(streamlog, "B");
					if (ch_baseline[0+n*2] == 2) fprintf(streamlog, "C");
					if (ch_baseline[1+n*2] == 0) fprintf(streamlog, "A");
					if (ch_baseline[1+n*2] == 1) fprintf(streamlog, "B");
					if (ch_baseline[1+n*2] == 2) fprintf(streamlog, "C");
				      }
				    fprintf(streamlog, "\n");


				    if ((fitsuccessnr > 5) || (fitampmethod == 1))//scannr*0.05)
				    {
					//*** Kernel Density Estimation for Amplitude
					outkde = dvector(0,nPtsScan-1);
					
					for (channel = 0; channel < 3; channel++)
					{
					    fitamp[channel] = -1.0; 
					    
					    //*** Fit the amplitude in the averaged power spectrum
					    if (fitampmethod == 1)
					    {
						powsum[channel] = 0.0;
						powsumbg[channel] = 0.0;
						
						if (powspecavgcount[channel] > 5)
						{
						    for (i=0; i<nPtsScan/2; i++)
							switch (channel)
							{
							    case 0:
								poweravg0[i] /= powspecavgcount[channel];
								break;
								
							    case 1:
								poweravg1[i] /= powspecavgcount[channel];
								break;
								
							    case 2:
								poweravg2[i] /= powspecavgcount[channel];
								break;
							}
						    
						    
						    if (channel == 0) 
						    {
							if (channel_fastfringe == 0)
							    powmaxpos[channel] = find_max(poweravg0, 2*powexpfreq-15, 2*powexpfreq+15);
							else
							    powmaxpos[channel] = find_max(poweravg0, powexpfreq-15, powexpfreq+15);
						    }

						    if (channel == 1)
						    {
							if (channel_fastfringe == 1)
							    powmaxpos[channel] = find_max(poweravg1, 2*powexpfreq-15, 2*powexpfreq+15);
							else
							    powmaxpos[channel] = find_max(poweravg1, powexpfreq-15, powexpfreq+15);
						    }

						    if (channel == 2)
						    {
							if (channel_fastfringe == 2)
							    powmaxpos[channel] = find_max(poweravg2, 2*powexpfreq-15, 2*powexpfreq+15);
							else
							    powmaxpos[channel] = find_max(poweravg2, powexpfreq-15, powexpfreq+15);
						    }

						    powmaxrangechannel = powmaxrange;
						    if (channel == channel_fastfringe) powmaxrangechannel = powmaxrange*1.5;
						    
						    powminleft = 0.0;
						    for (i=powmaxpos[channel]-powmaxrangechannel-2; i<=powmaxpos[channel]-powmaxrangechannel+2; i++)
						    {
							if (channel == 0) powminleft += poweravg0[i];
							if (channel == 1) powminleft += poweravg1[i];
							if (channel == 2) powminleft += poweravg2[i];
						    }
						    powminleft /= 5.0;
						    
						    powminright = 0.0;
						    for (i=powmaxpos[channel]+powmaxrangechannel-2; i<=powmaxpos[channel]+powmaxrangechannel+2; i++)
						    {
							if (channel == 0) powminright += poweravg0[i];
							if (channel == 1) powminright += poweravg1[i];
							if (channel == 2) powminright += poweravg2[i];
						    }
						    powminright /= 5.0;
						    
						    
						    for (i=powmaxpos[channel]-powmaxrangechannel; i<=powmaxpos[channel]+powmaxrangechannel; i++)
							switch (channel)
							{
							    case 0:
								poweravg0bg[i] = powminleft + ((powminright-powminleft)*(i-(powmaxpos[channel]-powmaxrangechannel-3))/(2*powmaxrangechannel));
								//poweravg0bg[i] = powminleft + 0.5*((powminright-powminleft)*i/(nPtsScan/2-1));
								powsum[channel] += (poweravg0[i] - poweravg0bg[i]);
								powsumbg[channel] += poweravg0bg[i];
								break;
								
							    case 1:
								poweravg1bg[i] = powminleft + ((powminright-powminleft)*(i-(powmaxpos[channel]-powmaxrangechannel-3))/(2*powmaxrangechannel));
								//poweravg1bg[i] = powminleft + 0.5*((powminright-powminleft)*i/(nPtsScan/2-1));
								powsum[channel] += (poweravg1[i] - poweravg1bg[i]);
								powsumbg[channel] += poweravg1bg[i];
								break;
								
							    case 2:
								poweravg2bg[i] = powminleft + ((powminright-powminleft)*(i-(powmaxpos[channel]-powmaxrangechannel-3))/(2*powmaxrangechannel));
								//poweravg2bg[i] = powminleft + 0.5*((powminright-powminleft)*i/(nPtsScan/2-1));
								powsum[channel] += (poweravg2[i] - poweravg2bg[i]);
								powsumbg[channel] += poweravg2bg[i];
								break;
							}
						   
						    expfreq = powexpfreq;
						    if (channel_fastfringe == 0)
						    {
							expfreq = 2*powexpfreq;
							exprange = 2*powexprange;
						    }
						    else
						    {
							expfreq = powexpfreq;
							exprange = powexprange;
						    }
						    if ((channel == 0) && (abs(powmaxpos[channel] - expfreq) <= exprange)) fitamp[channel] = powsum[channel];

						    expfreq = powexpfreq;
						    if (channel_fastfringe == 1)
						    {
							expfreq = 2*powexpfreq;
							exprange = 2*powexprange;
						    }
						    else
						    {
							expfreq = powexpfreq;
							exprange = powexprange;
						    }
						    if ((channel == 1) && (abs(powmaxpos[channel] - expfreq) <= exprange)) fitamp[channel] = powsum[channel];

						    expfreq = powexpfreq;
						    if (channel_fastfringe == 2)
						    {
							expfreq = 2*powexpfreq;
							exprange = 2*powexprange;
						    }
						    else
						    {
							expfreq = powexpfreq;
							exprange = powexprange;
						    }
						    if ((channel == 2) && (abs(powmaxpos[channel] - expfreq) <= exprange)) fitamp[channel] = powsum[channel];
						    

						    printf("Channel %i:    Signal: %8.4lf       - Peak at pixel #%i   (%i scans used)\n", channel, powsum[channel], powmaxpos[channel], powspecavgcount[channel]);
						    printf("          Background: %12.4lf   - left: %lf, right: %lf\n", powsumbg[channel]/(2*powmaxrangechannel), powminleft, powminright);
						    
						    if (doplot2 == 1)
						    {
							cpgslct(cpg_window2);

							cpgsci(1);
							ymin = 1e10;
							ymax = 1e-10;
							for (i=0; i<=nPtsScan/2; i++)
							{
							    plotx[i] = i;
							    
							    if (channel == 0) ploty[i] = poweravg0[i];
							    if (channel == 1) ploty[i] = poweravg1[i];
							    if (channel == 2) ploty[i] = poweravg2[i];

							    if (channel == 0) ploty2[i] = poweravg0bg[i];
							    if (channel == 1) ploty2[i] = poweravg1bg[i];
							    if (channel == 2) ploty2[i] = poweravg2bg[i];
							    
							    if (ploty[i] > ymax) ymax = ploty[i];
							    if (ploty[i] < ymin) ymin = ploty[i];
							}
							
							cpgenv(-1.0, nPtsScan/2, 0.0, ymax*1.1, 0, 1);
							if (channel == 0) cpglab("", "Power", "Channel 0");
							if (channel == 1) cpglab("", "Power", "Channel 1");
							if (channel == 2) cpglab("", "Power", "Channel 2");
							
							cpgsci(1);
							cpgline(nPtsScan/2-1, plotx, ploty);
							cpgsci(2);
							cpgline(nPtsScan/2-1, plotx, ploty2);
							//cpgask(1);
						    }
						  

						  if (repfiles == 1)
						  {
						      //writes the averaged powerspectrum in file
						      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "powspecavg.dat");
						      streammedian = fopen(ProcFilename, "w");
						      for (i=0; i<nPtsScan/2; i++)
							fprintf(streammedian,"%f %f %f %f\n", (float)i/((float)nPtsScan/2.0)*(FFT_maxfreq-FFT_minfreq)+FFT_minfreq, poweravg0[i], poweravg1[i], poweravg2[i], func_);
						      fclose(streammedian);
						    }


						  fitprob[channel] = powsum[channel]/powsumbg[channel];
						  printf("        Fringe Power: %12.4f   - SNR: %10.2f\n", fitamp[channel], powsum[channel]/powsumbg[channel]);
						}
					      else
						fitamp[channel] = 0.0;
					      
					      fitwid[channel] = 0.01;
					      ;


					      fprintf(streampow, "%i  %f %f %f   %i %i %i   %f %f %f\n", scriptnrall, fitamp[0], fitamp[1], fitamp[2], powmaxpos[0], powmaxpos[1], powmaxpos[2], powsum[0], powsum[1], powsum[2]);

					    } // if fitampmethod
					  
					  //*** Fit the averaged amplitude via KDE or mean
					  if ((fitampmethod == 0) || (fitampmethod == 2))
					    {
					      if ((fitsuccessnrchannel[channel] >= minvisnumber) && (refused_outoflimits[channel] <= 100))
						{
						  sprintf(channeltext, "%i", channel);
						  strcpy(filenameamp, ProcDir);
						  strcat(filenameamp, "amp");
						  strcat(filenameamp, channeltext);
						  strcat(filenameamp, ".tmp");						  

						  if (fitampmethod == 2)
						    {
						      //Do MEDIAN for wavelet methode!
						      fitampdata = vector(0, fitsuccessnrchannel[channel]-1);
						      
						      streammedian = fopen(filenameamp, "r");
						      
						      i = 0;
						      while (feof(streammedian) == 0)
							{
							  fscanf(streammedian,"%f\n", &fitampdata[i]);
							  if (fitampdata[i] > 0.0) i++;
							}
						      
						      fclose(streammedian);
						      
						      median50mean(i-2, fitampdata, &(fitamp[channel]), &(fitwid[channel]));
						      average_certain_range(i-2, fitampdata, &(fitamp[channel]), &(fitwid[channel]), fitamp[channel]*avg_range_low, fitamp[channel]*avg_range_high);
						      //average(i-2, fitampdata, &(fitamp[channel]), &(fitwid[channel]));

						      free_vector(fitampdata, 0, fitsuccessnrchannel[channel]-1);

						      //Plot histogram
						      if (doplot3 == 1)
						      {
							  cpgslct(cpg_window3);
							  histogram(filenameamp, 50);
						      }
						    }
						  else
						    {

						      if (fitviskde == 1)
							{
							  fitwid[channel] = kde(filenameamp, kernelwidth, outkde, &scalemin, &scalemax, cpg_window2);
							  printf(""); //this is needed in some way to avoid Segmentation fault
							  
							  if (fitwid[channel] == 0.0)
							    printf("KDE fits 0.0! Something wrong! - Absolutely no scattering?!?\n");
							  else
							    fitprob[channel] = kst(filenameamp, &fitpos[channel], fitwid[channel]);
							  
							  if (fitprob[channel] >= 0.34)
							    fitprobnot0[channel]++;
							  else
							    fitprobequal0[channel]++;
							}
						      else
							fitwid[channel] = kde(filenameamp, kernelwidth, outkde, &scalemin, &scalemax, cpg_window2);


						      fitwid[channel] = sqrt((fitampsqr[channel]-sqr(fitamp[channel])/fitsuccessnrchannel[channel])/(fitsuccessnrchannel[channel]-1))/(2.0*sqrt(fitsuccessnrchannel[channel]));
						      fitamp[channel] /= fitsuccessnrchannel[channel];
						    } //fitampmethod
						  
						  
						  printf("Channel #%i:  Fringe Amplitude: %10.2f     Variance: %10.2f\n", channel, fitamp[channel], fitwid[channel]);
						}
					      else
						{
						  fitamp[channel] = -0.1;
						  fitwid[channel] = 0.0;
						  fitprob[channel] = -1.0;
						} //fitsuccessnrchannel
					    } //if fitampmethod

					  if ((channel > 0) && (genFringeAmplitude > 0.0)) 
					      if (repfiles == 1)
						  fprintf(streamsim, "%i %f %f %f %f %f %f\n", channel, genFringeAmplitude, fitamp[channel], weight[channel], ADDITIVE_NOISE, weight[channel]/genFringeAmplitude, ADDITIVE_NOISE/genFringeAmplitude/2.0);
					} //for channel
				      free_dvector(outkde, 0, nPtsScan-1);
				    } //if fitsuccessnr
				  } // if redstep
				
				uavg[0] = (ustart[0]+uend[0])/2.0;
				vavg[0] = (vstart[0]+vend[0])/2.0;
				uavg[1] = (ustart[1]+uend[1])/2.0;
				vavg[1] = (vstart[1]+vend[1])/2.0;
				uavg[2] = (ustart[2]+uend[2])/2.0;
				vavg[2] = (vstart[2]+vend[2])/2.0;

				
				//*** Calculate mean closure-phase and variance
				if (redstep == 2)
				  {
				      if (((scriptnrall == scriptnrall2) || (redstepobs < 32000)) && (scriptnrall2proc == 0))
				      {
					scriptnrall2proc = 1;

					if (i_run2 > 5)
					{
					    average_cp(closurePhase, i_run2, &mean, &variance);
					    
					    printf(" -Raw Closure-Phase (%i phases used): Mean: %10.2f deg.      Variance: %10.2f deg.\n", i_run2, mean*RToD, variance*RToD);
					}
					else
					{
					    mean = -4.0;
					    variance = 0.0;
					}
					
					fprintf(streamamp, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
					fprintf(streamamp, streamampstring2, biascorr_norm[0], biascorr_norm[1], biascorr_norm[2], corrfactor[0], corrfactor[1], corrfactor[2]);
				      }
				  }
				else
				  {
				      for (channel=0; channel<3; channel++)
					  if (biascorrcount[channel] > 0)
					      biascorr_norm[channel] = biascorr[channel] / biascorrcount[channel];
					  else
					      biascorr_norm[channel] = -1.0;


				      //if (avgCWT == 1)
				      //write_wavelet(avgwavelet0);

				    if (simfringe == 1)
				      {
					fprintf(streamamp, streamampstring1, scriptnrall, genFringeAmplitude, objid, fitamp[0], fitwid[0], fitprob[0], fitamp[1], fitwid[1], fitprob[1], fitamp[2], fitwid[2], fitprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], 0.0, 0.0, calibtag);
					fprintf(streamamp, streamampstring2, biascorr_norm[0], biascorr_norm[1], biascorr_norm[2], corrfactor[0], corrfactor[1], corrfactor[2]);
				      }
				    else
				      {
					//if ((fitamp[0] > 0.0) || (fitamp[1] > 0.0) || (fitamp[2] > 0.0))
					fprintf(streamamp, streamampstring1, scriptnrall, jd-2452500.0, objid, fitamp[0], fitwid[0], fitprob[0], fitamp[1], fitwid[1], fitprob[1], fitamp[2], fitwid[2], fitprob[2], (ustart[0]+uend[0])/2.0, (vstart[0]+vend[0])/2.0, (ustart[1]+uend[1])/2.0, (vstart[1]+vend[1])/2.0, (ustart[2]+uend[2])/2.0, (vstart[2]+vend[2])/2.0, 0.0, 0.0, calibtag);
					//fprintf(streamamp, streamampstring2, biascorr[0]/biascorrcount[0], biascorr[1]/biascorrcount[1], biascorr[2]/biascorrcount[2], corrfactor[0], corrfactor[1], corrfactor[2]);
					fprintf(streamamp, streamampstring2, biascorr_norm[0], biascorr_norm[1], biascorr_norm[2], fitquality[0], fitquality[1], fitquality[2]); //(float)refused_outoflimits[0], (float)refused_outoflimits[1], (float)refused_outoflimits[2]);

					//printf(streamampstring1, scriptnrall, jd-2452500.0, objid, fitamp[0], fitwid[0], fitprob[0], fitamp[1], fitwid[1], fitprob[1], fitamp[2], fitwid[2], fitprob[2], (ustart[0]+uend[0])/2.0, (vstart[0]+vend[0])/2.0, (ustart[1]+uend[1])/2.0, (vstart[1]+vend[1])/2.0, (ustart[2]+uend[2])/2.0, (vstart[2]+vend[2])/2.0, 0.0, 0.0, calibtag);
					//printf(streamampstring2, biascorr_norm[0], biascorr_norm[1], biascorr_norm[2], fitquality[0], fitquality[1], fitquality[2]);
				      }
				  }//if redstep
			      } //if scriptcmd
			} //if calibfile
		  }
		  
		  scriptnrall = scriptnrallfound;
	      }
	    }
	  
            fclose(streampow);
	    if (repfiles == 1)
		fclose(streamsim);
	    fclose(streamscript);
	    if (redstep == 0) fclose(streamphotidx);
	    if (redstep > 0) fclose(streamamp);
	    if (redstep == 2) 
	      {
		fclose(streamcp);
		fclose(streamamp2);
		fclose(streambaseline);
	      }
	    
	    fclose(streaminfo);
	} //for redstep
 
        if (repfiles == 1) 
	  {
	    fclose(streammatrix);
	    fclose(streamkdeamp);
	    fclose(streamkdewid);
	  }
	
	free_dvector(y_raw_double0,0,nPtsScan-1);
	free_dvector(y_raw_double1,0,nPtsScan-1);
	free_dvector(filtered1,0,nPtsScan-1);
	free_dvector(filtered2,0,nPtsScan-1);
	free_dvector(power0,0,nPtsScan-1);
	free_dvector(power1,0,nPtsScan-1);
	free_dvector(power2,0,nPtsScan-1);
	free_dvector(poweravg0,0,nPtsScan-1);
	free_dvector(poweravg1,0,nPtsScan-1);
	free_dvector(poweravg2,0,nPtsScan-1);
	free_dvector(poweravg0bg,0,nPtsScan-1);
	free_dvector(poweravg1bg,0,nPtsScan-1);
	free_dvector(poweravg2bg,0,nPtsScan-1);
	free_dvector(envelope,0,nPtsScan-1);
	free_vector(y_raw,1,nPtsScan);

	free_dmatrix(corr, 0, 3, 0, nPtsScan-1);
	free_dmatrix(matrix4, 0, 5, 0, nPtsScan-1);
	
	fftw_destroy_plan(cwt_p);
	fftw_destroy_plan(cwt_p2);

	fftw_destroy_plan(fftw_photcorr_f);
	fftw_destroy_plan(fftw_photcorr_b);

	if (redstep == 1)
	  {
	    printf("#Fits with (KS-Prob=0):  %i  %i  %i\n", fitprobequal0[0], fitprobequal0[1], fitprobequal0[2]);
	    printf("#Fits with (KS-Prob!=0): %i  %i  %i\n\n\n", fitprobnot0[0], fitprobnot0[1], fitprobnot0[2]);
	  }

	if (simfringe == 1) printf("\nBias Log ChiSqr: %f      Sum: %f\n", biaslog, biaslog2);
	
      }

      if (createbiascorr == 1) fclose(streambias);


      //get calibrator list
      strcpy(filename, ProcDir); strcat(filename, "calib.ini");

      if (file_exist(filename) == 1)
      {
	  streamsre = fopen(filename, "r");
	  fscanf(streamsre, "%s %i %i %i %i %i %i %i %i %i %i", &temptext, &caliblist[0], &caliblist[1], &caliblist[2], &caliblist[3], &caliblist[4], &caliblist[5], &caliblist[6], &caliblist[7], &caliblist[8], &caliblist[9]);
	  fscanf(streamsre, "%s %i %i %i %i %i %i %i %i %i %i", &temptext, &srelist[0], &srelist[1], &srelist[2], &srelist[3], &srelist[4], &srelist[5], &srelist[6], &srelist[7], &srelist[8], &srelist[9]);
	  fscanf(streamsre, "%s %f %f %f %f %f %f %f %f %f %f", &temptext, &target_diameter[0], &target_diameter[1], &target_diameter[2], &target_diameter[3], &target_diameter[4], &target_diameter[5], &target_diameter[6], &target_diameter[7], &target_diameter[8], &target_diameter[9]);
	  fclose(streamsre);
      }



      //filter amp which are too high
      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "amp.dat");
      streamamp = fopen(ProcFilename, "r");

      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "ampfilter.dat");
      streamvis = fopen(ProcFilename, "w");

      while (feof(streamamp) == 0)
	{
	  fscanf(streamamp, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
	  fscanf(streamamp, streamampstring2, &biascorr[0], &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);
	  
	  for (i=0; i<3; i++)
	      if ((sreamp[i] > filterampthreshold) && (dofilteramp == 1))
	      {
		  sreamp[i] = -1.0;
		  sreamperror[i] = 0.0;
	      }
	  
	  is_target = 1;

	  for (i=0; i<max_targets; i++)
	      if (caliblist[i] == objid)
		  is_target = 0;

	  if ((is_target == 1) && (objid > 0)) targetid = objid;

	  fprintf(streamvis, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
	  fprintf(streamvis, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], corrfactor[2]);

	}

      fclose(streamamp);
      fclose(streamvis);


      strcpy(filename, ProcDir); strcat(filename, "ampfilter.dat");
      strcpy(filename2, ProcDir); strcat(filename2, "amp");
      separatecolors(filename, ProcFilename);


      printf("Use MainTargetID=#%i\n", targetid);
      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "target.ini");
      streamvis = fopen(ProcFilename, "w");
      fprintf(streamvis, "%i\n", targetid);
      fclose(streamvis);
      
      

      //*********************************
      //*** Stellar Radius estimation ***
      //*********************************

      if (wavelength == 0.0) wavelength = 1.65e-6;

      printf("\n*** Estimated target visibilities using calibrators:\n");


      //*** Determine calibrator-level for each night
      strcpy(filename, ProcDir); strcat(filename, "ampfilter.dat");
      strcpy(filename2, ProcDir); strcat(filename2, "ampcalib.dat");
      nr_calibset = calc_calibrator_levels(filename, filename2, 1, caliblist, srelist, (float *)avg_calib, (float *)avg_caliberror, (float *)calibset_vis, (float *)calibset_visscat, calibset_cp, calibset_cpscat, calibset_jd, calibset_objid, calibset_night, (float *)calibset_nr, calibset_from, calibset_to, nr_calibs);


      //*** Determine uv-limits and write them in file to plot it later
      umin = 1.0e10;
      umax = -1.0e10;
      vmin = 1.0e10;
      vmax = -1.0e10;

      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "ampfilter.dat");
      streamamp = fopen(ProcFilename, "r");

      while (feof(streamamp) == 0)
	{
	  fscanf(streamamp, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
	  fscanf(streamamp, streamampstring2, &biascorr[0], &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);
	  
	  //find min and max u/v-values to plot uv-plane
	  for (i=0; i<3; i++)
	    {
	      if (uavg[i] < umin) umin = uavg[i];
	      if (uavg[i] > umax) umax = uavg[i];
	      if (vavg[i] < vmin) vmin = vavg[i];
	      if (vavg[i] > vmax) vmax = vavg[i];
	    }
	}
      
      fclose(streamamp);

      if (abs2(umin) > abs2(umax)) umax = abs2(umin); 
      if (abs2(umax) > abs2(umin)) umin = -abs2(umax); 
      if (abs2(vmin) > abs2(vmax)) vmax = abs2(vmin); 
      if (abs2(vmax) > abs2(vmin)) vmin = -abs2(vmax); 

      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "uvlimits.dat");
      streamamp = fopen(ProcFilename, "w");
      fprintf(streamamp, "%f %f\n", umin, vmin);
      fprintf(streamamp, "%f %f\n", umax, vmax);
      fclose(streamamp);

      for (i=0;i<3;i++)
	for (n=0;n<nr_calibs[i];n++)
	  {
	    if ((calibset_vis[i][n] < 0.0) || (calibset_vis[i][n] > fitenv_max_vis))
	      calibset_vis[i][n] = -1.0;
	    
	    fprintf(streamlog, "Calc Calib #%i (Obj %i, ch %i) in night %i with MJD=%f using %i points to avg Vis %f\n", n, calibset_objid[n], i, calibset_night[n], calibset_jd[n], (int)calibset_nr[i][n], calibset_vis[i][n]);
	  }


      //*** Calculate the visibility for each target object
      basel_min = 1.0e10;
      basel_max = -1.0e10;

      for (i=0; i<=max_targets; i++)
	vispoints_nr[i] = -1;

      nr_night = 0;

      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "sre.dat");
      streamsre = fopen(ProcFilename, "w");

      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "ampfilter.dat");
      streamamp = fopen(ProcFilename, "r");

      while (feof(streamamp) == 0)
	{
	  fscanf(streamamp, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
	  fscanf(streamamp, streamampstring2, &biascorr[0], &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);

   	  if (abs2(newjd - oldjd) > 0.3) nr_night++;
	  oldjd = newjd;

	  is_target = 0;
	  
	  if (objid == targetid)
	  {
	      is_target = 1;
	      nr_target = i;
	  }
	  
	  if ((is_target == 1) && ((avg_calib[0][nr_night] + avg_calib[1][nr_night] + avg_calib[2][nr_night]) > 0.0))
	    {
	      for (i=0; i<3; i++)
		{
		  if (Viscalibmethod == 0)
		    calib_level = avg_calib[i][nr_night];
		  else
		    {
		      //Calculate the calibrator-level for this particular target
		      //search on the left
		      calib_level_l = -1.0;
		      calib_level_ljd = 1e-10;
		      for (n=0; n<nr_calibset; n++)
			if ((calibset_jd[n] < newjd) && (calibset_vis[i][n] != -1.0) && (calibset_jd[n] != 0.0) && (calibset_night[n] == nr_night))
			  {
			    calib_level_ljd = calibset_jd[n];
			    calib_level_l = calibset_vis[i][n];
			  }
		      
		      //search on the right
		      calib_level_r = -1.0;
		      calib_level_rjd = 1e10;
		      for (n=nr_calibset-1; n>=0; n--)
			if ((calibset_jd[n] > newjd) && (calibset_vis[i][n] != -1.0) && (calibset_night[n] == nr_night))
			  {
			    calib_level_rjd = calibset_jd[n];
			    calib_level_r = calibset_vis[i][n];
			  }
		      
		      if ((calib_level_l != -1.0) && (calib_level_r != -1.0)) 
			{
			  calib_level = calib_level_l + (calib_level_r-calib_level_l)*((newjd-calib_level_ljd)/(calib_level_rjd - calib_level_ljd));
			  fprintf(streamlog, "Calibrate V^2 @ MJD=%f (Obj %i, ch %i) with left (%f, Vis %f) and right calib. to %f (%f, Vis %f)\n", newjd, objid, i, calib_level_ljd, calib_level_l, calib_level, calib_level_rjd, calib_level_r);
			}
		      
		      if ((calib_level_l == -1.0) && (calib_level_r != -1.0))
			{
			  calib_level = calib_level_r;
			  fprintf(streamlog, "Calibrate V^2 @ MJD=%f (Obj %i, ch %i) with right calib. (%f, Vis %f)\n", newjd, objid, i, calib_level_rjd, calib_level);
			}
		      
		      if ((calib_level_l != -1.0) && (calib_level_r == -1.0))
			{
			  calib_level = calib_level_l;
			  fprintf(streamlog, "Calibrate V^2 @ MJD=%f (Obj %i, ch %i) with left calib. (%f, Vis %f)\n", newjd, objid, i, calib_level_ljd, calib_level);
			}
		      
		      if ((calib_level_l == -1.0) && (calib_level_r == -1.0)) fprintf(streamlog, "PROBLEM: No V^2 Calibrator found MJD=%f (Obj %i, ch %i)!\n", newjd, objid, i);
		      
		      
		      //calib_level = (calib_level + avg_calib[i][nr_night])/2;
		    }
		  

		  //we'll do the precise error estimation later for bispec.dat...
		  viserror[i] = sreamperror[i];


		  b = sqrt(sqr(uavg[i])+sqr(vavg[i]));
		  if (b < basel_min) basel_min = b;
		  if (b > basel_max) basel_max = b;

		  if ((vis[i] > 0.0)) // && (viserror[i] < 0.55))
		    {
		      vispoints_nr[nr_target]++;
		      vispoints[nr_target][vispoints_nr[nr_target]][0] = b;
		      vispoints[nr_target][vispoints_nr[nr_target]][1] = vis[i];
		      vispoints[nr_target][vispoints_nr[nr_target]][2] = viserror[i];
		    }
		  else
		    {
			//printf("Reject V^2 measurement (channel %i, Error %f)\n", i, viserror[i]);
		      vis[i] = -1.0;
		      viserror[i] = 0.0;
		    }
		}

	      if ((plot_target == objid) || (plot_target == 0))
		fprintf(streamsre, "%f %i  %f %f %f %f %f %f  %f %f %f\n", newjd, objid, vis[0], viserror[0], vis[1], viserror[1], vis[2], viserror[2], (sqrt(sqr(uavg[0])+sqr(vavg[0]))/100.0)/wavelength/1.0e6, (sqrt(sqr(uavg[1])+sqr(vavg[1]))/100.0)/wavelength/1.0e6, (sqrt(sqr(uavg[2])+sqr(vavg[2]))/100.0)/wavelength/1.0e6);
	    }
	}

      fclose(streamamp);
      fclose(streamsre);


      //Generate a model for the data
      found = 0;
      for (i=1; i<argc; i++) if (strcasecmp(argv[i], "final") == 0) found = i;

      if (found >= 1)
      {
	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "sremodel.dat");
	  streamsre = fopen(ProcFilename, "w");
	  
	  wavelength = 1.65e-6;
	  
	  for (i=0; i<max_targets; i++)
	  {
	      if ((plot_target == srelist[i]) || (plot_target == 0))
		  if ((vispoints_nr[i] > 0) && (srelist[i] > 0))
		  {
		      n = vispoints_nr[i];
		      fit_model_unif_disc2(wavelength, vispoints, i, n, &diameter, &diametererror);
		      printf("   Target #%2i: d=(%5.2f +/- %5.2f) mas (#Points=%4i)\n", srelist[i], diameter, diametererror, n);
		      
		      for (b=basel_min; b<=basel_max; b += 1)
		      {
			  v = vis_unif_disc(diameter, wavelength, b/100.0);
			  fprintf(streamsre, "%f %f\n", (b/100.0)/wavelength/1.0e6, v);
		      }
		  }
	  }
	  
	  fclose(streamsre);
	}

      //*** Correct the resolved calibrators
      printf("\n*** Correct Resolved Calibrators:\n");
      
      strcpy(filename, ProcDir); strcat(filename, "calib.ini");

      printf("Assuming Calibrator Diameters from '%s':\n", filename);
      for (i=0; i<10; i++)
	if (target_diameter[i] > 0)
	  printf("   Target #%2i: d=%6.3f mas\n", srelist[i], target_diameter[i]);
      printf("\n");


      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "ampfilter.dat");
      streamamp = fopen(ProcFilename, "r");
      
      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "ampcorr.dat");
      streamampcorr = fopen(ProcFilename, "w");

      while (feof(streamamp) == 0)
      {
	  fscanf(streamamp, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
	  fscanf(streamamp, streamampstring2, &biascorr[0], &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);

	  is_target = 0;
	  for (i=0; i<10; i++)
	      if (objid == srelist[i])
	      {
		  is_target = 1;
		  nr_target = i;
	      }

	  if (is_target == 1)
	      for (i=0; i<3; i++)
	      {
		  b = sqrt(sqr(uavg[i])+sqr(vavg[i]));
		  diameter = target_diameter[nr_target];

		  sreamp[i] /= sqr(vis_unif_disc(diameter, wavelength, b/100));
	      }

	  //correct V^2
/*
	  if ((objid == 2) && (1 == 0)) //BETA AUR
	  {
	      umax = 0.0;
	      vmax = 0.0;
	      delaymax = 0.0;

	      //determine delay for this configuration
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "betaaur.model");
	      streamdelay = fopen(ProcFilename, "r");
	      
	      delaymin = -1.0;
	      
	      while (feof(streamdelay) == 0)
	      {
		  fscanf(streamdelay, "%i %lf %lf  %f %f %f  %lf %lf %lf %lf\n", &scriptnrall2, &delaymjd, &delaymjd, &vis[0], &vis[1], &vis[2], &delaymjd, &delaymjd, &delaymjd, &delaymjd);
		  
		  if (vis[0] != -1.0)
		  {
		      if (scriptnrall == scriptnrall2)
		      {
			  printf("Perform BETA AUR-calibration for V^2 measurement #%i using ref. orbit #%i\n", scriptnrall, scriptnrall2);

			  printf("   %i   %f %f %f: (%f %f %f ", scriptnrall2, vis[0], vis[1], vis[2], sreamp[0], sreamp[1], sreamp[2]);

			  delaymin = scriptnrall2;
			  
			  for (i=0; i<3; i++)
			      if (vis[i] <= 0.0)
				  sreamp[i] = -1.0;
			      else
				  sreamp[i] /= vis[i];

			  printf("-> %f %f %f)\n", sreamp[0], sreamp[1], sreamp[2]);

		      }
		  }
	      }
	      
	      fclose(streamdelay);
	      
	      if (delaymin == -1.0)
		  printf("No BETA AUR-calibrator found for measurement #%i\n", scriptnrall);
	  }

	  //correct CP
	  if ((objid == 2) && (1 == 0))//BETA AUR
	  {
	      umax = 0.0;
	      vmax = 0.0;
	      delaymax = 0.0;
	      
	      //determine delay for this configuration
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "betaaur.model");
	      streamdelay = fopen(ProcFilename, "r");
	      
	      delaymin = -1.0;
	      
	      while (feof(streamdelay) == 0)
	      {
		  fscanf(streamdelay, "%i %lf %lf  %f %f %f  %lf %lf %lf %lf\n", &scriptnrall2, &delaymjd, &delaymjd, &vis[0], &vis[1], &vis[2], &delaymjd, &delaymjd, &delaymjd, &cpcalibrator);
		    
		  if (scriptnrall == scriptnrall2)
		  {
		      printf("Perform BETA AUR-calibration for CP measurement #%i using ref. orbit #%i\n", scriptnrall, scriptnrall2);
			
		      printf("   %i   %lf: (%f ", scriptnrall2, cpcalibrator, calibset_cp[m]);
			
		      delaymin = scriptnrall2;
		      calibset_cp[m] -= cpcalibrator;
			
		      printf("-> %f)\n", calibset_cp[m]);
		  }
	      }
		
	      fclose(streamdelay);
		
	      if (delaymin == -1.0)
		  printf("No BETA AUR-calibrator found for measurement #%i\n", scriptnrall);
	  }
*/

	  fprintf(streamampcorr, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
	  fprintf(streamampcorr, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0],corrfactor[1], corrfactor[2]);
      }
      fclose(streamamp);
      fclose(streamampcorr);


      //*** Determine calibrator-level for each night
      strcpy(filename, ProcDir); strcat(filename, "ampcorr.dat");
      strcpy(filename2, ProcDir); strcat(filename2, "bispeccalib.dat");
      nr_calibset = calc_calibrator_levels(filename, filename2, 0, caliblist, srelist, (float *)avg_calib, (float *)avg_caliberror, (float *)calibset_vis, (float *)calibset_visscat, calibset_cp, calibset_cpscat, calibset_jd, calibset_objid, calibset_night, (float *)calibset_nr, calibset_from, calibset_to, nr_calibs);


      printf("\nAveraged Fringe Power of the calibrators during each night:\n");
      for (n=1; n<=nr_night; n++)
	  printf("   Night #%i:  %12.5f    %12.5f    %12.5f\n", n, avg_calib[0][n], avg_calib[1][n], avg_calib[2][n]);
		 

      printf("\nScattering of the Fringe Power of the calibrators during each night:\n\n");
      for (n=1; n<=nr_night; n++)
	  printf("   Night #%i:  %12.5f    %12.5f    %12.5f\n", n, avg_caliberror[0][n], avg_caliberror[1][n], avg_caliberror[2][n]);

      //*** Determine Closure-Phase calibrator-level for each night
      for (n=0; n<nr_calibset; n++)
	{
	    for (m=0; m<nr_calibset; m++)
		if ((calibset_night[m] == n) && (calibset_cp[m] >= -PI) && ((calibset_cpscat[m] < filtercperror) || (dofilterscat == 0)))
		    printf("Night %i: CP calibrator #%i: %f\n", calibset_night[m], m, calibset_cp[m]*RToD);

	    //determine averaged CP for this night and scattering of calibrators
	    i = 0;
	    sum = 0.0;
	    sumsqr = 0.0;
	    cpmean[n] = 0.0;
	    for (m=0; m<nr_calibset; m++)
		if ((calibset_night[m] == n) && (calibset_cp[m] >= -PI) && ((calibset_cpscat[m] < filtercperror) || (dofilterscat == 0)))
		{
		    i++;
		    sum += calibset_cp[m];
		    sumsqr += (calibset_cp[m]*calibset_cp[m]);
		}
	    
	    if (i > 0)
	    {
		cpmean[n] = sum/i;
		if ((sumsqr - sum*sum/i)/i > 0.0)
		    cpscat[n] = sqrt((sumsqr - sum*sum/i)/i);
		else
		    cpscat[n] = 0.0;
		
		printf("\nNight %i: Subtract closure-phase offset using the mean calibrator CP of %f deg (Scattering: %f DEG).\n", n, cpmean[n]*RToD, cpscat[n]*RToD);
	    }
	}


      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "transf.dat");
      streamtransf = fopen(ProcFilename, "w");
      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "bispec.dat");
      streamvis = fopen(ProcFilename, "w");
      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "visnofilter.dat");
      streamvis2 = fopen(ProcFilename, "w");
      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "visnorot.dat");
      streamvis3 = fopen(ProcFilename, "w");
      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "ampcorr.dat");
      streamamp = fopen(ProcFilename, "r");
      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "borders.dat");
      streamborders = fopen(ProcFilename, "w");

      fprintf(streamborders, "%f %f\n", -1.0, -10.0);

      oldjd = 0;
      nr_night = 0;
      while (feof(streamamp) == 0)
	{
	  fscanf(streamamp, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
	  fscanf(streamamp, streamampstring2, &biascorr[0], &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);

	      
	  if (abs2(newjd - oldjd) > 0.3)
	    { 
	      nr_night++;
	      fprintf(streamborders, "%f %f\n", scriptnrall+0.4999, -10.0);
	      fprintf(streamborders, "%f %f\n", scriptnrall+0.5, 1000.0);
	      fprintf(streamborders, "%f %f\n", scriptnrall+0.5001, -10.0);
	    }
	  oldjd = newjd;
	  

   	  if (abs2(newjd - oldjd) > 0.3) nr_night++;
	  oldjd = newjd;

	  is_target = 0;
	  
	  if (objid == targetid)
	  {
	      is_target = 1;
	      nr_target = i;
	  }
	  
	  if ((avg_calib[0][nr_night] + avg_calib[1][nr_night] + avg_calib[2][nr_night]) > 0.0)
	  {
	      //Calibrate V^2
	      for (i=0; i<3; i++)
		{
        	  if ((Viscalibmethod == 0) || (is_target == 0))
		    calib_level = avg_calib[i][nr_night];
		  else
		    {
		      //Calculate the calibrator-level for this particular target
		      //search on the left
		      calib_level_l = -1.0;
		      calib_level_ljd = 1e-10;
		      for (n=0; n<nr_calibset; n++)
			if ((calibset_jd[n] < newjd) && (calibset_vis[i][n] != -1.0) && (calibset_jd[n] != 0.0) && (calibset_night[n] == nr_night))
			  {
			    calib_level_ljd = calibset_jd[n];
			    calib_level_l = calibset_vis[i][n];
			  }
		      
		      //search on the right
		      calib_level_r = -1.0;
		      calib_level_rjd = 1e10;
		      for (n=nr_calibset-1; n>=0; n--)
			if ((calibset_jd[n] > newjd) && (calibset_vis[i][n] != -1.0) && (calibset_night[n] == nr_night))
			  {
			    calib_level_rjd = calibset_jd[n];
			    calib_level_r = calibset_vis[i][n];
			  }
		      
		      if ((calib_level_l != -1.0) && (calib_level_r != -1.0))
			{
			  calib_level = calib_level_l + (calib_level_r-calib_level_l)*((newjd-calib_level_ljd)/(calib_level_rjd - calib_level_ljd));
			  fprintf(streamlog, "Calibrate V^2 @ MJD=%f (Obj %i, ch %i) with left (%f, Vis %f) and right calib. to %f (%f, Vis %f)\n", newjd, objid, i, calib_level_ljd, calib_level_l, calib_level, calib_level_rjd, calib_level_r);
			}
		      
		      if ((calib_level_l == -1.0) && (calib_level_r != -1.0))
			{
			  calib_level = calib_level_r;
			  fprintf(streamlog, "Calibrate V^2 @ MJD=%f (Obj %i, ch %i) with right calib. (%f, Vis %f)\n", newjd, objid, i, calib_level_rjd, calib_level);
			}
		      
		      if ((calib_level_l != -1.0) && (calib_level_r == -1.0))
			{
			  calib_level = calib_level_l;
			  fprintf(streamlog, "Calibrate V^2 @ MJD=%f (Obj %i, ch %i) with left calib. (%f, Vis %f)\n", newjd, objid, i, calib_level_ljd, calib_level);
			}
		      
		      if ((calib_level_l == -1.0) && (calib_level_r == -1.0)) fprintf(streamlog, "PROBLEM: No V^2 Calibrator found MJD=%f (Obj %i, ch %i)!\n", newjd, objid, i);
		    }

		  if (calib_level > 0.0)
		    {
			if (avg_calib[i][nr_night] > 0.0)
			{
			    if (calib_level == 0.0)
			    {
				vis[i] = -1.0;
				viserror[i] = 0.0;
			    }
			    else
			    {
				vis[i] = sreamp[i]/calib_level;
				if ((addcaliberror == 1) && (avg_caliberror[i][nr_night] > 0.0))
				    sreamperror[i] = sreamperror[i]/calib_level + avg_caliberror[i][nr_night]/calib_level + adderrorVis;
				else
				    sreamperror[i] = sreamperror[i]/calib_level + adderrorVis;
			    }
			}
			

			sreamp[i] /= calib_level;
			biascorr[i] = calib_level;

			calib_level3[i] = calib_level;
		    }
		  else
		    sreamp[i] = -0.05;
		}
	    }

	  //calibrate CP
	  if (CPcalibmethod == 0)
	  {
	      mean -= cpmean[nr_night];
	      variance = variance/(2.0*PI) + cpscat[nr_night] + adderrorCP;	  
	  }
	  else
	  {
	      //Calculate the calibrator-level for this particular target
	      //search on the left
	      calib_level_l = -4.0;
	      calib_level_ljd = 1e-10;
	      for (n=0; n<nr_calibset; n++)
		  if ((calibset_jd[n] < newjd) && (calibset_cp[n] > -4.0) && (calibset_jd[n] != 0.0) && (calibset_night[n] == nr_night))
		  {
		      calib_level_ljd = calibset_jd[n];
		      calib_level_l = calibset_cp[n];
		  }
		      
	      //search on the right
	      calib_level_r = -4.0;
	      calib_level_rjd = 1e10;
	      for (n=nr_calibset-1; n>=0; n--)
		  if ((calibset_jd[n] > newjd) && (calibset_cp[n] > -4.0) && (calibset_night[n] == nr_night))
		  {
		      calib_level_rjd = calibset_jd[n];
		      calib_level_r = calibset_cp[n];
		  }
	      
	      if ((calib_level_l > -4.0) && (calib_level_r > -4.0))
	      {
		  calib_level = calib_level_l + (calib_level_r-calib_level_l)*((newjd-calib_level_ljd)/(calib_level_rjd - calib_level_ljd));
		  fprintf(streamlog, "Calibrate CP @ MJD=%f (Obj %i) with left (%f, CP %f) and right calib. to %f (%f, CP %f)\n", newjd, objid, calib_level_ljd, calib_level_l, calib_level, calib_level_rjd, calib_level_r);
	      }
	      
	      if ((calib_level_l == -4.0) && (calib_level_r > -4.0))
	      {
		  calib_level = calib_level_r;
		  fprintf(streamlog, "Calibrate CP @ MJD=%f (Obj %i) with right calib. (%f, Vis %f)\n", newjd, objid, calib_level_rjd, calib_level);
	      }
	      
	      if ((calib_level_l > -4.0) && (calib_level_r == -4.0))
	      {
		  calib_level = calib_level_l;
		  fprintf(streamlog, "Calibrate CP @ MJD=%f (Obj %i) with left calib. (%f, Vis %f)\n", newjd, objid, calib_level_ljd, calib_level);
	      }
	      
	      if ((calib_level_l == -4.0) && (calib_level_r == -4.0)) fprintf(streamlog, "PROBLEM: No CP Calibrator found MJD=%f (Obj %i, ch %i)!\n", newjd, objid, i);
	      
	      if (calib_level > -4.0)
	      {
		  mean -= calib_level;
		  variance = variance + cpscat[nr_night] + adderrorCP;
		  //variance = variance/(2.0*PI) + cpscat[nr_night] + adderrorCP;
		  //variance = sqrt(sqr(variance) + sqr(cpscat[nr_night]) + sqr(adderrorCP));
	      }
	      else
		  mean = -4.0;
	  }


	  if (objid == targetid)
	  {
	      fprintf(streamtransf, streamampstring1, scriptnrall, newjd, objid, calib_level3[0], 0.0, 0.0, calib_level3[1], 0.0, 0.0, calib_level3[2], 0.0, 0.0, uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], calib_level, variance, calibtag);
	      fprintf(streamtransf, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], corrfactor[2]);
	  }
 
	  fprintf(streamvis, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
	  fprintf(streamvis, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], corrfactor[2]);

	  fprintf(streamvis2, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
	  fprintf(streamvis2, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], corrfactor[2]);

	  fprintf(streamvis3, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
	  fprintf(streamvis3, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], corrfactor[2]);
	}


      fclose(streamtransf);
      fclose(streamamp);
      fclose(streamvis);
      fclose(streamvis2);
      fclose(streamvis3);
      fclose(streamborders);



      //*** Filter bad data
      if (1 == 1)
	{
	  printf("\n*** Filter groups with high scattering in V^2 or CP.\n");

	  for (i=1; i<10; i++)
	    caliblist[i] = 0;
	  caliblist[0] = targetid;
	  
	  strcpy(filename, ProcDir); strcat(filename, "bispec.dat");
	  strcpy(filename2, ProcDir); strcat(filename2, "bispeccalib.dat");
	  nr_calibset = calc_calibrator_levels(filename, filename2, 2, caliblist, srelist, (float *)avg_calib, (float *)avg_caliberror, (float *)calibset_vis, (float *)calibset_visscat, calibset_cp, calibset_cpscat, calibset_jd, calibset_objid, calibset_night, (float *)calibset_nr, calibset_from, calibset_to, nr_calibs);
	  
	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "bispec.dat");
	  streamvis = fopen(ProcFilename, "w");
	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "bispec_maintarget.dat");
	  streamvis4 = fopen(ProcFilename, "w");
	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "visnorot.dat");
	  streamvis2 = fopen(ProcFilename, "w");
	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "vissimple.dat");
	  streamvis3 = fopen(ProcFilename, "w");
	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "visnofilter.dat");
	  streamamp = fopen(ProcFilename, "r");

	  
	  while (feof(streamamp) == 0)
	    {
	      fscanf(streamamp, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
	      fscanf(streamamp, streamampstring2, &biascorr[0], &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);
	      
	      if (dofilterscat == 1)
		  for (i=0; i<=nr_calibset; i++)
		      for (channel=0; channel<3; channel++)
			  if ((calibset_from[i]<=scriptnrall) && (calibset_from[i] != -1) && (calibset_to[i] != -1) && (calibset_to[i]>=scriptnrall))
			      if (calibset_visscat[channel][i] > filtervisscatthreshold) 
			      {
				  sreamp[channel]=-0.2;
				  sreamperror[channel]=0.0;
			      }

	      if (variance > filtercperror) 
	      {
		  mean = -4.1;
		  variance = 0.0;
	      }

	      if (sreamperror[0] > filterviserror) { sreamp[0] = -0.3; sreamperror[0] = 0.0; };
	      if (sreamperror[1] > filterviserror) { sreamp[1] = -0.3; sreamperror[1] = 0.0; };
	      if (sreamperror[2] > filterviserror) { sreamp[2] = -0.3; sreamperror[2] = 0.0; };
		  
	      fprintf(streamvis, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
	      fprintf(streamvis, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], corrfactor[2]);
	      
	      fprintf(streamvis2, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
	      fprintf(streamvis2, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], corrfactor[2]);

	      if (objid == targetid)
	      {
		  fprintf(streamvis3, "%i %f   %f %f  %f %f  %f %f    %f %f   %f %f  %f %f  %f %f\n", scriptnrall, newjd, sreamp[0], sreamperror[0], sreamp[1], sreamperror[1], sreamp[2], sreamperror[2], mean, variance, uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2]);

		  fprintf(streamvis4, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
		  fprintf(streamvis4, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], corrfactor[2]);
	      }
	    }
	  
	  fclose(streamamp);
	  fclose(streamvis);
	  fclose(streamvis2);
	  fclose(streamvis3);
	  fclose(streamvis4);


	  //correct bias
	  if (dobiascorr == 2)
	  {
	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "visbias.dat");
	      streamvis = fopen(ProcFilename, "w");

	      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "bispec.dat");
	      streamamp = fopen(ProcFilename, "r");
	      
	      while (feof(streamamp) == 0)
	      {
		  fscanf(streamamp, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
		  fscanf(streamamp, streamampstring2, &biascorr[0], &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);
		  
		  for (channel=0; channel<3; channel++)
		      if (sreamp[channel] < 1.0)
			  sreamp[channel] = calc_biascorr(channel, sreamp[channel], biascorr[channel]);
		  
		  fprintf(streamvis, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
		  fprintf(streamvis, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], wavelength*1.0e6); //corrfactor[2]);
		  
		  fprintf(streamvis2, streamampstring1, scriptnrall, newjd, objid, sreamp[0], sreamperror[0], sreampprob[0], sreamp[1], sreamperror[1], sreampprob[1], sreamp[2], sreamperror[2], sreampprob[2], uavg[0], vavg[0], uavg[1], vavg[1], uavg[2], vavg[2], mean, variance, calibtag);
		  fprintf(streamvis2, streamampstring2, biascorr[0], biascorr[1], biascorr[2], corrfactor[0], corrfactor[1], wavelength*1.0e6); //corrfactor[2]);
	      }
	      
	      fclose(streamamp);
	      fclose(streamvis);
	  }	      
	  
	  
	  //*** bin data
	  printf("Binning data...\n");
	  
	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "bispec_maintarget_bin.dat");
	  streambin = fopen(ProcFilename, "w");

	  strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "bispec_maintarget.dat");
	  streamamp = fopen(ProcFilename, "r");
	  
	  startjd = -1.0e10;
	  count = 0;

	  for (i=0; i<3; i++)
	  {
	      binV2[i] = 0.0;
	      binV2err[i] = 0.0;
	      binuavg[i] = 0.0;
	      binvavg[i] = 0.0;
	  }
	  binCP = 0.0;
	  binCPerr = 0.0;
	  binJD = 0;
	  
	  while (feof(streamamp) == 0)
	  {
	      fscanf(streamamp, streamampstring1, &scriptnrall, &newjd, &objid, &sreamp[0], &sreamperror[0], &sreampprob[0], &sreamp[1], &sreamperror[1], &sreampprob[1], &sreamp[2], &sreamperror[2], &sreampprob[2], &uavg[0], &vavg[0], &uavg[1], &vavg[1], &uavg[2], &vavg[2], &mean, &variance, &calibtag);
	      fscanf(streamamp, streamampstring2, &biascorr[0], &biascorr[1], &biascorr[2], &corrfactor[0], &corrfactor[1], &corrfactor[2]);

	      if (objid == targetid)
	      {
		  if (startjd == -1.0e10) startjd = newjd;

		  if (((newjd - startjd) > bininterval/60.0/24.0) && (count > 0))
		  {
		      printf("  |  ", scriptnrall);
		      for (i=0; i<3; i++)
		      {
			  if (binV2err[i] > 0.0)
			  {
			      binV2[i] /= binV2err[i];
			      binV2err[i] = sqrt(1.0/binV2err[i]);
			  }
			  else
			  {
			      binV2[i] = -1.0;
			      binV2err[i] = -0.0;
			  }
			  binuavg[i] /= (float)count;
			  binvavg[i] /= (float)count;
		      }

		      
		      if (binCPerr > 0.0)
		      {
			  if (count == 1)
			  {
			      binCP = cpdata[1];
			      binCPerr = cpdataerr[1];
			  }
			  else
			  {
			      average_cp(&cpdata[0], count, &binCP, &binCPerr);

			      if (binCPerr == 0.0) binCPerr = 0.01;
			      binCPerr += cpscat[nr_night] + adderrorCP;
			  }
		      }
		      else
		      {
			  binCP = -4.1;
			  binCPerr = 0.0;
		      }

		      binJD /= (float)count;
		      
/*
		      if (binCPerr > PI/4.0)
		      {
			  binCP = -4.1;
			  binCPerr = 0.0;
		      }
*/


		      if ((adddelayerror == 1) && (1 == 0))
		      {
			  //set binary vector
			  if ((binJD > 90.5) && (binJD < 91.5)) { sepRA =  -8.93; sepDEC = 44.70; };
			  if ((binJD > 91.5) && (binJD < 92.5)) { sepRA = -11.67; sepDEC = 43.25; };
			  if ((binJD > 92.5) && (binJD < 93.5)) { sepRA = -14.47; sepDEC = 41.60; };
			  if ((binJD > 93.5) && (binJD < 94.5)) { sepRA = -17.33; sepDEC = 39.72; };
			  if ((binJD > 94.5) && (binJD < 95.5)) { sepRA = -20.13; sepDEC = 37.68; };
			  
			  umax = 0.0;
			  vmax = 0.0;
			  delaymax = 0.0;

			  for (i=0; i<3; i++) {
			      //determine delay for this configuration
			      strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "newcapella.delays");
			      streamdelay = fopen(ProcFilename, "r");

			      delaydiffmin = 1.0e10;

			      while (feof(streamdelay) == 0)
			      {
				  fscanf(streamdelay, "%lf %f %f %f\n", &delaymjd, &delayu, &delayv, &delay);
			      
				  if (fabs(delaymjd - (52500.0+newjd)) < 0.01)
				  {
				      delaydiff = sqrt(fsqr(delayu-binuavg[i]/100.0) + fsqr(delayv-binvavg[i]/100.0));
				      
				      if (delaydiff < delaydiffmin)
				      {
					  delaydiffmin = delaydiff;
					  delaymin = fabs(delay);
					  delayumin = delayu;
					  delayvmin = delayv;
				      }
				  }
			      }
			      
			      if (fabs(delaydiffmin) > 0.1)
			      {
				  printf("CAREFULL: DELAYDIFF=%f\n", delaydiffmin);
				  
				  delaymin = -1000.0;
			      }

			      if (delaymin > delaymax) delaymax = delaymin;

			      fclose(streamdelay);

			      //binV2err[i] += 0.01 / 1000.0 * sqrt(fsqr(binuavg[i]*sepRA) + fsqr(binvavg[i]*sepDEC));
			      //binV2err[i] += 0.005*delaymin;
			      //printf("AddErrV2: %f\n", 0.005*delaymin);
			      
			      if (binuavg[i] > umax) umax = binuavg[i];
			      if (binvavg[i] > vmax) vmax = binvavg[i];

			      delaymin_channel[i] = delaymin;
			  }
		      
			  //binCPerr += DToR * 2.0 / 1000.0 * sqrt(fsqr(umax*sepRA) + fsqr(vmax*sepDEC));
			  //binCPerr += DToR*2.0*delaymax;
			  //printf("AddErrCP: %f\n", 2.0*delaymax);
		      }

		      fprintf(streambin, streamampstring1, binscriptnrall, binJD, objid, binV2[0], binV2err[0], 0.0, binV2[1], binV2err[1], 0.0, binV2[2], binV2err[2], 0.0, binuavg[0], binvavg[0], binuavg[1], binvavg[1], binuavg[2], binvavg[2], binCP, binCPerr, calibtag);
		      fprintf(streambin, streamampstring2, 0.0, 0.0, delaymin_channel[0], delaymin_channel[1], delaymin_channel[2], wavelength*1.0e6);
		      
		      for (i=0; i<3; i++)
		      {
			  binV2[i] = 0.0;
			  binV2err[i] = 0.0;
			  binuavg[i] = 0.0;
			  binvavg[i] = 0.0;
		      }
		      binCP = 0.0;
		      binCPerr = 0.0;
		      binscriptnrall = 0;

		      binJD = 0.0;
		      count = 0;
		      startjd = newjd;
		  }
		  
		  for (i=0; i<3; i++)
		  {
		      if (sreamp[i] > 0.0)
		      {
			  w[i] = 1.0/(sreamperror[i]*sreamperror[i]);
			  binV2[i] += w[i]*sreamp[i];
			  binV2err[i] += w[i];
		      }  

		      binuavg[i] += uavg[i];
		      binvavg[i] += vavg[i];
		  }
		  
		  if (mean >= -PI)
		  {
		      w[0] = 1.0/(variance*variance);
		      binCP += w[0]*mean;
		      binCPerr += w[0];

		      cpdata[count+1] = mean;
		      cpdataerr[count+1] = variance;
		  }

		  if (binscriptnrall == 0)
		      binscriptnrall = scriptnrall;

		  binJD += newjd;
		  count++;
		  oldobjid = objid;
	      }
	  }
	  
	  fclose(streamamp);
	  fclose(streambin);
	}

      fclose(streamlog);

      strcpy(filename, ProcDir); strcat(filename, "bispec_maintarget_bin.dat");
      strcpy(filename2, ProcDir); strcat(filename2, "bis");
      separatecolors(filename, filename2);

      strcpy(filename, ProcDir); strcat(filename, "bispec.dat");
      strcpy(filename2, "!"); strcat(filename2, ProcDir); strcat(filename2, "bispec.fits");
      convert2fitsoi(filename, filename2);

      strcpy(filename, ProcDir); strcat(filename, "bispec_maintarget.dat");
      strcpy(filename2, "!"); strcat(filename2, ProcDir); strcat(filename2, "bispec_maintarget.fits");
      convert2fitsoi(filename, filename2);

      strcpy(filename, ProcDir); strcat(filename, "bispec_maintarget_bin.dat");
      strcpy(filename2, "!"); strcat(filename2, ProcDir); strcat(filename2, "bispec_maintarget_bin.fits");
      convert2fitsoi(filename, filename2);

      //PGPLOT: close windows
      if (doplot0 == 1) { cpgslct(cpg_window0); cpgclos(); }
      if (doplotbg == 1) { cpgslct(cpg_windowbg); cpgclos(); }

      if ((doplot0 == 1) || (doplot1 == 1) || (doplot2 == 1) || (doplot3 == 1) || (doplotbg == 1))
	  cpgend();

      strcpy(filename, ProcDir); strcat(filename, "ampcorr.dat");
      strcpy(filename2, ProcDir); strcat(filename2, "ampcorr");
      separatecolors(filename, filename2);

      printf("\n\nDone.\n");

      return 1;
}
