/* kdeinc.c                                  */
/* Interferometry Data Reduction Software    */
/* by Stefan Kraus                           */
/* 2002-05                                   */
/*                                           */
/* copyright laws apply                      */


#include <stdio.h>
#include <fftw-int.h>
#include <math.h>

#define pi         (double)3.14159265358979323846264

#define nrdatakde  1000
#define Nkde       1024//8192
#define ukde       0.01
#define vkde       0.01

double sqr(double x)
{
  return (x*x);
}

double gaussian(double x, double w)
{
  return (exp(-sqr(x)/(2*w))/(sqrt(2*pi*w)));
}

double FTgaussian(double x, double w)
{
  return (exp(-2*sqr(x*pi*w)));
}

double kde(char *filename, float kernelwidth, double *outkde, double *scalemin, double *scalemax, int cpg_window)
{
     fftw_complex nj[Nkde], Nk[Nkde], out[Nkde], in2[Nkde], sj[Nkde];
     float data[nrdatakde];
     double datamin, datamax, nrbin;
     double mean, rootvar, x, sum;
     long nrdata;
     fftw_plan p;
     int i;
     float datax[Nkde], datay[Nkde];
     float ymin, ymax;
     FILE *stream;


     stream = fopen(filename, "r");

     nrdata = 0;
     datamin = 1e308;
     datamax = -1e308;
     for (i=0; i<nrdatakde; i++)
       {
	 if (feof(stream))
	   data[i] = 0;
	 else
	   {
	     nrdata++;
	     fscanf(stream, "%f\n", &data[i]);
	     if (data[i] > datamax) datamax = data[i];
	     if (data[i] < datamin) datamin = data[i];
	   }
       }

     fclose(stream);


     //*** Calculate the Root Variance
     mean = 0;
     for (i=0; i<nrdata; i++)
       mean += data[i];
     mean /= nrdata;

     rootvar = 0.0;
     for (i=0; i<nrdata; i++)
       rootvar += sqr(data[i] - mean);
     rootvar = sqrt(rootvar / (float)(nrdata-1));

     datamin = datamin-0.3*(datamax-datamin);
     datamax = datamax+0.3*(datamax-datamin);

     *scalemin = datamin;
     *scalemax = datamax;


     for (i=0; i<Nkde; i++)
       {
	 nj[i].re = 0;
	 nj[i].im = 0;
       }

     for (i=0; i<nrdata; i++)
       {
	 nrbin = Nkde*(data[i]-datamin)/(datamax-datamin);
	 nj[(int)nrbin].re += 1;
       }


     for (i=0; i<Nkde; i++)
	 out[i].re = FTgaussian((datamax-datamin)*i/Nkde, kernelwidth);


     //*** Forward transformation
     p = fftw_create_plan(Nkde, FFTW_FORWARD, FFTW_ESTIMATE);
     fftw_one(p, nj, Nk);
     fftw_destroy_plan(p);

     //*** Multiplication
     for (i=0; i<Nkde; i++)
       {
	 in2[i].re = Nk[i].re*out[i].re;
	 in2[i].im = Nk[i].im*out[i].re;
       }
     
     //*** Backward transformation
     p = fftw_create_plan(Nkde, FFTW_BACKWARD, FFTW_ESTIMATE);
     fftw_one(p, in2, sj);
     fftw_destroy_plan(p);

     //** Normalizing
     sum = 0;
     for (i=0; i<Nkde; i++)
       sum += sj[i].re;
     for (i=0; i<Nkde; i++)
       sj[i].re /= sum;
     

     stream = fopen("kde.dat", "w");

     for (i=0; i<Nkde; i++)
       {
	 x = datamin + (datamax-datamin)*i/Nkde;
	 fprintf(stream, "%f %f\n", x, sj[i].re);
	 outkde[i] = sj[i].re;
       }

     fclose(stream);

     return rootvar;
}
