/* kstinc.c                                  */
/* Interferometry Data Reduction Software    */
/* by Stefan Kraus                           */
/* 2002-05                                   */
/*                                           */
/* copyright laws apply                      */


#include <stdio.h>
#include <math.h>
#define NRANSI
#include "erfcc.c"
#include "ran1.c"
#include "gasdev.c"
#include "probks.c"
#include "ksone.c"


#define NPTS 2000

float gaussianwidth;


float ksone_gaussian(float x)
{
  return 1.0 - erfcc(x/sqrt(gaussianwidth));
}


double kst(char *filename, double *fitpos, double fitwid)
{
  unsigned long nrdata;
  FILE *stream;
  
  float datamin, datamax;
  int i,j;
  float d,prob,varnce,*data,x;

  float mean, var, maxprob, maxprob_varnce;

  
  data=vector(1,NPTS);
  

  //*** Read data
  stream = fopen(filename, "r");
  
  mean = 0;
  nrdata = 1;
  datamin = 1e308;
  datamax = -1e308;
  for (i=0; i<NPTS; i++)
    {
      if (feof(stream))
	data[i] = 0.0;
      else
	{
	  fscanf(stream, "%f\n", &x);
	  
	  if (x != 0.0)
	    {
	      data[nrdata] = x;
	      mean += data[nrdata];
	      nrdata++;

	      if (data[nrdata] > datamax) datamax = data[nrdata];
	      if (data[nrdata] < datamin) datamin = data[nrdata];
	    }
	}
    }


  nrdata--;
  mean /= nrdata;
  
  *fitpos = mean;

  var = 0;
  for (j=1;j<=nrdata;j++)
    {
      var += (data[j] - mean)*(data[j] - mean);
      data[j] = fabs(data[j] - mean);
    }
  var = var/nrdata;
  var = sqr(fitwid);
  
  fclose(stream);
  

  maxprob = 0.0;
  
  for (i=1;i<=30;i++) {
    gaussianwidth = var*(1.0+(float)(i-10)/30.0);
    varnce = gaussianwidth;
    
    ksone(data, nrdata, ksone_gaussian, &d, &prob);
    
    if (prob > maxprob) 
      {
	maxprob = prob;
	maxprob_varnce = var;
      }
  }
  free_vector(data,1,NPTS);


  printf("   KS-Test: #Points: %3i  Var: %8.6f MaxProb: %6.3f at Var %8.6f\n", nrdata, var, maxprob, maxprob_varnce);
  
  return maxprob;
}
