/* wavelet.c                                 */
/* Interferometry Data Reduction Software    */
/* by Stefan Kraus                           */
/* 2002-05                                   */
/*                                           */
/* copyright laws apply                      */


#define wavelet(ia,ja)           pwavelet[(ia)*(nPtsWavelets)+(ja)]
#define intmap(ia,ja)            pintmap[(int)((ja) + (nPtsWavelets)*(ia))]

#include "fillcwt.c"

#define printps            0

#define v0                 1.0         //wavenumber/(2*PI)
#define Wavelet_s_factor   0.07

//CWT-algorithm using FFT
#define CWT_s_interval     0.038             //0.05
#define CWT_s_start        1.0              //1.0
#define CWT_morlet_waves   6.0              //6.0  //wavenumber
#define npad               1024             //2*nPtsScan;
#define npad_min           ((int)((npad/2 - nPtsScan)/2))
#define npad_max           ((int)((npad/2 - nPtsScan)/2+nPtsScan))

float waveletre[npad][nPtsWavelets], waveletim[npad][nPtsWavelets];
float waveletamp;

float motherwavelet[npad], motherwavelet_ft[npad];




//Morlet's wavelet

float func_wavelet_re(float x)
{
  return (exp(-x*x/2)*cos(2.0*PI*v0*x));
}

float func_wavelet_im(float x)
{
  return (exp(-x*x/2)*sin(2.0*PI*v0*x));
}

/*
float fourier_wavelet(float x)
{
  return (sqrt(2.0*PI)*(exp(sqr(2.0*PI*x-5.0)/2.0)+exp(sqr(2.0*PI*x+5.0)/2.0)));
}
*/


void generate_motherwavelet()
{
    int k;
    float *y_raw1, *y_raw2;
    FILE *stream;
    
    y_raw1 = vector(1,npad);
    y_raw2 = vector(1,npad);
    
    genFringeAmplitude = 1.0;
    ADDITIVE_NOISE = 0.0;
    
    generateIdealPacket(y_raw1, 1, npad);
    generateIdealPacket(y_raw2, 1, npad);
    
    strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "mother.dat");
    stream = fopen(ProcFilename, "w");
    
    for (k=0; k<npad; k++)
    {
	fprintf(stream, "%i %f\n", k, y_raw1[k]);
    }

    fclose(stream);
}


void wave_function(int nk, double dt, double param, double scale1, double *kwave, double *period1, double *coi1, double *daughter)
{
  double norm, expnt, fourier_factor;
  int k;

  norm = sqrt(2.0*PI*scale1/dt)*(pow(PI, -0.25));
  for (k=1; k<=nk/2+1; k++)
    {
      expnt = -0.5*pow(scale1*kwave[k-1] - param, 2.0);
      daughter[k-1] = norm*exp(expnt);
      //printf("%f  ", daughter[k-1]);
      //printf("%f  %f  %f  ->  %f\n", (float)scale1, (float)kwave[k-1], (float)param, (float)expnt);
    }

  for (k=nk/2+2; k<=nk; k++)
    {
      daughter[k-1] = 0.0;
    }

  fourier_factor = (4.0*PI)/(param + sqrt(2.0+sqr(param)));
  *period1 = scale1*fourier_factor;
  *coi1 = fourier_factor/sqrt(2.0);
}


void cwt(fftw_plan cwt_p, fftw_plan cwt_p2, double *data, float *pwavelet, float s_factor, float *min, float *max)
{
  int jtot=nPtsWavelets;

  double kwave[npad], period[jtot], scale[jtot], daughter[npad];
  double freq1, dt, dj, s0, param, period1, coi1, J;
  int x, i, j, j2, k;

  fftw_complex in[npad], out[npad], in2[npad], out2[npad];


  dt = 50.0/(double)nPtsScan;
  dj = CWT_s_interval/s_factor;
  param = CWT_morlet_waves; //wavenumber


  //if (s_factor != 1.0)
  s0 = 1.5*dt;
  //else
  //s0 = CWT_s_start*dt;

  *min = 1.0e10;
  *max = -1.0e10;


  //*** FFT of the signal
  for (x=0; x<npad; x++)
  {
      in[x].im = 0.0;

      if ((x >= npad_min) && (x < npad_max))
	  in[x].re = data[x-npad_min];
      else
	  in[x].re = 0.0;
  }


  fftw_one(cwt_p, in, out);
  
  //construct the wavenumber array
  freq1 = 2.0*PI/((double)npad*dt);
  kwave[1-1] = 0.0;
  for (i=2; i<=npad/2+1; i++)
    kwave[i-1] = ((double)i-1.0)*freq1;
  for (i=npad/2+2; i<=npad; i++)
    kwave[i-1] = -kwave[npad-i+2-1];


  for (j=1; j<=jtot; j++)
    {
      scale[j-1] = s0*(pow(2.0, (double)(j-1)*dj));
      //if ((j==1) || (j==jtot))
      //printf("Wavelet-Scale %i: %f\n", j, scale[j-1]);

      wave_function(npad, dt, param, scale[j-1], kwave, &period1, &coi1, daughter);
      period[j-1] = period1;
      //if ((j==1) || (j==jtot))
      //printf("Wavelet-Period %i: %f\n", j, period[j-1]);

      for (k=0; k<npad; k++)
	{
	  //printf("%f   ", daughter[k]);
	  in2[k].re = out[k].re*daughter[k];
	  in2[k].im = out[k].im*0.0;
	}

      fftw_one(cwt_p2, in2, out2);

      j2 = j-1;

      for (i=npad_min; i<npad_max; i++)
	{
	  waveletre[i][j2] = out2[i].re;
	  waveletim[i][j2] = out2[i].im;
	  wavelet(i, j2) = sqr(sqrt(sqr(out2[i].re)+sqr(out2[i].im)));

	  if (wavelet(i, j2) < *min) *min = wavelet(i, j2);
	  if (wavelet(i, j2) > *max) *max = wavelet(i, j2);
	}
    }

}



void remove_artefacts_topbottom(float *pwavelet, int maxintx, int maxints, double *amp_removed)
{
  int x, s, s2;
  int partition, partition2, endpartition;
  int reachednull;

  //to the bottom
  partition = 0;
  endpartition = 0;

  for (x = maxintx; x < npad_max; x++)
    {
      partition2 = 0;
      reachednull = 0;
      
      for (s = 0; s < nPtsWavelets; s++)
	if (wavelet(x, s) > 0.0)
	{
	    if (reachednull == 1)
	    {
		endpartition = 1;
		for (s2 = 0; s2 < nPtsWavelets; s2++)
		    wavelet(x, s2) = 0;
	    }

	    if (endpartition == 1)
	    {
		*amp_removed += wavelet(x, s);
		wavelet(x, s) = 0;
	    }
	    
	    partition2 = 1;
	}
	else
	{
	    if (partition2 == 1)
	    {
		reachednull = 1;
	    }
	}
    }

  //to the top
  partition = 0;
  endpartition = 0;

  for (x = maxintx-1; x >= 0; x--)
    {
      partition2 = 0;
      reachednull = 0;

      for (s = 0; s < nPtsWavelets; s++)
	if (wavelet(x, s) > 0.0)
	{
	    if (reachednull == 1)
	    {
		endpartition = 1;
		for (s2 = 0; s2 < nPtsWavelets; s2++)
		    wavelet(x, s2) = 0;
	    }

	    if (endpartition == 1)
	    {
		*amp_removed += wavelet(x, s);
		wavelet(x, s) = 0;
	    }

	    partition2 = 1;
	}
	else
	{
	    if (partition2 == 1)
	    {
		reachednull = 1;
	    }
	}
    }
}


void remove_artefacts_leftright(float *pwavelet, int maxintx, int maxints, double *amp_removed)
{
  int x, x2, s;
  int partition, partition2, endpartition, reachednull;

  //to the left
  partition = 0;
  endpartition = 0;

  for (s = maxints; s >= 0; s--)
    {
      partition2 = 0;
      reachednull = 0;

      for (x = npad_min; x < npad_max; x++)
	if (wavelet(x, s) > 0.0)
	{
	    if (reachednull == 1)
	    {
		endpartition = 1;
		for (x2 = npad_min; x2 < npad_max; x2++)
		    wavelet(x2, s) = 0;
	    }

	    if (endpartition == 1)
	    {
		*amp_removed += wavelet(x, s);
		wavelet(x, s) = 0;
	    }

	    partition2 = 1;
	}
	else
	{
	    if (partition2 == 1)
		reachednull = 1;
	}


      if (partition2 == 1)
	partition = 1;
      else
	if (partition == 1)
	  endpartition = 1;
    }


  //to the right
  partition = 0;
  endpartition = 0;

  for (s = maxints; s < nPtsWavelets; s++)
    {
      partition2 = 0;
      reachednull = 0;

      for (x = npad_min; x < npad_max; x++)
	if (wavelet(x, s) > 0.0)
	{
	    if (reachednull == 1)
	    {
		endpartition = 1;
		for (x2 = npad_min; x2 < npad_max; x2++)
		    wavelet(x2, s) = 0;
	    }

	    if (endpartition == 1)
	    {
		*amp_removed += wavelet(x, s);
	        wavelet(x, s) = 0;
	    }

	    partition2 = 1;
	}
	else
	{
	    if (partition2 == 1)
		reachednull = 1;
	}

      
      if (partition2 == 1)
	partition = 1;
      else
	if (partition == 1)
	  endpartition = 1;
    }

}


void write_wavelet(char *filename, float *pwavelet)
{
  FILE *stream;
  int x, y;
  float max, temp;

  max =0.0;
  for (x=npad_min; x<npad_max; x++)
    for (y=0; y<nPtsWavelets; y++)
      if (wavelet(x, y) > max) max = wavelet(x, y);

  stream = fopen(filename, "wb");
  
  x = nPtsScan;
  y = nPtsWavelets;
  fwrite(&x, sizeof(int), 1, stream);
  fwrite(&y, sizeof(int), 1, stream);

  for (y=0; y<nPtsWavelets; y++)
      for (x=npad_min; x<npad_max; x++)
      {
	  temp = wavelet(x, y)/max;

	  //temp = max-wavelet(x, y);
	  fwrite(&temp, sizeof(float), 1, stream);
      }

  fclose(stream);
}


float cwavelettrans(fftw_plan cwt_p, fftw_plan cwt_p2, int channel, float centerBin, double *data, float *pwavelet, double *inv, int objid)
{
  char text[100];
  float plotx[npad], ploty[npad], ymin, ymax;
  static float tr[6] = {0.0, 1.0, 0.0, 0.0, 0.0, 1.0};
  float re, im, min, max, s_factor;
  float maxint;
  int maxintx, maxints;
  int x, x2, s, s2, i, minx, maxx, mins, maxs;
  double sum;
  double *power;
  long area_, deltax_, deltas_;
  double amp_;
  double amp_removed;
  FILE *stream;
  int cpg_windowps;
  int *pintmap;
  float wavelet2[npad][nPtsWavelets];
  float waveletplot[nPtsScan][nPtsWavelets];

  float background_overall;
  int background_overallcount;
  float dist, mindist;

  pintmap = (int *) malloc(npad * nPtsWavelets * sizeof(int));

  //if (doplot2 == 1) cpgask(1);

  if ((channel == 1) && (printps == 1))
    {
      cpg_windowps = cpgopen("/CPS");
      cpgslct(cpg_windowps);
      cpgask(0);
      cpgsubp(2, 2);
    }
  
  
  if (algCWT == 0)
  {
      if (channel == channel_fastfringe)
	  cwt(cwt_p, cwt_p2, data, pwavelet, 2.0, &min, &max);
      else
	  cwt(cwt_p, cwt_p2, data, pwavelet, 1.0, &min, &max);
  }
  else
  {
//      if (channel == channel_fastfringe)
//	  s_factor = Wavelet_s_factor;
//      else
	  s_factor = Wavelet_s_factor*2.0;
      
      
      for (x = npad_min; x < npad_max; x++)
	  for (s = 0; s < nPtsWavelets; s++)
	  {
	      re = 0.0;
	      im = 0.0;
	      
	      for (x2 = npad_min; x2 <= npad_max; x2++)
	      {
		  re += data[x2] * func_wavelet_re(((float)(x2-x))/((float)(s+1)*s_factor));
		  im += data[x2] * func_wavelet_im(((float)(x2-x))/((float)(s+1)*s_factor));
	      }
	      
	      re /= sqrt(2.0*PI);
	      im /= sqrt(2.0*PI);
	      
	      wavelet(x, s) = sqr(sqrt(re*re+im*im));
	      waveletre[x][s] = re;
	      waveletim[x][s] = im;
	      //wavelet(x, s) = 1.0*sin((float)x/30.0)*cos((float)s/20.0);
	      
	      if (wavelet(x, s) < min) min = wavelet(x, s);
	      if (wavelet(x, s) > max) max = wavelet(x, s);
	  }
  }
  
  if (cwt_narrowband == 0)
    {
      for (x = 0; x < 30; x++)
	for (s = 0; s < nPtsWavelets; s++)
	  {
	    wavelet(x, s) = 0.0;
	    waveletre[x][s] = 0.0;
	    waveletim[x][s] = 0.0;
	  }
      
      for (x = 0; x > -200; x--)
	for (s = nPtsWavelets*2/3-20; s < nPtsWavelets; s++)
	  {
	    x2 = x+2*(int)(s-(nPtsWavelets*2/3-20));
	    if (x2 < 0) x2 = 0;
	    wavelet(x2, s) = 0.0;
	    waveletre[x2][s] = 0.0;
	    waveletim[x2][s] = 0.0;
	  }
    }

  //if (channel == channel_fastfringe)
  //  write_wavelet("wavelet.dat", pwavelet);


  if ((doplot2 == 1))// && (channel == channel_fastfringe))
    {
      //plot wavelet-transformed
      cpgbbuf();
      
      tr[0]=0;
      tr[1]=1.0;
      tr[2]=0;
      tr[3]=0;
      tr[4]=0;
      tr[5]=1.0;

      cpgswin(0.0, (float)nPtsWavelets, 0.0, (float)nPtsScan);
      cpgscr(0, 0, 0, 0);

      cpgslw(1);
      cpgsls(1);
      cpgsci(1);
      cpgpage();
      cpgbox("BCNST", 0.0, 10, "BCNST", 0.0, 10);

      sprintf(text, "Wavelet transformed (Channel %i)", channel);

      cpgmtxt("t", 1.0, 0.0, 0.0, text);

      if (printps == 1)
	{
	  for (x = npad_min; x < npad_max; x++)
	    for (s = 0; s < nPtsWavelets; s++)
		wavelet2[x][s] = max-(wavelet(x, s)-min);

	  for (x = npad_min; x < npad_max; x++)
	      for (s = 0; s < nPtsWavelets; s++)
		  waveletplot[x-npad_min][s] = wavelet2[x][s];

      
	  cpgimag(&waveletplot[0][0], nPtsWavelets, nPtsScan, 1, nPtsWavelets, 1, nPtsScan, min, max, tr);
	}
      else
      {
	  for (x = npad_min; x < npad_max; x++)
	      for (s = 0; s < nPtsWavelets; s++)
		  waveletplot[x-npad_min][s] = wavelet(x, s);

	  cpgimag(&waveletplot[0][0], nPtsWavelets, nPtsScan, 1, nPtsWavelets, 1, nPtsScan, min, max, tr);
      }

      cpgebuf();
    }

  
  //apply filter
  sum = 0.0;
  amp_removed = 0.0;
  maxint = 0.0;
  maxintx = -1;
  maxints = -1;

  for (x = npad_min; x < npad_max; x++)
    for (s = 0; s < nPtsWavelets; s++)
	intmap(x, s) = 1;

  
  for (x = npad_min; x < npad_max; x++)
    for (s = 0; s < nPtsWavelets; s++)
      {
	wavelet2[x][s] = wavelet(x, s);
	sum += wavelet(x, s);
	
	if ((x < 5+npad_min) || (x > nPtsScan-5+npad_min) || (s < 5) || (s > nPtsWavelets-5))
	{
	    amp_removed += wavelet(x, s);
	    wavelet(x, s) = 0.0;
	    waveletre[x][s] = 0.0;
	    waveletim[x][s] = 0.0;

	    intmap(x, s) = 0;
	}
	else
	{
	    if (wavelet(x, s) < cwt_filter_threshold_filter*(max-min)+min)
	    {
		amp_removed += wavelet(x, s);
		wavelet(x, s) = 0.0;
		waveletre[x][s] = 0.0;
		waveletim[x][s] = 0.0;

		intmap(x, s) = 0;
	    }
	    else
	    {
		if (wavelet(x, s) > maxint)
		{
		    maxint = wavelet(x, s);
		    maxintx = x;
		    maxints = s;
		}
	    }
	}
      }
  
  fill(pintmap, 1.0, maxintx, maxints, npad, nPtsWavelets);

  for (x = npad_min; x < npad_max; x++)
    for (s = 0; s < nPtsWavelets; s++)
	if (intmap(x, s) != 2)
	{
	    amp_removed += wavelet(x, s);
	    wavelet(x, s) = 0.0;
	    waveletre[x][s] = 0.0;
	    waveletim[x][s] = 0.0;
	}

  //remove_artefacts_leftright(pwavelet, maxintx, maxints, &amp_removed);
  //remove_artefacts_topbottom(pwavelet, maxintx, maxints, &amp_removed);


  //printf("\nArea: %i   X: %i %i -> %i   S: %i %i -> %i\n", area_, minx, maxx, deltax_, mins, maxs, deltas_);

  //printf("\n %i     ... %i %i  (%f)\n", area_, cwt_threshold_area_min, cwt_threshold_area_max, cwt_filter_threshold_power);


  //*** determine noise background
  //determine the borders of the area first
  minx = npad_max;
  maxx = npad_min;
  mins = nPtsWavelets;
  maxs = 0;
  for (x = npad_min; x < npad_max; x++)
      for (s = 0; s < nPtsWavelets; s++)
	  if (wavelet(x, s) > 0.0)
	  {
	      if (x < minx) minx = x;
	      if (x > maxx) maxx = x;
	      if (s < mins) mins = s;
	      if (s > maxs) maxs = s;
	  }
  
  //printf("\nborders: %i %i (%i..%i) %i %i (%i..%i)\n", minx, maxx, npad_min, npad_max, mins, maxs, 0, nPtsWavelets);
  
  background_overall = 0.0;
  background_overallcount = 0;
  /*
    for (s2 = mins-5; s2 <= maxs+5; s2++)
    for (x2 = npad_min+1; x2 < npad_max-1; x2++)
	      if (((x2 <= minx+0.25*(maxx-minx)) || (x2 >= minx+0.75*(maxx-minx))) || ((s2 <= mins+0.25*(maxs-mins)) || (s2 >= mins+0.75*(maxs-mins))))
	      {
		  mindist = 1.0e10;
		  
		  for (x = minx; x <= maxx; x++)
		      for (s = mins; s <= maxs; s++)
			  if (wavelet(x, s) > 0.0)
			  {
			      dist = (x-x2)*(x-x2) + (s-s2)*(s-s2);
			      
			      if (dist < mindist) mindist = dist;
			  }
		  
		  if (mindist > cwt_background_border*cwt_background_border)
		  {
		      background_overall += wavelet2[x2][s2];
		      background_overallcount++;
		  }
	      }
      */

  for (s2 = mins-25; s2 <= maxs+25; s2++)
      for (x2 = npad_min; x2 < npad_max; x2++)
	  if (((x2 < minx - cwt_background_border) || (x2 > maxx + cwt_background_border)) || ((s2 < mins - cwt_background_border) || (s2 > maxs + cwt_background_border)))
	  {
	      background_overall += wavelet2[x2][s2];
	      background_overallcount++;
	  }
  
  if (background_overallcount > 0)
      background_overall /= background_overallcount;
  else
      printf("\n*** Noise Subtraction failed!\n");
  
  
  area_ = 0;
  amp_ = 0.0;
  minx = 32000;
  maxx = 0.0;
  mins = 32000;
  maxs = 0.0;
  
  for (x = npad_min; x < npad_max; x++)
      for (s = 0; s < nPtsWavelets; s++)
	  if (wavelet(x, s) > 0.0)
	  {
	      if (x < minx) minx = x;
	      if (x > maxx) maxx = x;
	      if (s > maxs) maxs = s;
	      if (s < mins) mins = s;
	      
	      if (wavelet(x, s) >= cwt_filter_threshold_power*(max-min)+min)
	      {
		  area_++;
		  if (cwt_subtract_background == 1)
		      amp_ += (wavelet(x, s) - background_overall);
		  else
		      amp_ += wavelet(x, s);
	      }
	  }
  
  deltax_ = maxx-minx;
  deltas_ = maxs-mins;
  
  if (repdetails == 1) printf("Area: %i   X: %i %i -> %i   S: %i %i -> %i\n", area_, minx, maxx, deltax_, mins, maxs, deltas_);


  //printf("Ratio: %f\n", amp_/amp_removed);
  free(pintmap);




  if ((area_ > cwt_threshold_area_max) || (area_ < cwt_threshold_area_min))
  {
      amp_ = -1.0;
      if (repdetails == 1) printf("\nPISTON is too large or area too small! SCAN REJECTED!\n");
      return -1.0;
  }

  if ((maxintx < npad_min+cwt_threshold_pos_intpeak_spatial) || (maxintx > npad_min+nPtsScan-cwt_threshold_pos_intpeak_spatial))
  {
      amp_ = -2.0;
      if (repdetails == 1) printf("\nMaximal Intensity is OUT OF RANGE!\n");
      return -2.0;
  }
      
  if (((deltas_ > cwt_threshold_size_peak_freq01) && (channel > 0)) || ((deltas_ > cwt_threshold_size_peak_freq2) && (channel == channel_fastfringe)))
  {
      amp_ = -3.0;
      if (repdetails == 1) printf("\nFringe is too wide spread in FREQUENCY!\n");
      return -3.0;
  }

  //printf("MaxIntS: %i, %i\n", maxints, cwt_threshold_pos_intpeak_freq);

  if (maxints < cwt_threshold_pos_intpeak_freq)
  {
      amp_ = -4.0;
      if (repdetails == 1) printf("\nMaximal Intensity is AT WRONG FREQUENCY!\n");
      return -4.0;
  }

  if ((amp_/amp_removed < cwt_threshold_SignalToNoise)) // && (objid != targetid))
  {
      amp_ = -5.0;
      if (repdetails == 1) printf("\nSignal/Noise < 2.0!\n");
      return -5.0;
  }

  if (amp_ < 0.0)
  {
      amp_ = -7.0;
      if (repdetails == 1) printf("\nNegative power!\n");
      return -7.0;
  }

  if (amp_ > 1.0e11)
  {
      amp_ = -8.0;
      if (repdetails == 1) printf("\nToo high power!\n");
      return -8.0;
  }


  if (algCWT == 0)
  {
      //inverse wavelet transform
      for (x2 = npad_min-1; x2 < npad_max; x2++)
      {
	  inv[x2] = 0.0;

	  if ((x2 > npad_min+10) && (x2 < npad_max-10))
	    for (s = 0; s < nPtsWavelets; s++)
		inv[x2-npad_min] += waveletre[x2][s] + waveletim[x2][s];
	}

      //Normalize
      sum = 0.0;
      for (x2 = 0; x2 < nPtsScan; x2++)
	if (inv[x2] > sum) sum = inv[x2];

      if (sum > 0.0)
	for (x2 = 0; x2 < nPtsScan; x2++)
	  inv[x2] /= (sum*1.25);
    }
  else
    {
      //inverse wavelet transform
      for (x2 = 0; x2 <= nPtsScan; x2++)
	{
	  inv[x2] = 0.0;
	  
	  re = 0.0;
	  im = 0.0;
	  
	  for (x = npad_min; x < npad_max; x++)
	    for (s = 0; s < nPtsWavelets; s++)
	      {
		re += waveletre[x][s]/((s+1)*(s+1))*func_wavelet_re(((float)(x2-x))/((float)(s+1)*s_factor));
		//im += waveletim[x][s]/((s+1)*(s+1))*func_wavelet_im(((float)(x2-x))/((float)(s+1)*s_factor));
	      }
	  
	  inv[x2] = re*4.0;
	}
    }


  if ((doplot2 == 1))// && (channel == channel_fastfringe))
    {
      //plot filtered wavelet-transformed
      cpgbbuf();
      
      tr[0]=0;
      tr[1]=1.0;
      tr[2]=0;
      tr[3]=0;
      tr[4]=0;
      tr[5]=1.0;

      cpgswin(0.0, (float)nPtsWavelets, 0.0, (float)nPtsScan);
      cpgscr(0, 0, 0, 0);

      cpgslw(1);
      cpgsls(1);
      cpgsci(1);
      cpgpage();
      cpgbox("BCNST", 0.0, 10, "BCNST", 0.0, 10);
      sprintf(text, "Filtered Wavelet transformed (Area=%i, Ratio=%f)", area_, (float)amp_/sum);
      cpgmtxt("t", 1.0, 0.0, 0.0, text);
      
      for (x = npad_min; x < npad_max; x++)
	  for (s = 0; s < nPtsWavelets; s++)
	      waveletplot[x-npad_min][s] = wavelet(x, s);
      
      cpgimag(&waveletplot[0][0], nPtsWavelets, nPtsScan, 1, nPtsWavelets, 1, nPtsScan, min, max, tr);

      cpgebuf();


      //if (algCWT == 1)
	{
	  //plot signal
	  cpgsci(1);
	  ymin = 1e10;
	  ymax = 1e-10;
	  for (i=0; i<nPtsScan; i++)
	    {
	      plotx[i] = i;
	      ploty[i] = data[i];
	      
	      if (ploty[i] > ymax) ymax = ploty[i];
	      if (ploty[i] < ymin) ymin = ploty[i];
	    }
	  
	  cpgenv(-1.0, nPtsScan, -1.5, 1.5, 0, 1);
	  cpglab("", "Visibility", "Signal");
	  
	  cpgline(nPtsScan-1, plotx, ploty);

	  //plot filtered signal
	  cpgsci(1);
	  ymin = 1e10;
	  ymax = 1e-10;
	  for (i=0; i<nPtsScan; i++)
	    {
	      plotx[i] = i;
	      ploty[i] = inv[i];

	      if (ploty[i] > ymax) ymax = ploty[i];
	      if (ploty[i] < ymin) ymin = ploty[i];
	    }
	  
	  cpgenv(-1.0, nPtsScan, -1.5, 1.5, 0, 1);
	  cpglab("", "Visibility", "Filtered Signal");
	  
	  cpgline(nPtsScan-1, plotx, ploty);
	  
/*	  
	  //plot powerspec
	  power = dvector(0,npad-1);
	  
	  //powerspec signal
	  //calc_powspec(data, power);
	  
	  cpgsci(1);
	  ymin = 1e10;
	  ymax = 1e-10;
	  for (i=0; i<npad; i++)
	    {
	      plotx[i] = i;
	      ploty[i] = power[i];
	      
	      if (ploty[i] > ymax) ymax = ploty[i];
	      if (ploty[i] < ymin) ymin = ploty[i];
	    }
	  
	  cpgenv(-1.0, nPtsScan/2, 0.0, ymax*1.1, 0, 1);
	  cpglab("", "Power", "Power Spectrum Signal");
	  
	  cpgline(npad-1, plotx, ploty);
	  

	  //powerspec filtered signal
	  //calc_powspec(inv, power);
	  
	  if (repfiles == 1)
	    {
	      printf("centerBin: %f\n", centerBin);
	  
	      stream = fopen("sigfringesfilter.dat","w");
	      for (i=0; i<nPtsScan; i++)
		fprintf(stream, "%f %f\n", (float)(i-centerBin)*OPD_slowfringe/(float)npad, inv[i]);
		fprintf(stream, "%f %f %f %f\n", i*(FFT_maxfreq-FFT_minfreq)+FFT_minfreq, (float)power[i]);
	      fclose(stream);

	      stream = fopen("powspecfilter.dat","w");
	      for (i=0; i<npad/2; i++)
		fprintf(stream, "%f %f\n", (float)i*(FFT_maxfreq-FFT_minfreq)+FFT_minfreq, (float)power[i]);
	      fclose(stream);

	      cpgask(1);
	      printf("Break now!\n");
	    }

	  
	  cpgsci(1);
	  //ymin = 1e10;
	  //ymax = 1e-10;
	  for (i=0; i<npad; i++)
	    {
	      plotx[i] = i;
	      ploty[i] = power[i];
	      
	      //if (ploty[i] > ymax) ymax = ploty[i];
	      //if (ploty[i] < ymin) ymin = ploty[i];
	    }
	  
	  cpgenv(-1.0, nPtsScan/2, 0.0, ymax*1.1, 0, 1);
	  cpglab("", "Power", "Power Spectrum Filtered Signal");
	  
	  cpgline(npad-1, plotx, ploty);
	  
	  free_dvector(power,0,npad-1);  
*/	  
	}
    }

  if ((channel == 1) && (printps == 1)) cpgclos();
  
  waveletamp = amp_/1.0e6;

  //if (channel == channel_fastfringe)
  //    write_wavelet("waveletfiltered.dat", pwavelet);

  //restore original wavelet for averaging
  for (x = npad_min; x < npad_max; x++)
    for (s = 0; s < nPtsWavelets; s++)
      wavelet(x, s) = wavelet2[x][s];

  return waveletamp;
}

#undef wavelet
