/* idrsplot.c                                */
/* IOTA Data Reduction Software              */
/* (c) Stefan Kraus                          */
/* 2004                                      */

// order visibilities, uv-coordinates: baselines AB, AC, BC

#include "../idrs/general.c"
#include "stdio.h"
#include <cpgplot.h>
#include <stdlib.h>
#include "../idrs/nrutil.c"
#include "../idrs/sort.c"
#include <sys/stat.h>
#include <termios.h> 

#define PI         (double)3.14159265358979323846264
#define RToD       (180.0/PI)

char ProcDir[300], ProcFilename[300];
int targetid;


char kbhit(void) { 
    struct termios term, oterm; 
    int fd = 0; 
    int c = 0; 
    tcgetattr(fd, &oterm); 
    memcpy(&term, &oterm, sizeof(term)); 
    term.c_lflag = term.c_lflag & (!ICANON); 
    term.c_cc[VMIN] = 0; 
    term.c_cc[VTIME] = 1; 
    tcsetattr(fd, TCSANOW, &term); 
    c = getchar(); 
    tcsetattr(fd, TCSANOW, &oterm);
    //if (c != -1)  ungetc(c, stdin); 
    //return ((c != -1) ? 1 : 0); 
    return c;
} 

int file_exist(char *file_name) 
{
      struct stat file_stat;
      return ( (stat(file_name, &file_stat) == 0) ? 1:0);
}


//calculates the median for an double-array data[0..number]

double median(int number, float *data)
{
  float *sortdata;
  double med;
  int i, numberhalf;

  sortdata = vector(1, number+1);
 
  for (i=0; i<=number; i++)
    sortdata[i+1] = data[i];


  sort((long)number+1, sortdata);

  for (i=0; i<=number; i++)
    data[i] = sortdata[i+1];
  

  numberhalf = (int)((float)number/2.0);

  if (numberhalf % 2 != 0)
    med = ((sortdata[numberhalf] + sortdata[numberhalf+1])/2.0);
  else
    med = (sortdata[numberhalf+1]);
  
  free_vector(sortdata, 1, number+1);
  
  return med;
}

int main(int argc, char *argv[])
{
    int i, j, n, dataint[3], points, quit, id, plotjd, defplotfromto, nfrom, nto;
    int silent, waitsec;
    float plotfrom, plotto;
    float data[24], vismax[3];
    char filename[300], filename2[300], filename2nr[10], key, PGPLOT_window[100], modelfile[300], modelfile2[300], inAMP[300], inBIS[300], inTRANS[300], scriptfile[300];
     FILE *stream, *streamscript;
     float viscp[max_objects+1][4][max_vispoints], viscperrlower[max_objects+1][4][max_vispoints], viscperrupper[max_objects+1][4][max_vispoints], JD[max_objects+1][max_vispoints];
     float mediandata[max_vispoints];
     int objid[max_objects+1][max_vispoints];
     int nr[max_objects+1], found, plotamp, plotvis;
     float number[max_objects+1][max_vispoints];

     int transfpoints;
     float transfnumber[max_vispoints], transfJD[max_vispoints];
     float transfviscp[4][max_vispoints];
     int modelpoints;
     float modelnumber[max_vispoints], modelJD[max_vispoints];
     float modelviscp[4][max_vispoints];
     int modelpoints2;
     float modelnumber2[max_vispoints], modelJD2[max_vispoints];
     float modelviscp2[4][max_vispoints];
     
     char c;

     int cpg_window1, cpg_window2, channel, channelfrom, channelto;
     

     found = 0;
     for (i=1; i<argc; i++) if (strcasecmp(argv[i], "-h") == 0) found = i;
     for (i=1; i<argc; i++) if (strcasecmp(argv[i], "-help") == 0) found = i;
     if (found > 0)
     {
	 printf("IOTA Data Reduction Software - First glance plotting tool\n(c) 2004-05 by Stefan Kraus\n    at  University of Massachusetts at Amherst, Amherst MA 01002, USA\n    and Max-Planck-Institute for Radioastronomy (MPIfR), Auf dem Huegel 69, D-53121 Bonn, Germany\n\nUsage: idrsplot -dev [pgplot_device]\n                -iraw [filename]\n                -icalib [filename]\n                -itrans [filename]\n                -model [filename]\n                -model2 [filename]\n                -silent\n                -onlycalib\n                -onlyraw\n                -wait [sec]\npgplot_device: default: /xwindow\n                     or file.ps/CPS (for postscript file)\n");
	 
	 return;
     }



     c = ' ';

     found = 0;
     for (i=1; i<argc; i++) if (strcasecmp(argv[i], "-dev") == 0) found = i;
     if (found >= 1)
	 strcpy(PGPLOT_window, argv[found+1]);
     else 
	 strcpy(PGPLOT_window, "/xwindow");
     strcpy(modelfile, "");
     strcpy(modelfile2, "");

     strcpy(ProcDir, "");
     strcpy(scriptfile, "idrs.ini");

     plotamp = 1;
     plotvis = 1;
     silent = 0;
     waitsec = 0;
     for (i=1; i<argc; i++) 
     {
	 if (strcasecmp(argv[i], "-onlycalib") == 0) plotvis = 0;
	 if (strcasecmp(argv[i], "-onlyraw") == 0) plotamp = 0;
	 if (strcasecmp(argv[i], "-silent") == 0) silent = 1;
	 if (strcasecmp(argv[i], "-wait") == 0) waitsec = atoi(argv[i+1]);
	 if (strcasecmp(argv[i], "-model") == 0) strcpy(modelfile, argv[i+1]);
	 if (strcasecmp(argv[i], "-model2") == 0) strcpy(modelfile2, argv[i+1]);
	 if (strcasecmp(argv[i], "-i") == 0) { strcpy(scriptfile, argv[i+1]); }
     }

     printf("Read ProcDir from ini-file %s.\n", scriptfile);
     streamscript = fopen(scriptfile,"r");
     fscanf(streamscript, "ProcDir: %s\n", ProcDir);
     fclose(streamscript);

     strcpy(inAMP, ProcDir); strcat(inAMP, "ampcorr.dat");
     strcpy(inBIS, ProcDir); strcat(inBIS, "bispec.dat");
     strcpy(inTRANS, ProcDir); strcat(inTRANS, "transf.dat");
     for (i=1; i<argc; i++) 
     {
	 if (strcasecmp(argv[i], "-iraw") == 0) strcpy(inAMP, argv[i+1]);
	 if (strcasecmp(argv[i], "-icalib") == 0) strcpy(inBIS, argv[i+1]);
	 if (strcasecmp(argv[i], "-itrans") == 0) strcpy(inTRANS, argv[i+1]);
     }

     printf("Read        Raw Visibilities from %s\n", inAMP);
     printf("Read Calibrated Visibilities from %s\n", inBIS);
     printf("Read       Transfer Function from %s\n", inTRANS);

     if ((plotamp == 0) && (plotvis == 0))
     {
	 printf("Please do not use ''-onlycalib'' and ''-onlyraw'' at the same time!\n");
	 return 0;
     }


     if (plotamp == 1)
     {
	 cpg_window1 = cpgopen(PGPLOT_window);
	 cpgask(0);
	 nfrom = 0;
     }
     else
	 nfrom = 1;
     
     if (plotvis == 1)
     {
	 cpg_window2 = cpgopen(PGPLOT_window);
	 cpgask(0);
	 nto = 1;
     }
     else
	 nto = 0;



     strcpy(ProcFilename, ProcDir); strcat(ProcFilename, "target.ini");
     printf("Read            MainTargetID from %s\n\n", ProcFilename);
     stream = fopen(ProcFilename,"r");
     fscanf(stream, "%i", &targetid);
     fclose(stream);
     printf("Show transfer function for MainTargetID #%i\n", targetid);



     channelfrom = 0;
     channelto = 3;
     plotjd = 0;
     defplotfromto = 0;

     printf("\nCommands for user interaction:\n");
     printf("   'A'=Plot V^2+CP   '1'=V^2_AB        '2'=V^2_AC        '3'=V^2_BC        'C'=CP\n");
     printf("   'Q'=quit          'L'=Set Abscissa Limits             'J'=Switch Abscissa: JD/Scan#\n");
     
     quit = 0;
     while (quit == 0)
     {
	 for (n=nfrom; n<=nto; n++)
	 {
	     if ((plotamp == 1) && (plotvis == 1))
	     {
		 if (n == 1)
		     cpgslct(cpg_window1);
		 else
		     cpgslct(cpg_window2);
	     }

	     cpgsubp(1, 1+(channelto-channelfrom));

	     for (channel=channelfrom; channel<=channelto; channel++)
		 for (i=1; i<=max_vispoints; i++)
		 {
		     transfviscp[channel][i] = -1000.0;

		     for (id=0; id<=max_objects; id++)
			 viscp[id][channel][i] = -1000.0;
		 }
	     
	     
	     for (i=0; i<3; i++)
		 vismax[i] = -1.0;
	     
	     for (id=0; id<=max_objects; id++)
	     {
		 if (n == 1)
		     strcpy(filename, inAMP);
		 else
		     strcpy(filename, inBIS);

		 points = 0;


		 if (file_exist(filename) == 1)
		 {
		     stream = fopen(filename, "r");

		     j = 0;
		 
		     while (feof(stream) == 0)
		     {
			 fscanf(stream, streamampstring1, &dataint[0], &data[0], &dataint[1], &data[1], &data[2], &data[3], &data[4], &data[5], &data[6], &data[7], &data[8], &data[9], &data[10], &data[11], &data[12], &data[13], &data[14], &data[15], &data[16], &data[17], &dataint[2]);
			 fscanf(stream, streamampstring2, &data[18], &data[19], &data[20], &data[21], &data[22], &data[23]);

			 if ((dataint[1] == id) || (id == 0))
			 {
			     nr[id] = (int)points;
			     number[id][points] = (float)dataint[0];
			     JD[id][points] = data[0];
			     viscp[id][0][points] = data[1];
			     viscperrupper[id][0][points] = data[1]+data[2];
			     viscperrlower[id][0][points] = data[1]-data[2];
			     viscp[id][1][points] = data[4];
			     viscperrupper[id][1][points] = data[4]+data[5];
			     viscperrlower[id][1][points] = data[4]-data[5];
			     viscp[id][2][points] = data[7];
			     viscperrupper[id][2][points] = data[7]+data[8];
			     viscperrlower[id][2][points] = data[7]-data[8];
			     viscp[id][3][points] = data[16]*RToD;
			     viscperrupper[id][3][points] = (data[16]+data[17])*RToD;
			     viscperrlower[id][3][points] = (data[16]-data[17])*RToD;

			     objid[id][points] = dataint[1];
		     
			     points++;
			 }
		     }

		     fclose(stream);


		     if (id == 0)
		     {
			 for (i=0; i<3; i++)
			     if (n == 2)
				 vismax[i] = 1.5;
			     else
			     {
				 for (j=0; j<points; j++)
				     mediandata[j] = viscp[0][i][j];
				 median(points-1, mediandata);
				 
				 if (points > 10)
				     vismax[i] = mediandata[(int)(points*0.99)]*1.2;
				 else
				     vismax[i] = mediandata[points-1];
			     }
		     }
		 }
	     }

	     //read transfer function
	     if (n == 1)
	     {
		 stream = fopen(inTRANS, "r");
		 
		 transfpoints = 0;
		 
		 while (feof(stream) == 0)
		 {
		     fscanf(stream, streamampstring1, &dataint[0], &data[0], &dataint[1], &data[1], &data[2], &data[3], &data[4], &data[5], &data[6], &data[7], &data[8], &data[9], &data[10], &data[11], &data[12], &data[13], &data[14], &data[15], &data[16], &data[17], &dataint[2]);
		     fscanf(stream, streamampstring2, &data[18], &data[19], &data[20], &data[21], &data[22], &data[23]);

		     if (dataint[1] == targetid)
		     {
			 transfnumber[transfpoints] = (float)dataint[0];
			 transfJD[transfpoints] = data[0];
			 transfviscp[0][transfpoints] = data[1];
			 transfviscp[1][transfpoints] = data[4];
			 transfviscp[2][transfpoints] = data[7];
			 transfviscp[3][transfpoints] = data[16]*RToD;
		     
			 transfpoints++;
		     }
		 }

		 fclose(stream);
	     }


	     //read model
	     if ((n == 0) && strlen(modelfile) > 0)
	     {
		 stream = fopen(modelfile, "r");
		 
		 modelpoints = 0;
		 
		 while (feof(stream) == 0)
		 {
		     fscanf(stream, streamampstring1, &dataint[0], &data[0], &dataint[1], &data[1], &data[2], &data[3], &data[4], &data[5], &data[6], &data[7], &data[8], &data[9], &data[10], &data[11], &data[12], &data[13], &data[14], &data[15], &data[16], &data[17], &dataint[2]);
		     fscanf(stream, streamampstring2, &data[18], &data[19], &data[20], &data[21], &data[22], &data[23]);
		     
		     if (dataint[1] == targetid)
		     {
			 modelnumber[modelpoints] = (float)dataint[0];
			 modelJD[modelpoints] = data[0];
			 modelviscp[0][modelpoints] = data[1];
			 modelviscp[1][modelpoints] = data[4];
			 modelviscp[2][modelpoints] = data[7];
			 modelviscp[3][modelpoints] = data[16]*RToD;
			 
			 modelpoints++;
		     }
		 }
		 
		 //printf("Read model: %i\n", modelpoints);
		 
		 fclose(stream);
	     }


	     //read model
	     if ((n == 0) && strlen(modelfile2) > 0)
	     {
		 stream = fopen(modelfile2, "r");
		 
		 modelpoints2 = 0;
		 
		 while (feof(stream) == 0)
		 {
		     fscanf(stream, streamampstring1, &dataint[0], &data[0], &dataint[1], &data[1], &data[2], &data[3], &data[4], &data[5], &data[6], &data[7], &data[8], &data[9], &data[10], &data[11], &data[12], &data[13], &data[14], &data[15], &data[16], &data[17], &dataint[2]);
		     fscanf(stream, streamampstring2, &data[18], &data[19], &data[20], &data[21], &data[22], &data[23]);
		     
		     if (dataint[1] == targetid)
		     {
			 modelnumber2[modelpoints2] = (float)dataint[0];
			 modelJD2[modelpoints2] = data[0];
			 modelviscp2[0][modelpoints2] = data[1];
			 modelviscp2[1][modelpoints2] = data[4];
			 modelviscp2[2][modelpoints2] = data[7];
			 modelviscp2[3][modelpoints2] = data[16]*RToD;
			 
			 modelpoints2++;
		     }
		 }
		 
		 //printf("Read model: %i\n", modelpoints);
		 
		 fclose(stream);
	     }

	     cpgsci(1);
	     for (channel=channelfrom; channel<=channelto; channel++)
	     {
		 if (defplotfromto == 0)
		 {
		     if (plotjd == 0)
		     {
			 plotfrom = (float)number[0][0];
			 plotto = (float)number[0][nr[0]];
		     }
		     else
		     {
			 plotfrom = 1.0e10;
			 plotto = -1.0e10;
			 for (i=1; i<=max_vispoints; i++)
			     if (JD[0][i] != 0.0)
			     {
				 if (plotfrom > JD[0][i]) plotfrom = JD[0][i];
				 if (plotto < JD[0][i]) plotto = JD[0][i];
			     }
		     }
		 }

		 if ((plotto - plotfrom) > 0.0)
		 {
		     if (channel == 3)
		     {
			 cpgenv(plotfrom, plotto, -180, 180, 0, 1);
			 cpglab("", "Closure Phase AB->BC->CA", "");
		     }
		     else
		     {
			 cpgenv(plotfrom, plotto, -0.15, vismax[channel], 0, 1);
		     
			 if (n == 1)
			 {
			     if (channel == 0) cpglab("", "(Raw Visibility)^2 AB", "");
			     if (channel == 1) cpglab("", "(Raw Visibility)^2 AC", "");
			     if (channel == 2) cpglab("", "(Raw Visibility)^2 BC", "");
			 }
			 else
			 {
			     if (channel == 0) cpglab("", "(Calib. Visibility)^2 AB", "");
			     if (channel == 1) cpglab("", "(Calib. Visibility)^2 AC", "");
			     if (channel == 2) cpglab("", "(Calib. Visibility)^2 BC", "");
			 }
		     }
		 
		     cpgsci(1);
		 
		     if (n == 1)
		     {
			 for (id=0; id<=max_objects; id++)
			 {
			     if (id >= 16)
				 cpgsci((int)(id-15));
			     else
				 cpgsci((int)(id));

			     if (nr[id] > 0)
			     {
				 if (plotjd == 0)
				 {
				     cpgpt(nr[id], number[id], viscp[id][channel], 0);
				     cpgerry(nr[id], number[id], viscperrlower[id][channel], viscperrupper[id][channel], 0);
				 }
				 else
				 {
				     cpgpt(nr[id], JD[id], viscp[id][channel], 0);
				     cpgerry(nr[id], JD[id], viscperrlower[id][channel], viscperrupper[id][channel], 0);
				 }
			     }
			 }

			 
			 id = targetid;

			 cpgsci(2);
			 if (plotjd == 0)
			     cpgpt(transfpoints, transfnumber, transfviscp[channel], 0);
			 else
			     cpgpt(transfpoints, transfJD, transfviscp[channel], 0);
		     }
		     else
		     {
			 for (id=0; id<=max_objects; id++)
			 {
			     if (id >= 16)
				 cpgsci((int)(id-15));
			     else
				 cpgsci((int)(id));

			     if (nr[id] > 0)
			     {
				 if (plotjd == 0)
				 {
				     cpgpt(nr[id], number[id], viscp[id][channel], 0);
				     cpgerry(nr[id], number[id], viscperrlower[id][channel], viscperrupper[id][channel], 0);
				 }
				 else
				 {
				     cpgpt(nr[id], JD[id], viscp[id][channel], 0);
				     cpgerry(nr[id], JD[id], viscperrlower[id][channel], viscperrupper[id][channel], 0);
				 }
			     }
			 }

			 cpgsci(2);
			 if (plotjd == 0)
			     cpgpt(modelpoints, modelnumber, modelviscp[channel], 0);
			 else
			     cpgpt(modelpoints, modelJD, modelviscp[channel], 0);

			 cpgsci(3);
			 if (plotjd == 0)
			     cpgpt(modelpoints2, modelnumber2, modelviscp2[channel], 0);
			 else
			     cpgpt(modelpoints2, modelJD2, modelviscp2[channel], 0);
		     }
		     cpgsci(1);
		 }
	     }
	 }
	 sleep(waitsec);

	 if (((plotto - plotfrom) > 0.0) && (silent == 0))
	 {
	     printf(".");
	     fflush(stdout);

	     for (i=0; i<3; i++)
	     {
		key=kbhit();

		if (key != -1)
		{
		     printf("\n");
		     fflush(stdout);

		     switch (key)
		     {
			 case 'q':
			     quit = 1;
			     break;
			 case 'Q':
			     printf("Quit\n");
			     quit = 1;
			     break;
			 case '1':
			     printf("Plot Visibility, Channel 1\n");
			     channelfrom = 0;
			     channelto = 0;
			     break;
			 case '2':
			     printf("Plot Visibility, Channel 2\n");
			     channelfrom = 1;
			     channelto = 1;
			     break;
			 case '3':
			     printf("Plot Visibility, Channel 3\n");
			     channelfrom = 2;
			     channelto = 2;
			     break;
			 case 'c':
			     printf("Plot Closure Phase\n");
			     channelfrom = 3;
			     channelto = 3;
			     break;
			 case 'a':
			     printf("Plot all channels\n");
			     channelfrom = 0;
			     channelto = 3;
			     break;
			 case 'A':
			     printf("Plot all channels\n");
			     channelfrom = 0;
			     channelto = 3;
			     break;
			 case 'C':
			     channelfrom = 3;
			     channelto = 3;
			     break;
			 case 'l':
			     printf("Current Abscissa limits: %f to %f\n", plotfrom, plotto);
			     defplotfromto = 1;
			     printf("New Abscissa limits from: ");
			     scanf("%f", &plotfrom);
			     printf("                      to: ");
			     scanf("%f", &plotto);
			     break;
			 case 'L':
			     printf("Current Abscissa limits: %f to %f\n", plotfrom, plotto);
			     defplotfromto = 1;
			     printf("New Abscissa limits from: ");
			     scanf("%f", &plotfrom);
			     printf("                      to: ");
			     scanf("%f", &plotto);
			     break;
			 case 'j':
			     defplotfromto = 0;
			     if (plotjd == 0)
			     {
				 printf("Plot JD on abscissa!\n");
				 plotjd = 1;
			     }
			     else
			     {
				 printf("Plot Nr. on abscissa!\n");
				 plotjd = 0;
			     }
			     break;
			 case 'J':
			     defplotfromto = 0;
			     if (plotjd == 0)
			     {
				 printf("Plot JD on abscissa!\n");
				 plotjd = 1;
			     }
			     else
			     {
				 printf("Plot Nr. on abscissa!\n");
				 plotjd = 0;
			     }
			     break;
		     }
		 }
	     }
	 }
     }

     cpgend();
}
