#!/bin/bash

# idrs.c                                 #
# Interferometry Data Reduction Software #
# by Stefan Kraus                        #
# 2002-05                                #
#                                        #
# copyright laws apply                   #


if test $# -lt 2;then
  echo "Usage: install.sh [OS] [compiler]"
  echo "Offered combinations for [OS] [compiler]:"
  #echo "      linux   f77_gcc"
  echo "      linux   g77_gcc"
  #echo "      sol2    f77_cc         (Solaris 2.x, SunOs 5.x)"
  echo "      sol2    f77_gcc     (Solaris 2.x, SunOs 5.x)"
  #echo "      sol2    g77_gcc"
  #echo "      sun4    f77_acc        (SunOS 4.x)"
  #echo "      sun4    f77_cc"
  echo "      sun4    f77_gcc     (SunOS 4.x)"
  exit 1
else
  OS=$1
  COMP=$2
fi


# Verify that the ${IDRSROOT} is set
if
    test ${IDRSROOT} == "" ;
    then
    echo "!!! The variable IDRSROOT was not set or the directory does not exists !!!"
    exit ;
else
    mkdir -p ${IDRSROOT} ;
fi


# create all directories

mkdir -p ${IDRSROOT}
mkdir -p ${IDRSROOT}/bin
mkdir -p ${IDRSROOT}/lib
mkdir -p ${IDRSROOT}/include
mkdir -p ${IDRSROOT}/src
mkdir -p ${IDRSROOT}/src/pgplot
mkdir -p ${IDRSROOT}/manual

LOGFILE=`echo "${IDRSROOT}/install.log"`
rm -f $LOGFILE

echo "=== Installation directory: ${IDRSROOT} ===" | tee -a $LOGFILE

# Set the CFITSIO_DIR and verify it has well been set
echo "INFO: Prepare the installation of CFITSIO" | tee -a $LOGFILE
export CFITSIO_DIR=${IDRSROOT}

# Set the FFTW_DIR and verify it has well been set
echo "INFO: Prepare the installation of FFTW" | tee -a $LOGFILE
export FFTW_DIR=${IDRSROOT}

echo "INFO: Decompress all packages" | tee -a $LOGFILE

cp -f ./idrs.cfg ${IDRSROOT}/bin/
cp -f ./createobslog.sh ${IDRSROOT}/bin/
cp -f ./idrsobs.sh ${IDRSROOT}/bin/
cp -f ./manual.ps ${IDRSROOT}/manual/

chmod u+x ${IDRSROOT}/bin/createobslog.sh
chmod u+x ${IDRSROOT}/bin/idrsobs.sh

echo "ProcDir: ${IDRSROOT}/testdata.proc/" > ${IDRSROOT}/bin/idrs.ini
echo "Dir: ${IDRSROOT}/testdata/2003-03-24/  Files: All  Scans: All" >> ${IDRSROOT}/bin/idrs.ini


if
    test -r testdata.tar.gz
    then
    #mv instead of cp
    mkdir -p ${IDRSROOT}/testdata.proc
    cp testdata.tar.gz ${IDRSROOT}/
fi

# move to ${IDRSROOT}/src all packages
if
    test -r idrs.inc-?.*.tar.gz
    then
    #mv instead of cp
    cp idrs.inc-*.tar.gz ${IDRSROOT}/include
fi

if
    test -r idrs.src-?.*.tar.gz
    then
    #mv instead of cp
    cp idrs.src-*.tar.gz ${IDRSROOT}/src
fi

if
    test -r pgplot*.tar.gz
    then
    #mv instead of cp
    cp pgplot*.tar.gz ${IDRSROOT}/src
    cp ./drivers.list ${IDRSROOT}/src/drivers.list
fi

if
    test -r fftw-?.?.?.tar.gz
    then
    #mv instead of cp
    cp fftw*.tar.gz ${IDRSROOT}/src
fi

if
    test -r cfitsio*.tar.gz
    then
    #mv instead of cp
    cp cfitsio*.tar.gz ${IDRSROOT}/src
fi

cd ${IDRSROOT}/include

# Unzip and untar idrs.inc-*.tar.gz
if
    test -r idrs.inc-*.tar.gz -o -r ${IDRSROOT}/include/idrs.inc-*.tar.gz ;
    then
    echo "--- Package idrs.inc-*.tar.gz found, uncompressing..." | tee -a $LOGFILE
    gunzip idrs.inc-*.tar.gz
    tar xf idrs.inc-*.tar
    rm -f idrs.inc-*.tar
    echo "Done." | tee -a $LOGFILE ;
else 
    echo "!!! Package idrs.inc-*.tar.gz missing !!! " | tee -a $LOGFILE
fi

cd ${IDRSROOT}/src

# Unzip and untar idrs.src-*.tar.gz
if
    test -r idrs.src-*.tar.gz -o -r ${IDRSROOT}/src/idrs.src-*.tar.gz ;
    then
    echo "--- Package idrs.src-*.tar.gz found, uncompressing..." | tee -a $LOGFILE
    gunzip idrs.src-*.tar.gz
    tar xf idrs.src-*.tar
    rm -f idrs.src-*.tar
    echo "Done." | tee -a $LOGFILE ;
else 
    echo "!!! Package idrs.src-*.tar.gz missing !!! " | tee -a $LOGFILE
fi

# Unzip and untar pgplot*.tar.gz
if
    test -r pgplot*.tar.gz -o -r ${IDRSROOT}/src/pgplot*.tar.gz ;
    then
    echo "--- Package pgplot*.tar.gz found, uncompressing..." | tee -a $LOGFILE
    gunzip pgplot*.tar.gz
    tar xf pgplot*.tar
    rm -f pgplot*.tar
    cp -f ./drivers.list ./pgplot/drivers.list
    echo "Done." | tee -a $LOGFILE ;
else 
    echo "!!! Package gplot*.tar.gz missing !!! " | tee -a $LOGFILE
fi

# Unzip and untar cfitsio*.tar.gz
if
    test -r cfitsio*.tar.gz -o -r ${IDRSROOT}/src/cfitsio*.t*gz ;
    then
    echo "--- Package cfitsio*.tar.gz found, uncompressing..." | tee -a $LOGFILE
    gunzip cfitsio*.tar.gz
    tar xf cfitsio*.tar
    rm -f cfitsio*.tar
    echo "Done." | tee -a $LOGFILE ;
else 
    echo "!!! Package cfitsio*.tar.gz missing !!! " | tee -a $LOGFILE
fi

# Unzip and untar fftw-*.tar.gz
if
    test -r fftw-*.tar.gz -o -r ${IDRSROOT}/src/fftw-*.tar.gz ;
    then
    echo "--- Package fftw-*.tar.gz found, uncompressing..." | tee -a $LOGFILE
    gunzip fftw-*.tar.gz
    tar xf fftw-*.tar
    rm -f fftw-*.tar
    echo "Done." | tee -a $LOGFILE ;
else 
    echo "!!! Package fftw-?.?.?.tar.gz missing !!! " | tee -a $LOGFILE
fi

cd ${IDRSROOT}/

# Unzip and untar testdata.tar.gz
if
    test -r testdata.tar.gz -o -r ${IDRSROOT}/testdata.tar.gz ;
    then
    echo "--- Package testdata.tar.gz found, uncompressing..." | tee -a $LOGFILE
    gunzip testdata.tar.gz
    tar xf testdata.tar
    rm -f testdata.tar
    echo "Done." | tee -a $LOGFILE ;
else 
    echo "!!! Package testdata.tar.gz missing !!! " | tee -a $LOGFILE
fi


# Do Byte Swap?
SWAPBYTES="1"
test "$OS" == "sol2" && { SWAPBYTES="0"; }
test "$OS" == "sun4" && { SWAPBYTES="0"; }


# Install PGPLOT
echo | tee -a $LOGFILE
echo "=== INSTALLING PGPLOT ===" | tee -a $LOGFILE
echo | tee -a $LOGFILE
cd ${IDRSROOT}/src/pgplot
./makemake ${IDRSROOT}/src/pgplot ${OS} ${COMP} | tee -a $LOGFILE
make all install | tee -a $LOGFILE
make cpg | tee -a $LOGFILE
cp ${IDRSROOT}/src/pgplot/libpgplot.a ${IDRSROOT}/lib/
cp ${IDRSROOT}/src/pgplot/libcpgplot.a ${IDRSROOT}/lib/
cp ${IDRSROOT}/src/pgplot/libpgplot.so ${IDRSROOT}/lib/
cd ${IDRSROOT}/lib
ranlib *pgplot*.a



# Install CFITSIO
echo | tee -a $LOGFILE
echo "=== INSTALLING CFITSIO ===" | tee -a $LOGFILE
echo | tee -a $LOGFILE
cd ${IDRSROOT}/src/cfitsio
./configure --prefix=${IDRSROOT} | tee -a $LOGFILE
make all install | tee -a $LOGFILE
cd ${IDRSROOT}/lib
ranlib *cfits*.a


# Install FFTW
echo | tee -a $LOGFILE
echo "=== INSTALLING FFTW ===" | tee -a $LOGFILE
echo | tee -a $LOGFILE
cd ${IDRSROOT}/src/fftw-?.?.?
./configure --enable-type-prefix --prefix=${IDRSROOT} | tee -a $LOGFILE
make all install | tee -a $LOGFILE ;
cd ${IDRSROOT}/lib
ranlib *fftw*.a


# Make idrs
echo | tee -a $LOGFILE
echo "=== INSTALLING IDRS / IDRSPLOT ===" | tee -a $LOGFILE
echo | tee -a $LOGFILE
cd ${IDRSROOT}/src/
cat ${IDRSROOT}/src/configure.distrib1 >${IDRSROOT}/src/configure
echo "IDRSROOT=\"${IDRSROOT}\"" >> ${IDRSROOT}/src/configure
echo "SWAPBYTES=\"$SWAPBYTES\"" >> ${IDRSROOT}/src/configure
cat ${IDRSROOT}/src/configure.distrib2 >> ${IDRSROOT}/src/configure
rm -f ${IDRSROOT}/src/configure.distrib1
rm -f ${IDRSROOT}/src/configure.distrib2
chmod u+x ${IDRSROOT}/src/configure
./configure ${OS} ${COMP} | tee -a $LOGFILE
make all -s | tee -a $LOGFILE



if
    test -r ${IDRSROOT}/bin/idrs ;
    then
    echo | tee -a $LOGFILE
    echo "=== INSTALLATION COMPLETED SUCCESSFULLY ===" | tee -a $LOGFILE
    echo | tee -a $LOGFILE
    echo "Please start idrs and idrsplot from the directory ${IDRSROOT}/bin" | tee -a $LOGFILE
    echo "(You might add this directory to your PATH variable now.)" | tee -a $LOGFILE
else
    echo | tee -a $LOGFILE
    echo "=== SOME PROBLEMS OCCURED ===" | tee -a $LOGFILE
    echo | tee -a $LOGFILE
    echo "Please check the screen output for error messages." | tee -a $LOGFILE
    echo "You are welcome to ask the author for assistance" | tee -a $LOGFILE
    echo "(email: skraus@mpifr-bonn.mpg.de)" | tee -a $LOGFILE
fi
