/****************************************************
* This is the packet tracker callable from DAQ_Scheduler.cpp
*
* example usage in DAQ_Scheduler.cpp
*	result = ptracker_ed(in + col,0, n - 1, nDFT + j, 2, 11, nFringe, 1, 0, 
*				edThreshold, edWeight, edPhase);
*   notes:	in this version, the value (not the address) of nFringe is passed
*			nFringe should be set to 20 rather than 35 as presently done
*			iotadaq_data->ft_threshold is what sets thresholdRatio, so that is what
*				should be set to 2.0
*
* The returned value is 1 if a packet was found successfully, otherwise 0
* y_raw_double is a pointer to an array of doubles - the raw (normalized) data
* nl and nh are the index limits of that array, e.g. 0 and 127 or 1 and 256, etc.
* nDFT is a pointer to an integer containing the number of points used in the
* 		DFT window. It must be an odd number, and should equal the number of
* 		data points in two wavelengths of the fringe.
* 		It is updated by the function, so it should be kept alive for use
* 		on the next scan. Recommended value: 13
* nPeriodsInDFTWindow is the number of fringe periods used to calculate the
*		DFT. Corresponds to the bin number checked in the DFT result 
*		(bin # = nPeriodsInDFTWindow + 1). Recommended value: 2
* nDFTSmooth: the first step after DFT calculation is to smooth it by taking
*		the mean across a window. This is the width of that window 
*		(total number of points). Recommended value: 11
* nFringe determines the width of the template used in convolution with the 
*		DFT result. If a rectangular window is used (if templateType = 1,
*		which is the only option supported as of 5/8/2002), this is the width 
*		from the center point to the outer edge of the rectangle. So if 
*		nFringe = 20, the window size is 41 (2*20 + 1). Recommended setting: to
*		cover the half-width at half maximum.
*		If the window is rectangular with negative side lobes
*		(templateType = 2, not supported as of 5/8/2002), this is the width 
*		from the center point to the outer edge of each negative side lobe.
*		Recommended setting: so the negative side lobes are at the
*		edge of the main lobe of the interferogram.
*		Capability to adaptively set this within ptracker_ed() may be added, 
*       fairly easily if there is a need.
* templateType sets which template shape to use for convolving the smoothed
*		DFT signal. 1 - rectangular, 2 - rectangular with negative side lobes. 
*		Recommended setting = 1 (only choice presently supported)
* useSymmetry: 0/1 - whether or not to use symmetry in post-processing the 
*		DFT result. Recommended setting = 0 (only choice presently supported)
* thresholdRatio, a float, is the limit for comparing *weight 
* 	with the dft readings away from the detected fringe packet. If this calculated
*   ratio > thresholdRatio, the returned value will be 1, indicating
* 	that a packet was found. Recommended setting = 2.0
* weight is a pointer to the returned confidence measurement. Higher value means 
*		higher confidence that the fringe was found. Designed to be similar to the
*		weight calculation designed by Ettore Pedretti.
* centerPhase is a pointer to a float containing the phase angle of the identified
*		center of the fringe packet. -pi corresponds to the very first data point in 
*		the scan, and +pi corresponds to the last one.
*
* These recommended values are good starting points, and should work with a wide
*	variety of scan parameters, but setting them more carefully will produce better
*	results. For example, if the scan is 128 points vs 256 points, then nFringe
*   should be set accordingly. Also, some tuning of thresholdRatio may be required
*	depending on the seeing conditions.
*
* There are some settings in trackerdefines.h that can turn on printing or logMsg'ing
*   if desired for debugging or analysis of the algorithm's performance.
*
****************************************************/
/* uncomment the extern "C" stuff if including this function in a .cpp file */
int ptracker_ed(double *y_raw_double, unsigned long nl, unsigned long nh, 
				int *nDFT, int nPeriodsInDFTWindow, int nDFTSmooth, int nFringe, int templateType, 
				int useSymmetry, float thresholdRatio, float *weight, float *centerPhase);

